/*
 * Copyright (C) 2014 greilich
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package fntd;

import ij.IJ;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

/**
 *
 * @author greilich
 */
public class About {
    
    /**
     * Reads version and build time information from
     * "version.txt" (included in jar and filled by maven
     * during build) and returns corresponding message
     * @return  String containing version and build date
     */
    public static String getMessageTitle(){
        return("DKFZ FNTD ImageJ Plugin");
    }

    public static String getBuildInfo(){
        String version      =  "n/a";
        String timestamp    =  "n/a";
        
        try {
            InputStream in = About.class.getResourceAsStream("/version.txt"); 
            BufferedReader reader = new BufferedReader(new InputStreamReader(in));          
            String aLine;
            String[] parts;
            
            while ((aLine = reader.readLine()) != null) {
                if (aLine.startsWith("version")) {
                    parts = aLine.split("=");
                    version = parts[1];
                }
                if (aLine.startsWith("build.date")) {
                    parts = aLine.split("=");
                    timestamp = parts[1];
                }
            }
        } catch (FileNotFoundException ex) {
            IJ.showMessage("Could not find version.txt");
        } catch (IOException ex) {
            IJ.showMessage("Error reading version.txt");
        }
        return("Version: "+version+" ("+timestamp+")");
    }
    
    public static String getMessageBody(){
        return( getBuildInfo() + "\n"+
                "This collection is intended for use with FNTD images.\n"+
                "Maintainer: Steffen Greilich <s.greilich@dkfz.de>\n"+
                "URL: www.dkfz.de/e0408");
    }
}
