/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package fntd;

import fntd.access.FNTDadjustmentListener;
import fntd.access.FNTDkeyListener;
import fntd.access.FNTDmouseListener;
import fntd.access.FNTDmouseWheelListener;
import fntd.tracking.FNTDtracks;
import fntd.tracking.FNTDtracks.Trajectory;
import ij.ImagePlus;
import ij.ImageStack;
import ij.gui.ImageCanvas;
import ij.gui.ImageWindow;
import ij.gui.Line;
import ij.gui.OvalRoi;
import ij.gui.Overlay;
import ij.gui.Roi;
import ij.gui.ScrollbarWithLabel;
import ij.plugin.HyperStackConverter;
import java.awt.Color;
import java.awt.Component;
import java.util.Vector;
import mosaic.core.detection.Particle;

/**
 *
 * @author greilich
 */
public class FNTDimagePlus extends ImagePlus {

    FNTDtracks ft = null;
    int trackOverlayRadius = 8;
    public Overlay[][][] overlays;

    /**
     *
     * @param orgImp Original ImagePlus object
     * @param orgFt Original FNTDTracks object
     * @throws FNTDexception
     */
    public FNTDimagePlus(ImagePlus orgImp, FNTDtracks orgFt) throws FNTDexception {

        super(orgImp.getTitle(), orgImp.getImageStack());
        if ((orgImp.getNChannels() > 1) || (orgImp.getNFrames() > 1)) {
            HyperStackConverter.toHyperStack(this, orgImp.getNChannels(),
                    orgImp.getNSlices(), orgImp.getNFrames(),
                    "xyztc", "grayscale");
        }
        if (orgFt != null)
            ft = orgFt;

        this.overlays = new Overlay[getNChannels()][getNFrames()][getNSlices()];

        if (ft == null)
            return;

        for (int i = 0; i < getNChannels(); i++) {
            for (int j = 0; j < getNFrames(); j++) {
                for (int k = 0; k < getNSlices(); k++) {
                    overlays[i][j][k] = new Overlay();
                    fillOverlayWithTrackSpots(k, j, i);
                }

                Vector<Trajectory> trajectories = ft.toTrajectories(i, j);

                if ((overlays != null) && (trajectories.size() > 0)) {
                    for (int m = 0; m < trajectories.size(); m++) {
                        Color trajColor = new Color((float) (0.25 + 0.75 * Math.random()),
                                (float) (0.25 + 0.75 * Math.random()),
                                (float) (0.25 + 0.75 * Math.random()));

                        Trajectory t = trajectories.elementAt(m);
                        if (t.existing_particles == null)
                            continue;
                        
                        for (int n = 0; n < t.existing_particles.size() - 1; n++) {
                            Particle p1, p2;
                            p1 = t.existing_particles.elementAt(n);
                            p2 = t.existing_particles.elementAt(n + 1);
                            Roi curRoi = new Line(p1.iX, p1.iY, p2.iX, p2.iY);
                            curRoi.setStrokeWidth(2);
                            curRoi.setStrokeColor(trajColor);

                            for (Overlay overlay : overlays[i][j])
                                if (overlay != null)
                                    overlay.add(curRoi);
                        }
                    }
                }
            }
        }
    }
    
    public FNTDimagePlus(ImagePlus orgImp) throws FNTDexception {
        super(orgImp.getTitle(), orgImp.getImageStack());
    }

    /**
     *
     */
    public void setOverlay() {
        int iSlice = getSlice() - 1;
        int iFrame = getFrame() - 1;
        int iChannel = getChannel() - 1;
        if (overlays != null && overlays[iChannel][iFrame][iSlice] != null)
            setOverlay(overlays[iChannel][iFrame][iSlice]);
    }

    /**
     *
     */
    @Override
    public void show() {
        super.show();

        ImageWindow iw = getWindow();
        iw.removeKeyListener(null);
        FNTDkeyListener kl = new FNTDkeyListener();
        iw.addKeyListener(kl);

        Component cc;
        for (Component component : iw.getComponents()) {
            cc = component;
            if (cc instanceof ImageCanvas) {
                ImageCanvas ic = (ImageCanvas) component;
                ic.removeMouseListener(null);
                ic.removeMouseMotionListener(null);
                ic.removeMouseWheelListener(null);
                FNTDmouseListener ml = new FNTDmouseListener(ic);
                ic.addMouseListener(ml);
                FNTDmouseWheelListener mwl = new FNTDmouseWheelListener(this);
                ic.addMouseWheelListener(mwl);
            }
            if (cc instanceof ScrollbarWithLabel) {
                ScrollbarWithLabel sl = (ScrollbarWithLabel) component;
                FNTDadjustmentListener al = new FNTDadjustmentListener(this);
                sl.addAdjustmentListener(al);
            }
        }

        setOverlay();
    }

    /**
     *
     * @return
     */
    public ImagePlus flattenImage() {
        ImagePlus tmpImp;
        ImageStack is = new ImageStack(this.getWidth(), this.getHeight());
        
        for (int i = 0; i < this.getNChannels(); i++) {
            for (int j = 0; j < this.getNFrames(); j++) {
                for (int k = 0; k < this.getNSlices(); k++) {
                    this.setPosition(i+1, k+1, j+1);
                    tmpImp = new ImagePlus("", this.getProcessor()); 
                    tmpImp.setOverlay(this.getOverlay(k, j, i));
                    tmpImp = tmpImp.flatten();
                    is.addSlice(tmpImp.getProcessor());
                }
            }
        }

        ImagePlus resImp  = new ImagePlus(this.getTitle() + "_flattened", is);
        return(resImp);
    }

    
    /**
     * @param iSlice Slice number
     * @param jFrame Frame number
     * @param kChannel Channel number
     */
    public void updateOverlay(int iSlice, int jFrame, int kChannel) {
        if ((ft != null) && (overlays != null)
                && (kChannel < overlays.length) && (jFrame < overlays[0].length) && (iSlice < overlays[0][0].length)) {
            overlays[kChannel][jFrame][iSlice] = new Overlay();
            fillOverlayWithTrackSpots(iSlice, jFrame, kChannel);
        }
    }

    /**
     *
     * @param iSlice Slice number
     * @param jFrame Frame number
     * @param kChannel Channel number
     */
    private void fillOverlayWithTrackSpots(int iSlice, int jFrame, int kChannel) {

        double[] x = ft.getX(iSlice, jFrame, kChannel);
        double[] y = ft.getY(iSlice, jFrame, kChannel);
        int trackNo[] = ft.getTrackNo(iSlice, jFrame, kChannel);

        if (x != null) {
            for (int j = 0; j < x.length; j++) {
                Roi curRoi;
                if (trackNo[j] == 0) {
                    curRoi = new OvalRoi(x[j] - trackOverlayRadius / 2,
                            y[j] - trackOverlayRadius / 2,
                            trackOverlayRadius,
                            trackOverlayRadius);
                    curRoi.setStrokeWidth(1.5);
                    curRoi.setStrokeColor(Color.red);
                } else {
                    curRoi = new Roi(x[j] - trackOverlayRadius / 2 - 1,
                            y[j] - trackOverlayRadius / 2 - 1,
                            trackOverlayRadius + 2,
                            trackOverlayRadius + 2);
                    curRoi.setStrokeWidth(1.5);
                    curRoi.setStrokeColor(Color.pink);

                }
                overlays[kChannel][jFrame][iSlice].add(curRoi);
            }
        }
    }

    /**
     *
     * @param iSlice Slice number
     * @param jFrame Frame number
     * @param kChannel Channel number
     * @return
     */
    public Overlay getOverlay(int iSlice, int jFrame, int kChannel) {
        if ((overlays != null) && (kChannel < overlays.length) && 
                (jFrame < overlays[0].length) && (iSlice < overlays[0][0].length))
            return (overlays[kChannel][jFrame][iSlice]);
        else
            return (new Overlay());
    }

}
