/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package fntd.access;

import fntd.Commons;
import fntd.FNTDimagePlus;
import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.plugin.Duplicator;
import ij.plugin.HyperStackConverter;
import ij.plugin.ImageCalculator;
import ij.process.ByteProcessor;
import ij.process.ColorProcessor;
import ij.process.FloatProcessor;
import ij.process.ImageProcessor;
import ij.process.ShortProcessor;
import java.util.Arrays;

/**
 * Returns pixel values
 *
 * @author greilich
 */
public class AccessImage {

    public static ImagePlus scale(ImagePlus imp, double factor) {

        int newWidth = (int) Math.round(imp.getWidth() * factor);
        int newHeight = (int) Math.round(imp.getHeight() * factor);
        ImageStack is = new ImageStack(newWidth, newHeight);
        for (int i = 0; i < imp.getNSlices(); i++) {
            imp.setSlice(i);
            ImageProcessor ic = imp.getProcessor().resize(newWidth);
            is.addSlice(ic);
        }
        ImagePlus resImp = new ImagePlus(imp.getTitle() + "_scaled", is);
        return (resImp);
    }

    public static int[] getSizes(FNTDimagePlus fimp) {
        return (getSizes((ImagePlus) fimp));
    }

    public static int[] getSizes(ImagePlus imp) {
        int[] size = new int[5];
        size[0] = imp.getWidth();
        size[1] = imp.getHeight();
        size[2] = imp.getNSlices();
        size[3] = imp.getNFrames();
        size[4] = imp.getNChannels();
        return (size);
    }

    public static double[] getPixels(ImagePlus imp,
            int[] slices,
            int[] frames,
            int[] channels) {

        ImagePlus subImp = AccessImage.getSubHyperStack(imp, slices, frames, channels);
        ImageStack imageStack = subImp.getImageStack();

        double[] pixels = new double[subImp.getWidth() * subImp.getHeight() * imageStack.getSize()];

        for (int i = 0; i < imageStack.getSize(); i++) {
            switch (subImp.getType()) {
                case ImagePlus.GRAY8:
                    ByteProcessor byteProcessor = (ByteProcessor) imageStack.getProcessor(i + 1);
                    for (int iX = 0; iX < subImp.getWidth(); iX++) {
                        for (int iY = 0; iY < subImp.getHeight(); iY++) {
                            pixels[i * subImp.getWidth() * subImp.getHeight() + iY * subImp.getWidth() + iX] = (double) byteProcessor.getPixelValue(iX, iY);
                        }
                    }
                    break;
                case ImagePlus.GRAY16:
                    ShortProcessor shortProcessor = (ShortProcessor) imageStack.getProcessor(i + 1);
                    for (int iX = 0; iX < subImp.getWidth(); iX++) {
                        for (int iY = 0; iY < subImp.getHeight(); iY++) {
                            pixels[i * subImp.getWidth() * subImp.getHeight() + iY * subImp.getWidth() + iX] = (double) shortProcessor.getPixelValue(iX, iY);
                        }
                    }
                    break;
                case ImagePlus.GRAY32:
                    FloatProcessor floatProcessor = (FloatProcessor) imageStack.getProcessor(i + 1);
                    for (int iX = 0; iX < subImp.getWidth(); iX++) {
                        for (int iY = 0; iY < subImp.getHeight(); iY++) {
                            pixels[i * subImp.getWidth() * subImp.getHeight() + iY * subImp.getWidth() + iX] = (double) floatProcessor.getPixelValue(iX, iY);
                        }
                    }
                    break;
                case ImagePlus.COLOR_RGB:
                    ColorProcessor colorProcessor = (ColorProcessor) imageStack.getProcessor(i + 1);
                    int nX = subImp.getWidth();
                    int nY = subImp.getHeight();
                    byte[] red = new byte[nX * nY];
                    byte[] green = new byte[nX * nY];
                    byte[] blue = new byte[nX * nY];
                    colorProcessor.getRGB(red, green, blue);
                    int r,
                     g,
                     b;

                    for (int j = 0; j < nX * nY; j++) {
                        r = red[j] << 16;
                        g = green[j] << 8;
                        b = blue[j];
                        pixels[i * nX * nY + j] = (double) (r + g + b);
                    }
                    break;
            }
        }

        return (pixels);
    }

    public static ImagePlus add(ImagePlus a, ImagePlus b) {
        return (new ImageCalculator().run("Add create 32-bit stack", a, b));
    }

    public static ImagePlus add(ImagePlus a, double b) {
        Duplicator dp = new Duplicator();
        ImagePlus c = dp.run(a);
        IJ.run(c, "32-bit", "");
        IJ.run(c, "Add...", "value=" + Double.toString(b));
        return (c);
    }

    public static ImagePlus subtract(ImagePlus a, ImagePlus b) {

        boolean silent = (IJ.getInstance() == null);

        int nX = a.getWidth();
        int nY = a.getHeight();
        int nSlices = a.getNSlices();
        int nFrames = a.getNFrames();
        int nChannels = a.getNChannels();

        if ((nX != b.getWidth())
                || (nY != b.getHeight())
                || (nSlices != b.getNSlices())
                || (nFrames != b.getNFrames())
                || (nChannels != b.getNChannels())) {
            if (silent) {
                System.out.println("Subtract: image dimensions do not match, return nothing.");
            } else {
                IJ.showMessage("Subtract: image dimensions do not match, return nothing.");
            }
        }

        ImageStack resIms = new ImageStack(nX, nY);
        FloatProcessor fp;
        ImageProcessor ip1, ip2;

        int total = nSlices * nFrames * nChannels;
        double prog = 0;

        if (silent) {
            System.out.println("SUBSTRACTING BACKGROUND");
        } else {
            IJ.showStatus("SUBSTRACTING BACKGROUND");
        }

        for (int i = 0; i < nChannels; i++) {
            for (int j = 0; j < nFrames; j++) {

                for (int k = 0; k < nSlices; k++) {
                    if (silent) {
                        FNTDprogress.updateProgress(prog / total);
                    } else {
                        IJ.showProgress(prog / total);
                    }

                    prog++;

                    a.setPosition(i + 1, k + 1, j + 1);
                    b.setPosition(i + 1, k + 1, j + 1);
                    ip1 = a.getProcessor();
                    ip2 = b.getProcessor();
                    fp = new FloatProcessor(nX, nY);

                    for (int m = 0; m < nX; m++) {
                        for (int n = 0; n < nY; n++) {
                            fp.putPixelValue(m, n, ip1.getPixelValue(m, n) - ip2.getPixelValue(m, n));
                        }
                    }
                    resIms.addSlice(fp);
                }
            }
        }
        FNTDprogress.updateProgress(1);
        System.out.println("");

        ImagePlus resImp = new ImagePlus("Result of" + a.getTitle() + "-" + b.getTitle(), resIms);
        if (resIms.getSize() > 1) {
            resImp = HyperStackConverter.toHyperStack(resImp,
                    nChannels,
                    nSlices,
                    nFrames,
                    "xyztc",
                    "grayscale");
        }
        return (resImp);
    }

    public static ImagePlus subtract(ImagePlus a, double b) {
        Duplicator dp = new Duplicator();
        ImagePlus c = dp.run(a);
        IJ.run(c, "32-bit", "");
        IJ.run(c, "Subtract...", "value=" + Double.toString(b));
        return (c);
    }

    public static ImagePlus multiply(ImagePlus a, ImagePlus b) {
        return (new ImageCalculator().run("Multiply create 32-bit stack", a, b));
    }

    public static ImagePlus multiply(ImagePlus a, double b) {
        Duplicator dp = new Duplicator();
        ImagePlus c = dp.run(a);
        IJ.run(c, "32-bit", "");
        IJ.run(c, "Multiply...", "value=" + Double.toString(b));
        return (c);
    }

    public static ImagePlus multiply(ImagePlus a, double[] b) {

        Duplicator dp = new Duplicator();
        ImagePlus c = dp.run(a);
        IJ.run(c, "32-bit", "");

        ImageStack ims = c.getStack();
        ImagePlus tmpImp;
        for (int i = 0; i < c.getNSlices(); i++) {
            tmpImp = new ImagePlus("tmp", ims.getProcessor(i + 1));
            IJ.run(tmpImp, "Multiply...", "value=" + Double.toString(b[i]));
            ims.setProcessor(tmpImp.getProcessor(), i + 1);
        }

        c = new ImagePlus(a.getTitle() + "_mult", ims);
        return (c);
    }

    /**
     * Divides all pixel values in an image (hyperstack) by a single image
     *
     * @param a Image
     * @param b Single number
     * @return
     */
    public static ImagePlus divide(ImagePlus a, ImagePlus b) {
        return (new ImageCalculator().run("Divide create 32-bit stack", a, b));
    }

    /**
     * Divides all pixel values in an image (hyperstack) by a single number
     *
     * @param a Image
     * @param b Single number
     * @return
     */
    public static ImagePlus divide(ImagePlus a, double b) {
        Duplicator dp = new Duplicator();
        ImagePlus c = dp.run(a);
        IJ.run(c, "32-bit", "");
        IJ.run(c, "Divide...", "value=" + Double.toString(b));
        return (c);
    }

    /**
     * Divide pixel values of a selected image in a hyperstack by a
     * corresponding single number
     *
     * @param a ImagePlus object
     * @param b Factor
     * @param slices Selected slices
     * @param frames Selected frames
     * @param channels Selected channels
     * @return
     */
    public static ImagePlus divide(ImagePlus a, double b, int slices[], int frames[], int channels[]) {
        int sizes[] = a.getDimensions();

        // Zero index --> keep all indices
        if (slices.length == 1 && slices[0] == 0) {
            slices = Commons.getSequence(1, sizes[3]);
        }
        if (frames.length == 1 && frames[0] == 0) {
            frames = Commons.getSequence(1, sizes[4]);
        }
        if (channels.length == 1 && channels[0] == 0) {
            channels = Commons.getSequence(1, sizes[2]);
        }

        // Check input arguments
        if ((Commons.getMin(slices) < 1) || (Commons.getMax(slices) > sizes[3])) {
            System.out.println("Subscripts for slices out of bounds.");
            return a;
        }
        if ((Commons.getMin(frames) < 1) || (Commons.getMax(frames) > sizes[4])) {
            System.out.println("Subscripts for frames out of bounds.");
            return a;
        }
        if ((Commons.getMin(channels) < 1) || (Commons.getMax(channels) > sizes[2])) {
            System.out.println("Subscripts for channels out of bounds.");
            return a;
        }

        Duplicator dp = new Duplicator();
        ImagePlus c = dp.run(a);

        for (int i = 0; i < slices.length; i++) {
            for (int j = 0; j < frames.length; j++) {
                for (int k = 0; k < channels.length; k++) {
                    c.setPosition(channels[k], slices[i], frames[j]);
                    ImageProcessor ip = c.getProcessor();
                    if (b != 0.0) {
                        ip.multiply(1 / b);
                    } else {
                        ip = new FloatProcessor(c.getWidth(), c.getHeight());
                        c.setProcessor(ip);
                    }

                }
            }
        }
        return (c);
    }

    public static ImagePlus invert(ImagePlus a) {
        Duplicator dp = new Duplicator();
        ImagePlus c = dp.run(a);
        IJ.run(c, "Invert", "");
        return (c);
    }

    public static boolean isBinary(ImagePlus a) {
        return (a.getType() == ImagePlus.GRAY8);
    }

    public static ImagePlus asImagePlus(String title,
            double[] data,
            int width,
            int height,
            int nSlices,
            int nFrames,
            int nChannels) {
        ImageStack imageStack = new ImageStack(width, height);
        FloatProcessor ip;
        
        for (int i = 0; i < nSlices * nFrames * nChannels; i++) {
            ip = new FloatProcessor(width, height);
            for (int iX = 0; iX < width; iX++)
                for (int iY = 0; iY < height; iY++)
                    ip.putPixelValue(iX, iY, data[i * height * width + iY * width + iX]);
            
            imageStack.addSlice(ip);
        }

        ImagePlus resImp = new ImagePlus(title, imageStack);
        resImp.setOpenAsHyperStack(true);
        resImp.setDimensions(nChannels, nSlices, nFrames);

        return resImp;
    }

    public static ImagePlus getSubHyperStack(FNTDimagePlus fimp,
            int[] slices,
            int[] frames,
            int[] channels) {
        return (getSubHyperStack((ImagePlus) fimp, slices, frames, channels));
    }

    /**
     * Returns subset of given hyperstack
     *
     * @param imp
     * @param slices
     * @param frames
     * @param channels
     * @return
     */
    public static ImagePlus getSubHyperStack(ImagePlus imp,
            int[] slices,
            int[] frames,
            int[] channels) {

        int sizes[] = imp.getDimensions();

        // Zero index --> keep all indices
        if (slices.length == 1 && slices[0] == 0) {
            slices = Commons.getSequence(1, sizes[3]);
        }
        if (frames.length == 1 && frames[0] == 0) {
            frames = Commons.getSequence(1, sizes[4]);
        }
        if (channels.length == 1 && channels[0] == 0) {
            channels = Commons.getSequence(1, sizes[2]);
        }

        // Check input arguments
        if ((Commons.getMin(slices) < 1) || (Commons.getMax(slices) > sizes[3])) {
            System.out.println("Subscripts for slices out of bounds.");
            return imp;
        }
        if ((Commons.getMin(frames) < 1) || (Commons.getMax(frames) > sizes[4])) {
            System.out.println("Subscripts for frames out of bounds.");
            return imp;
        }
        if ((Commons.getMin(channels) < 1) || (Commons.getMax(channels) > sizes[2])) {
            System.out.println("Subscripts for channels out of bounds.");
            return imp;
        }

        // Convert indices into stack indices
        ImageStack ims = imp.getImageStack();
        boolean keep[] = new boolean[ims.getSize()];
        Arrays.fill(keep, false);

        int slicesToKeep = 0;
        for (int i = 0; i < slices.length; i++) {
            for (int j = 0; j < frames.length; j++) {
                for (int k = 0; k < channels.length; k++) {
                    int test = imp.getStackIndex(channels[k], slices[i], frames[j]);
                    keep[imp.getStackIndex(channels[k], slices[i], frames[j]) - 1] = true;
                    slicesToKeep++;
                }
            }
        }

        ImageStack resIms = new ImageStack(imp.getWidth(), imp.getHeight());

        for (int i = 0; i < ims.getSize(); i++) {
            if (keep[i]) {
                resIms.addSlice(ims.getProcessor(i + 1));
            }
        }
        ImagePlus resImp = new ImagePlus("SubHyperStack of " + imp.getTitle(), resIms);
        resImp.setOpenAsHyperStack(true);
        resImp.setDimensions(channels.length, slices.length, frames.length);

        return resImp;
    }

}
