package fntd.background;

import fntd.background.mosaic.BackgroundSubtractor_FNTDAdapted;
import fntd.access.FNTDprogress;
import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.plugin.Duplicator;
import ij.plugin.HyperStackConverter;

public class Background extends BackgroundSubtractor_FNTDAdapted {

    public ImagePlus getBackground(ImagePlus imp, int slidingWindowSize) {

        boolean silent = (IJ.getInstance() == null);
                
        if (silent)
            System.out.println("BACKGROUND CALCULATION");
        else
            IJ.showStatus("BACKGROUND CALCULATION");
        
        Duplicator dp = new Duplicator();
        ImageStack resIms = new ImageStack(imp.getWidth(), imp.getHeight());

        int nSlices   = imp.getNSlices();
        int nFrames   = imp.getNFrames();
        int nChannels = imp.getNChannels();
        
        int total     = nSlices*nFrames*nChannels;
        double prog   = 0;

        for (int i = 0; i < nChannels; i++) {
            for (int j = 0; j < nFrames; j++) {
                
                for (int k = 0; k < nSlices; k++) {
                    if(silent)
                        FNTDprogress.updateProgress(prog/total);
                    else
                        IJ.showProgress((double)(k+1)/(double)nSlices);
                    prog++;
                    
                    ImagePlus tmpImp = dp.run(imp, i+1, i+1, k+1, k+1, j+1, j+1);

                    // Initialize background stack and resulting stack
                    mBackgroundImageStack = new ImageStack(imp.getWidth(), imp.getHeight());
                    // Subtract background, that will fill mBackgroudImageStack
                    SubtractBackground(tmpImp, slidingWindowSize);

                    resIms.addSlice(mBackgroundImageStack.getProcessor(1));
                }
            }
        }
        FNTDprogress.updateProgress(1);
        System.out.println("");

        ImagePlus resImp = new ImagePlus("Background of" + imp.getTitle(), resIms);
        if(resIms.getSize() > 1){
            resImp = HyperStackConverter.toHyperStack(resImp,
                    imp.getNChannels(),
                    imp.getNSlices(),
                    imp.getNFrames(),
                    "xyztc",
                    "grayscale");
        }
        
        return (resImp);
    }
}
