package fntd.lsmImport;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.process.ImageProcessor;
import ij.gui.NewImage;
import ij.io.*;
import ij.plugin.Converter;
import org.imagearchive.lsm.reader.Reader;

/**
 * First version of LSM image reader, retrieving metadata
 * At the moment not used, instead the LSM_Reader plugin
 * is directly employed and metadata are skipped.
 * @author Roman Martel
 */
public class FNTD_LSMFileReader {
	
	// Methode, die ein LSM-File laedt und dessen Pixel als float-Array ausgibt
	
	public static short[] getFileData(String Path) {
		// Ueberpruefen ob file ein .lsm ist
		if (Path.endsWith(".lsm")) {

			// Loads the image with the reader-method of the LSM_Reader
			ImagePlus imp = new Reader().open(Path,true);
			// Objects and variables that we need
			ImageProcessor ip = imp.getProcessor();
			ImageStack ims = imp.getImageStack();
			int nslice = ims.getSize();						// Number of slices in the stack
			int npixel = ip.getWidth()* ip.getHeight() ;	// Number of pixel in one image
			
			// Create Array which contains the pixelinformation
			short[] stackpixel = new short[ nslice * npixel ];
			// Array that contains the pixel of one image
			short[] imppixel = new short[ npixel ];
			// Array that contains the indices of the pixel
//			short[]  pixelindex = new short[ npixel ];
			
			// Fill the pixelinformation array
			for ( int i=1; i<= nslice; i++) {
				// First fill with the Pixeldata
				imppixel = (short[]) ims.getPixels(i);
				System.arraycopy( (short[]) imppixel, 0 , stackpixel, (i-1)*npixel , npixel );
				
				// Next fill with the number that indicates to which image the pixel belong
/*				for (int j = 0; j < npixel; j++){
					stackpixel[0][j + (i-1)*npixel] = (short) i;
				}
*/
				
			}

//	old version returns just pixel of the first image
//			short[] pixel    = (short[]) ip.getPixels();	// Array mit Pixeln laden	
			
			return ( stackpixel);
		}
		else {
			System.out.println("getFileData: File is not a .lsm-File! Returnvalue is 0.\n");
			short[] zero = new short[0];
			return (zero);
		}
	}
	
	
	// we overload getFileDate in order that the user can choose whether he reads the whole stack or
	// just a part of it betwenn begin and end
	
	public static short[] getFileData(String Path, int begin, int end) {
		
		// Ueberpruefen ob file ein .lsm ist
		if (Path.endsWith(".lsm")) {

			// Loads the image with the reader-method of the LSM_Reader
			ImagePlus imp = new Reader().open(Path,true);
			// Objects and variables that we need
			ImageProcessor ip = imp.getProcessor();
			ImageStack ims = imp.getImageStack();
			int nslice = ims.getSize();						// Number of slices in the stack
			if (end > nslice) {
				System.out.println("The stack contains just " + nslice + " slices!");
				short[] zero = new short[0];
				return (zero);
			}
			else if (begin > end) {
				System.out.println("The first int value must be bigger than the second. (int begin, int end)");
				short[] zero = new short[0];
				return (zero);
			}
			else {
				int npixel = ip.getWidth()* ip.getHeight() ;	// Number of pixel in one image
			
			
				// Create Array which contains the pixelinformation
				short[] stackpixel = new short[ ((end - begin)+1) * npixel ];	// +1 because in case of whole stack: 30 - 1 = 29
				// Array that contains the pixel of one image
				short[] imppixel = new short[ npixel ];
				// Array that contains the indices of the pixel
	//			short[]  pixelindex = new short[ npixel ];
				
				// Fill the pixelinformation array
				for ( int i=begin; i<= end; i++) {
					// First fill with the Pixeldata
					imppixel = (short[]) ims.getPixels(i);
					System.arraycopy( (short[]) imppixel, 0 , stackpixel, (i-begin)*npixel , npixel );
																		//((i-begin+1)-1)
					// Next fill with the number that indicates to which image the pixel belong
	/*				for (int j = 0; j < npixel; j++){
						stackpixel[0][j + (i-1)*npixel] = (short) i;
					}
	*/
					
				}
	
	//	old version returns just pixel of the first image
	//			short[] pixel    = (short[]) ip.getPixels();	// Array mit Pixeln laden	
				
				return ( stackpixel);
			}
		}
		else {
			System.out.println("getFileData: File is not a .lsm-File! Returnvalue is 0.\n");
			short[] zero = new short[0];
			return (zero);
		}
	}
	
	
	
	
	/* Hier sind die Methoden um die Byte-Reihenfolge zu 'swappen'. Denn .lsm Files sind in little-endian-
	Format gespeichert. d.h. die Zahlen mit dem kleinsten Wert kommen zuerst (Bsp. 07 entspricht 70).
	Java arbeitet aber im big-endian-Format. Deshalb muessen wir tauschen.
	Bsp.bei short  11101111 00100010 -> 00100010 11101111 */

		private static short swap(short x) {
			return (short) ((x << 8) | ((x >> 8) & 0xff));
		}

		private static char swap(char x) {
			return (char) ((x << 8) | ((x >> 8) & 0xff));
		}

		private static int swap(int x) {
			return (int) ((swap((short) x) << 16) | (swap((short) (x >> 16)) & 0xffff));
		}

		private static long swap(long x) {
			return (long) (((long) swap((int) (x)) << 32) | ((long) swap((int) (x >> 32)) & 0xffffffffL));
		}

		private static float swap(float x) {
			return Float.intBitsToFloat(swap(Float.floatToIntBits(x)));
		}

		private static double swap(double x) {
			return Double.longBitsToDouble(swap(Double.doubleToLongBits(x)));
		}




	// Methode metaReader: Soll Metadaten (dwell time, repetions, laser power) ausgeben
	// Um das zu verstehen muss man ins .lsm-Fileinfo-Heft schauen

	// *************************************************************
		public static String[] getMetaData(String Path) {
			
		
			// Ueberpruefen ob file ein .lsm ist
			if (Path.endsWith(".lsm")) {
				
				String[] datenarray = new String[5];	// 0 : dwell_time, 1 : power, 2 : repetitions , 3 : pinhole_diameter 4 : Objektiv
				// Datenarray initialisieren
				for (int i=0; i< datenarray.length; i++) {
					datenarray[i] = "kein Eintrag";
				}
				int counter = 0;		// Zaehlt die detektierten Metadaten. Haben wir alle sorgt er fuer den Abbruch der for-Schleife
	//			Daten dat = new Daten();
				RandomAccessFile file;
				//String fileName = imp.getTitle();
				//String directory = IJ.getDirectory(fileName);
				ImagePlus Bild = new Reader().open(Path,true);
				FileInfo info = Bild.getOriginalFileInfo();
				String directory = info.directory;
				String fileName = info.fileName;
	//			IJ.log( fileName + " " + directory + " ");
				
				try {
					file = new RandomAccessFile(new File(directory, fileName), "r");
				//	RandomAccessStream stream = new RandomAccessStream(file);
	
			//********************* Gehe auf Position wo Einstellungsdaten beginnen
					int position = 0;
					boolean running = true;
					for (int i=position; running; i+=1) {
						file.seek(i);
						if (file.readInt() == 0x4C490004) {		//Nach MagicNumber suchen
							position = i;
							running = false;
						}
						if (i > 1000000) {
							IJ.showMessage(" Keine MagicNumber vorhanden! File ist möglicherweise"
							+ " kein .lsm ! ");
							running = false;
							break;
							
						}
	
					}
					position += 140 ;						//zu u32 OffsetScanInformation gehen
					file.seek(position);
					position = swap(file.readInt());		//zum Offset in diesem gehen
	
	
	
					int finalPosition = position;
					for (int i=position; counter<5; i+=1) {			// file durchgehen	
						file.seek(i);
						
			//********************* dwell time
						switch(file.readInt()) {
							case (0x19000040):	{	// Bei den Vergleichen sind die Bytes schon geswappt!
								i+=4;
								file.seek(i);
									if(file.readInt() == 0x05000000) {
										i+=4;
										file.seek(i);
										if(file.readInt() == 0x08000000) {
											i+=4;
											finalPosition = i;
											file.seek(finalPosition);
											datenarray[0] = String.valueOf( swap(file.readDouble()) );  // dwell_time zuordnen
											counter += 1; 		// Counter hochsetzen fuer Abbruchbedingung
										}
									}
								
								break;		// case verlassen!
							}			
	
			//********************** Repetition
	
							case(0x05000040):	{
								i+=4;
								file.seek(i);
								if(file.readInt() == 0x04000000) {
									i+=4;
									file.seek(i);
									if(file.readInt() == 0x04000000) {
										i+=4;
										finalPosition = i;
										file.seek(finalPosition);
										datenarray[2] = String.valueOf( swap(file.readInt()) ); // repetitions zuordnen
										counter += 1;
										break;
									}
								}
								break; // case verlassen
							}
					
	
	
			//************************ Power
	
							case(0x02000090):	{
								i+=4;
								file.seek(i);
								if(file.readInt() == 0x05000000) {
	
									i+=4;
									file.seek(i);
	
									if(file.readInt() == 0x08000000) {
										i+=4;
										finalPosition = i;
										file.seek(finalPosition);
										datenarray[1] = String.valueOf( swap(file.readDouble()) ); // power zuordnen
										counter += 1;
										break;
									}
								}
								break;
							}
					
					
					
			//************************ Pinhole_Diameter
					
							case(0x09000070):	{
								i+=4;
								file.seek(i);
								if(file.readInt() == 0x05000000) {
									i+=4;
									file.seek(i);
									if(file.readInt() == 0x08000000) {
										i+=4;
										finalPosition = i;
										file.seek(finalPosition);
										datenarray[3] = String.valueOf( swap(file.readDouble()) ); // pinhole_diameter zuordnen
										counter += 1;
										break;
									}
								}
								break;
							}						
					
					
					
			//************************ Objektiv
					
							
							case(0x04000010):	{
								i+=4;
								file.seek(i);
								if(file.readInt() == 0x02000000) {
									i+=4;
									file.seek(i);	// an dieser Stelle steht die Laenge des Strings
									int length = swap(file.readInt());
									if (length > 3 && length < 60) {	//zweiter Schutz falls die obere Kombi doppelt ist
										
										// Bytearray mit Laenge des Strings erzeugen
										byte[] b = new byte[ length -1]; // -1 weil die angehaengte 0 bei der Laenge dabei ist
								
										// an Anfangsposition des Strings gehen (weil Info ueber Laenge des Strings 4 Byte lang):
										i+=4;	
										file.seek(i); 
										// Auslesen der Daten in das Bytearray
										file.read(b, 0, b.length );
										// Bytearray in char umwandeln und in Ausgabestring geben
										datenarray[4] = ""; // Initialisieren
										for (int j=0; j< b.length; j++){
											datenarray[4] += (char) b[j];
										}
										counter += 1;
										break;
									}
								
								}
								break;
							}
							
						}		// Switch zuende
						
						if (i>1000000) {
							IJ.showMessage("Metadaten nicht gefunden!");
							break;
						}
						
						
					} // Ende der for-Schleife zu Beginn
	
									
	
			//************************ Exeptions
	
				} catch (FileNotFoundException e) {
					IJ.error("File not found");
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				}
	
	
	//			Bild.close();				// schlie�t das Bild zum Schlu�
				
	
				return datenarray;
			}	// Ende der if-Schleife, die prueft ob file .lsm ist
			
			// Falls file kein .lsm ist
			else { 
				System.out.println("getMetaData: File is not a .lsm-File! Returnvalue is null.\n");
				String[] zero = new String[5];
				return zero;
			}
		}

	// **************************************** metaReader Ende


	// Diese Methode liefert die X und Y Koordinate eines LSM-Bildes zurueck	
		
		public static int[] getSize(String Path) {
			// Ueberpruefen ob file ein .lsm ist
			if (Path.endsWith(".lsm")) {
			
				// Variable fuer die Koordinaten
				int [] coord = new int[3];
				
				// Bild laden mit der Readermethode von LSM_Reader
				ImagePlus imp = new Reader().open(Path,true);
				ImageProcessor ip = imp.getProcessor();	
				ImageStack ims = imp.getImageStack();
				int nslice = ims.getSize();						// Number of slices in the stack
				
				coord[0] = ip.getWidth();	//X-Koordinate zuweisen
				coord[1] = ip.getHeight();	//Y-Koordinate zuweisen
				coord[2] = nslice;			//Z-Koordinate zuweisen
				
	//			imp.close();	// Bild schliessen weil lsm_reader dass momentan noch nicht macht
				
				return (coord);
			}
			// Falls file kein .lsm
			else {
				System.out.println("getXYCoord: File is not a .lsm-File! Returnvalue is 0.\n");
				int[] zero = new int[2];
				return zero;
			}
		}
	
	
	
	// Diese main-Funktion wird nicht gebraucht. Sie dient nur dazu den Code in der Entwicklungsphase zu testen.
		
//	public static void main(String[] args) {
//		// TODO Auto-generated method stub
//		String Filepath = "C:\\Users\\Roman\\Studium\\Projektpraktikum\\Daten\\Sg35041_c_spect.stack.lsm";
////		String Filepath = "C:\\Users\\Roman\\Studium\\Projektpraktikum\\Daten\\gk1001_a_range_long.lsm";
//		//gk1001_a_range_long_N.tif
//		//Daten metadat = new Daten();
//		
//		
//		short[] a= getFileData(Filepath, 7, 8);
////		short[] a = getFileData(Filepath);
////		String[] metadat = getMetaData(Filepath);
////		System.out.println("dwelltime: "+ metadat[0] + "\npower: " + metadat[1] + "\nrepetitions: "
////				+ metadat[2] + "\npinhole_diameter:" + metadat[3] + "\nObjektiv: "+ metadat[4]);
//	
////		if (a.length > 20) {
////		for ( int z = 0; z<30; z++) {
///*			for (int i =0; i < 10 ; i++) {
//				System.out.println( a[0][i + z*512*512] +" "+ a[1][i + z*512*512]);
//			}
//			*/
//		
////			System.out.println("");
//			for (int i =0; i < 1000; i++) {
//				System.out.print( a[a.length-1-i] + " ");
//			}
//			System.out.println("\nLength of Pixelstring is:" + a.length );
//
////		}
////		}
//		
//		int[] xycoord = getSize(Filepath);
//		System.out.println("X-Koordinate: "+ xycoord[0] + "\nY-Koordinate: " + xycoord[1] + " " + xycoord[2]);
//	}

}
