/*
 * Copyright (C) 2014 Steffen Greilich
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package fntd.plugins;

/**
 *
 * @author schudell
 */
import ij.*;
import ij.plugin.*;

/**
 * Split stack tile-wise into new stacks
 *
 * @author schudell, greilich
 */
public class SplitTiledStack_ implements PlugIn {

    @Override
    public void run(String arg) {

        ImagePlus imp = WindowManager.getCurrentImage();
        ImagePlus imp_sub = WindowManager.getCurrentImage();

        String abc = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
        String save_name = "";
        char idx = 'A';
        double n_x = IJ.getNumber("Hello. This macro will split your tiled stack into several separate stacks. How many divisions in X direction do you want to make", 2);
        double n_y = IJ.getNumber("How many divisions in Y direction do you want to make?", 2);
        String stackname = IJ.getString("SAVE AS: Enter the name of the stacks (*.tif):", "stack");
        String choice_slices = IJ.getString("SUBSTACK-MAKER: Which frames of the stack do you need? (e.g. 1-12, or 4,7,11)", "1-1");
        String dir = IJ.getDirectory("image");
        String title = imp.getTitle();

        int width = imp.getWidth();
        int height = imp.getHeight();
        double tileWidth = width / n_x;
        double tileHeight = height / n_y;
        int i = 0;
        double offsetX;
        double offsetY;
        for (int y = 0; y < n_y; y++) {
            offsetY = y * height / n_y;
            for (int x = 0; x < n_x; x++) {
                offsetX = x * width / n_x;
                IJ.selectWindow(title);
                IJ.makeRectangle((int) offsetX, (int) offsetY, (int) tileWidth, (int) tileHeight);
                IJ.run("Make Substack...", "slices=" + choice_slices);
                imp_sub = WindowManager.getCurrentImage();
                save_name = dir + stackname + "_" + abc.charAt(idx) + ".tif";
                IJ.saveAs("Tiff", save_name);
                IJ.log(save_name + ": Rectangle [" + offsetX + ", " + offsetY + ", " + tileWidth + ", " + tileHeight + "]\n");
                imp_sub.close();
                i++;
            }
            IJ.log("\nThe tiled stack " + title + " is splitted into " + i + " stacks. \n All stacks are saved (directory: " + dir + ").\n Have a good day.");
        }
    }

}
