/*
 * Copyright (C) 2014 Steffen Greilich
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package fntd.plugins;

/**
 *
 * @author greilich
 */
import fntd.access.AccessImage;
import fntd.background.Background;
import ij.*;
import ij.gui.GenericDialog;
import ij.plugin.*;

/**
 * Normalizes all images in a stack independently, i.e. the mean of each image
 * is 1.0
 *
 * @author greilich
 */
public class SubtractBackground_ implements PlugIn {

    @Override
    public void run(String arg) {

        // Get current image
        ImagePlus imp = IJ.getImage();

        /**
         * Get user input
         */
        int slidingWindowLength = 50;
        boolean showBackground = false;
        GenericDialog gd = new GenericDialog("Background subtraction");
        gd.addMessage("Applies MOSAIC background subtractor stackwise (but allows negative values).");
        gd.addNumericField("Sliding window legth", slidingWindowLength, (int)0);
        gd.addCheckbox("Show background image", showBackground);
        gd.showDialog();

        if (gd.wasCanceled()) return;

        Background bk    = new Background();
        ImagePlus bkgImp = bk.getBackground(imp, slidingWindowLength);
        bkgImp.show();
        ImagePlus resImp = AccessImage.subtract(imp, bkgImp);
        resImp.setTitle("Background subtracted "+ imp.getTitle());
        
        if(showBackground) bkgImp.show();

        resImp.show();
    }
}
