/*
 * Copyright (C) 2014 Steffen Greilich
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package fntd.plugins;

/**
 *
 * @author greilich
 */
import fntd.stacks.StackOperations;
import ij.*;
import ij.gui.*;
import ij.plugin.*;

/**
 * Transforms a stack into a single image or new stack by tiling successive
 * slices to new images, e.g. transform a 100-image stack into 25 slices with
 * 2x2 tiles.
 *
 * @author greilich
 */
public class TileStack_ implements PlugIn {

    static int nX = 0;
    static int nY = 0;

    @Override
    public void run(String arg) {
        ImagePlus imp = IJ.getImage();
        int dX = imp.getWidth();
        int dY = imp.getHeight();
        int nZ = imp.getNSlices();

        nX = 1;
        nY = nZ;

        /**
         * Get user input
         */
        GenericDialog gd = new GenericDialog("Tile stack");
        gd.addNumericField("Number of tiles in x: ", nX, 0);
        gd.addNumericField("Number of tiles in y: ", nY, 0);

        gd.showDialog();

        if (gd.wasCanceled()) {
            return;
        }

        nX = (int) gd.getNextNumber();
        nY = (int) gd.getNextNumber();

        if ((nZ % (nX * nY)) != 0) {
            IJ.showMessage("Stack to tiled stack", "The product of " + Integer.toString(nX)
                    + " and " + Integer.toString(nY) + " is not a divider of the number of slices "
                    + Integer.toString(nZ) + ". Tiles with pixel value zero will be padded.");
        }

        ImagePlus resImp = StackOperations.tileXYZ(nX, nY, imp);
        resImp.show();
    }
}
