/*
 * Copyright (C) 2014 Steffen Greilich
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package fntd.plugins;

/**
 *
 * @author greilich
 */
import fntd.stacks.StackOperations;
import ij.*;
import ij.gui.*;
import ij.plugin.*;

public class UntileStack_ implements PlugIn {

    static int nX = 10;
    static int nY = 10;

    /**
     * Transforms image or image stack into new stack by rearranging the tiles
     * in each slice into subsequent single slices, e.g. 2x2 tiles in a 25 slice
     * stack into a new 100 slice stack.
     * 
     * @param arg 
     */
    public void run(String arg) {
        ImagePlus imp = IJ.getImage();

        /**
         * Get user input
         */
        GenericDialog gd = new GenericDialog("Untile stack");
        gd.addNumericField("Number of tiles in x: ", nX, 0);
        gd.addNumericField("Number of tiles in y: ", nY, 0);

        gd.showDialog();

        if (gd.wasCanceled()) {
            return;
        }

        nX = (int) gd.getNextNumber();
        nY = (int) gd.getNextNumber();

        int dX = imp.getWidth() / nX;
        int dY = imp.getHeight() / nY;

        ImagePlus resImp = StackOperations.untile(nX, nY, dX, dY, imp);
        resImp.show();
    }
}
