package fntd.tracking;

import ij.ImageStack;

import java.util.Vector;
import mosaic.core.detection.MyFrame;
import mosaic.core.detection.Particle;

/**
 * Extension of the MOSAIC MyFrame class. Contains FNTDParticles instead of 
 * MOSAIC Particles.
 *
 * @author Jasper Kouwenberg
 */
public class FNTDMyFrame extends mosaic.core.detection.MyFrame {

    public Vector<FNTDParticle> FNTDparticles;
    public int FNTDparticles_number;

    /**
     * Default constructor
     *
     */
    public FNTDMyFrame() {
        super(null,0,0);
    }

    /**
     * Default constructor
     *
     * @param frame
     * @param linkrange
     */
    public FNTDMyFrame(MyFrame frame, int linkrange) {
        super(frame.getOriginalImageStack(), frame.frame_number, linkrange);
        
        this.FNTDparticles = new Vector<>();

        frame.getParticles().stream().forEach((q) -> {
            this.FNTDparticles.addElement(new FNTDParticle(q));
        });
        
        this.FNTDparticles_number = this.FNTDparticles.size();
    }
		
    /**
     *
     * Constructor for text mode from a vector of particles
     *
     * @param p
     * @param frame
     *
     */
    public FNTDMyFrame(Vector<FNTDParticle> p, int frame) {
        super(null, frame, 0);
        
        this.frame_number = frame;
        this.FNTDparticles_number = p.size();

        this.FNTDparticles = p;
    }

    /**
     * Constructor for ImageProcessor based MyFrame.
     * <br>All particles and other information will be derived from the given
     * <code>ImageProcessor</code> by applying Detector methods
     *
     * @param ips
     * @param frame_num the serial number of this frame in the movie
     * @param aLinkrange link range
     */
    public FNTDMyFrame(ImageStack ips, int frame_num, int aLinkrange) {
        super(ips, frame_num, aLinkrange);
        
        this.FNTDparticles = new Vector<>();
    }

    /**
     *
     * Return particles vector
     *
     * @return a vector of particle
     */
    public Vector<FNTDParticle> getFNTDParticles() {
        return this.FNTDparticles;
    }

    public void setFNTDParticles(Vector<FNTDParticle> FNTDparticles, int particles_number) {
        this.FNTDparticles = FNTDparticles;
        this.FNTDparticles_number = particles_number;
    }

    public void MosaicToFNTD() {
        Vector<Particle> particles = this.getParticles();
        this.FNTDparticles = new Vector<>();

        particles.stream().forEach((q) -> {
            this.FNTDparticles.addElement(new FNTDParticle(q));
        });
        
        this.FNTDparticles_number = this.FNTDparticles.size();
    }

    public void FNTDToMosaic() {
        Vector<Particle> particles = new Vector<>();

        this.FNTDparticles.stream().forEach((q) -> {
            particles.addElement((Particle) q);
        });
        
        this.setParticles(particles);
    }

    /**
     *
     * Remove double particles in the frame, needed for segmentation some tool
     * produce double regions
     *
     */
    public void removeDoubleFNTDParticles() {
        boolean f;
        Vector<FNTDParticle> p = new Vector<>();

        for (int i = 0; i < FNTDparticles.size(); i++) {
            f = false;

            for (int j = i + 1; j < FNTDparticles.size(); j++) {
                if (FNTDparticles.get(i).match(FNTDparticles.get(j))) {
                    f = true;
                }
            }

            if (f == false) {
                p.add(FNTDparticles.get(i));
            }
        }

        FNTDparticles = p;
        
        this.FNTDToMosaic();
    }
}