# Corrects the measured track intensity based on the depth in the crystal
# Equation and default parameters from Alexander Stadler's thesis
# This routine is an alternative to FNTD.normalize.stacks and can be used when the track density changes
# significantly between the slides
# !! z.start is the z-position of the first slice, not the reference depth  !!

FNTD.apply.axial.correction <- function(jimg,
                                        parameters = list(A   = 1.167,
                                                          B   = 1.324E-5,
                                                          C   = -5.424E-3,
                                                          z.start = 0,
                                                          dz  = 2)) {
  dims <- dim(jimg)
  z <- parameters$z.start + (0:(dims[3]-1))*parameters$dz
  f.z  <- 1 / (parameters$A*exp(parameters$B*z^2+parameters$C*z))
  
  v    <- array(1,dim=c(dims[1],dims[2],dims[3])) * rep(f.z, each=dims[1]*dims[2])
  
  kimg <- FNTDjImage(v,
                     c(dims[1],dims[2],dims[3],1,1),
                     "Axial calibration (function)")
  
  return(jimg * kimg)
}