FNTD.apply.flatfield <- function(jimg, 
                                 method = "file", 
                                 file = system.file("extdata", 
                                                    "flatfields", 
                                                    "FXR700RG_DKFZ_150611.tif.zip",
                                                    package = "FNTD"), 
                                 parameters = list( x0  = 238.9,
                                                    y0  = 352.4,
                                                    a.c = -1.158e-6,
                                                    b.c = 5.02e-5,
                                                    mu  = 0.950), 
                                 n.tiles = 8){
  
  if(!method%in%c("file", "function", "normalize"))
    stop("Invalid method given.")
  
  if(method == "file"){
#    if(missing(file)){
#      stop("No file given for method \'file\'.")
#    }  
    kimg <- FNTD.read.image(file)
    mimg <- jimg / kimg
  }
  
  if(method == "function"){
#    if(missing(parameters)){
#      stop("No parameters given for method \'function\'.")
#    }  
    dims <- dim(jimg)
    x    <- rep(1:dims[1], dims[2])
    y    <- sort(rep(1:dims[2], dims[1]))
    r    <- sqrt((x-parameters$x0)^2+(y-parameters$y0)^2)
    v    <- parameters$mu / (parameters$a.c * r^2 + parameters$b.c * r + 1)
    kimg <- FNTDjImage(v,
                       c(dims[1],dims[2],1,1,1),
                       "Flatfield (function)")
    mimg <- jimg * kimg
  }

  if(method == "normalize"){
    #n.tiles <- 4
    m    <- jimg[,,1,1,1]
    dims <- dim(jimg)    
    idx.factor <- dims[1] / n.tiles
    get.idx <- function(i){
      return(floor((i-1)/idx.factor)  )
    }
    
    # Konstruiere Matrix m aber mit Auflösung n.tiles/n.tiles
    mm <- as.matrix(expand.grid(i=1:(n.tiles*idx.factor), 
                                j=1:(n.tiles*idx.factor)))
    mm[,1] <- get.idx(mm[,1])
    mm[,2] <- get.idx(mm[,2])
    
    n <- matrix(mm[,1]+mm[,2]*n.tiles, # Hier Offset eintragen
                ncol = n.tiles*idx.factor, 
                nrow = n.tiles*idx.factor,
                byrow = TRUE)
    o <- matrix(tapply(m, n, mean), ncol = n.tiles, byrow = TRUE)
    levelplot(o, byrow = TRUE)
    
  }

  return(mimg)
}