# Function for calculating the penetration into the FNTD using the surface reflection fit
# and a possible offset in the fluorescence image
FNTD.find.depth <- function(trackspots, fit.param, voxel.size, optical.param) {
  
  zpos  <- voxel.size$z.um * trackspots$slice.no + fit.param$image.offset - 
    (fit.param$z.offset + fit.param$x.slope*trackspots$x*voxel.size$x.um + fit.param$y.slope*trackspots$y*voxel.size$y.um)
  
  dn    <- (optical.param$n2 - optical.param$n1)/optical.param$n1
  f_s.p <- zpos * dn
  G     <- data.frame(a=c(rep(1/sqrt(1 - (optical.param$NumAp/optical.param$n1)^2),length(f_s.p))),
                      b=(1.145 + 1/(0.65 + 1.145*abs(f_s.p))))
  
  if (optical.param$depth.corr == "Full") {
    z2   <- apply(G,1,min)*f_s.p + zpos
  } else if (optical.param$depth.corr == "Partial") {
    z2   <- mean(apply(G,1,min)*f_s.p/zpos + 1)*zpos
  } else {
    z2   <- zpos
  }
  
  return(z2)
}