FNTD.get.energy <- function(tracks, LET.files,
                            layer.thickness,
                            procedural.track.length = FALSE) {
  #V1 = ENERGY, V2 = LET, V3 = RANGE
  
  n                 <- length(layer.thickness)
  tracks$energy     <- vector(mode = "list", length = nrow(tracks))
  na.count          <- 0
  
  for (layer in 1:n) {
    LET <- read.table(LET.files[layer], sep=',')
    
    if (layer == 1) {
      for (j in 1:nrow(tracks)) {
        tracks$energy[[j]]        <- rep(0,n)
        if (procedural.track.length) 
          tracks$energy[[j]][layer] <- approx(x=LET$V3, y=LET$V1, tracks$csda.length[j], method = "linear")$y #Find energy corresponding to tracklength in layer
        else
          tracks$energy[[j]][layer] <- approx(x=LET$V3, y=LET$V1, tracks$proj.length[j], method = "linear")$y
      }
    } else {
      d <- layer.thickness[layer]*sqrt(1 + tracks$xz.slope^2 + tracks$yz.slope^2)
      
      for (j in 1:nrow(tracks)) {
        d0 <- approx(x=LET$V1, y=LET$V3, tracks$energy[[j]][layer - 1], method = "linear")$y
        if (!is.na(d0) && !is.nan(d0))
          tracks$energy[[j]][layer] <- approx(x=LET$V3, y=LET$V1, d0 + d[j], method = "linear")$y
        else
          na.count <- na.count + 1
      }
    }
  }
  
  tracks$energy.total <- matrix(unlist(tracks$energy), ncol = 3, byrow = TRUE)[,3]
  
  if (na.count > 0)
    cat("Errors:", na.count, "\n")
  
  return (tracks)
}