FNTD.get.macro.dose <- function(tracks, LET.files = list(),
                                image.size    = list(n.x = 1024,
                                                     n.y = 1024,
                                                     n.slices = 30),
                                voxel.size    = list(x.um = .1,
                                                     y.um = .1,
                                                     z.um = .1),
                                border.distance = 15,
                                target.layer.height = 8,
                                pre.target.heights = c(0),
                                target.density = 1, #kg/m3
                                interval = 0.005,
                                slice = F){
  
  energy <- rep(0,nrow(tracks))
  
  #prelayers
  for (k in 1:length(pre.target.heights)) {
    LET <- read.table(LET.files[k], sep=',')
    
    for (j in 1:length(energy)) {
      d0 <- approx(x=LET$V1, y=LET$V3, tracks$energy[[j]][1], method = "linear")$y
      
      l <- sqrt(1 + tracks$yz.slope[j]^2 + tracks$xz.slope[j]^2)*pre.target.heights[k]
      
      tracks$energy[[j]][1] <- approx(x=LET$V3, y=LET$V1, d0 - l, method = "linear")$y
    }
  }
  
  #dose layer
  LET <- read.table(tail(LET.files, n=1), sep=',')
  if (!slice) { #Calculate volume
    z   <- (round(sum(pre.target.heights)/interval):(round((sum(pre.target.heights) + target.layer.height)/interval)))*interval
    
    if (length(z) > 0) {
      for (j in 1:length(energy)) {
        d0 <- approx(x=LET$V1, y=LET$V3, tracks$energy[[j]][1], method = "linear")$y
        
        x <- z*tracks$xz.slope[j] + tracks$xz.int.um[j]
        y <- z*tracks$yz.slope[j] + tracks$yz.int.um[j]
        l <- sqrt(1 + tracks$yz.slope[j]^2 + tracks$xz.slope[j]^2)
        
        z_adj <- z[x >= border.distance*voxel.size$x.um & x <= voxel.size$x.um*(image.size$n.x - border.distance) &
                     y >= border.distance*voxel.size$y.um & y <= voxel.size$y.um*(image.size$n.y - border.distance)]*l
        
        if (length(z_adj) > 1) {
          t <- approx(x=LET$V3, y=LET$V1, c(d0 - min(z_adj), d0 - max(z_adj)), method = "linear")$y
          energy[j] <- t[1] - t[2]
        }
      }
    }
    
    mass <- target.density * 1E-18 * target.layer.height*(image.size$n.x - 2*border.distance)*
      voxel.size$x.um*(image.size$n.y - 2*border.distance)*voxel.size$y.um
  } else {
    for (j in 1:length(energy)) {
      x <- pre.target.heights*tracks$xz.slope[j] + tracks$xz.int.um[j]
      y <- pre.target.heights*tracks$yz.slope[j] + tracks$yz.int.um[j]
      l <- sqrt(1 + tracks$yz.slope[j]^2 + tracks$xz.slope[j]^2)
      
      if (x >= border.distance*voxel.size$x.um & x <= voxel.size$x.um*(image.size$n.x - border.distance) &
          y >= border.distance*voxel.size$y.um & y <= voxel.size$y.um*(image.size$n.y - border.distance)) {
        t <- approx(x=LET$V1, y=LET$V2, tracks$energy[[j]][1], method = "linear")$y
        energy[j] <- t*l
      }
    }
    
    mass <- target.density * 1E-18 * (image.size$n.x - 2*border.distance)*voxel.size$x.um*(image.size$n.y - 2*border.distance)*voxel.size$y.um
  }
  
  return(sum(energy, na.rm = TRUE)/mass*1.60218e-16)
}