FNTD.get.macro.dose.sd <- function(tracks, LET.file.loc = '', FNTD.WER,
                                      image.size    = list(n.x = 1024,
                                                           n.y = 1024,
                                                           n.slices = 30),
                                      voxel.size    = list(x.um = .1,
                                                           y.um = .1,
                                                           z.um = .1),
                                      border.distance = 15,
                                      dose.depth = 0,
                                      dose.layer.thickness = 0,
                                      LET.slope = 2.875,
                                      WER.sd = 3.25) {
  
  LET <- read.table(LET.file.loc, sep=',')
  
  tracks$WER <- (tracks$zpos.last - dose.depth)*FNTD.WER/cos(tracks$theta.deg*pi/180)
  tracks$WER[tracks$WER > max(LET$V1)] <- max(LET$V1)
  
  tracks$LET <- approx(x=LET$V1, y=LET$V2, tracks$WER, method = "linear")$y
  
  it <- round(dose.layer.thickness/0.5)
  
  if (it > 0) {
    for (i in 1:it) {
      tracks$LET <- tracks$LET + approx(x=LET$V1, y=LET$V2, tracks$WER - (dose.layer.thickness/it * i)/cos(tracks$theta.deg*pi/180), method = "linear")$y
    }
    tracks$LET <- tracks$LET/(it+1)
  }
  
  mass <- (image.size$n.x - border.distance)*voxel.size$x.um*(image.size$n.y - border.distance)*voxel.size$y.um*1E-15
  
  return(sum((LET.slope*WER.sd/tracks$LET)/cos(tracks$theta.deg*pi/180))/mass*1.60218e-16)
}