FNTD.get.track.properties <- function(x, radius, critical.distance, method = "center-adjacent", 
                                      voxel.size    = list(x.um = .1,
                                                           y.um = .1,
                                                           z.um = .1),
                                      surface.param = list(z.offset = 0,
                                                           x.slope = 0,
                                                           y.slope = 0,
                                                           image.offset = 0,
                                                           theta.cor = FALSE),
                                      optical.param = list(n1 = 1.517,
                                                           n2 = 1.7659,
                                                           NumAp = 1.40,
                                                           depth.corr = T),
                                      image.size    = list(n.x = 1024,
                                                           n.y = 1024,
                                                           n.slices = 30),
                                      depth.refinement = "Full",
                                      amplitude.endpoint.mod = 0.5,
                                      detour.range = 5) {
  
  if(!method%in%c("maximum", "mean", "center-adjacent"))
    stop("Invalid method name")
  
  if(class(x) != "data.frame" & class(x) != "FNTDjImage")
    stop("Object x has to be of class \"FNTDjImage\" or \"data.frame\".")
  
  if(class(x) == "FNTDjImage" & missing(radius))
    stop("Please give a radius.")
    
  if(class(x) == "FNTDjImage") {
    df <- FNTD.get.trackspot.properties(jimg=x, radius=radius, critical.distance=critical.distance)
    n  <- dim(x)[1:3]
    image.size  = list(n.x = n[1],
                       n.y = n[2],
                       n.slices = n[3])
  } else
    df <- x
  
  if(sum(df$track.no) == 0)
    stop("No track information available.")
  
  if (method == "center-adjacent")
    df$amplitude <- df$center.adj
  else if (method == "mean")
    df$amplitude <- df$mean
  else
    df$amplitude <- df$max
  
  fits <- FNTD.get.track.fits(df, voxel.size, surface.param, optical.param, image.size, 
                              depth.refinement, amplitude.endpoint.mod, detour.range)
  
  cv       <- paste(df$track.no, df$frame.no)
  df.track <- data.frame(track.no       = tapply(df$track.no,   cv,  unique),
                         frame.no       = tapply(df$frame.no,   cv,  unique),
                         channel.no     = tapply(df$channel.no, cv,  unique),
                         n.slices       = tapply(df$slice.no,   cv,  function(x){max(x)-min(x)+1}),
                         n.track.spots  = tapply(df$slice.no,   cv,  length),
                         slice.start    = tapply(df$slice.no,   cv,  min),
                         slice.end      = tapply(df$slice.no,   cv,  max),
                         x.end          = tapply(df$x,          cv,  function(x){tail(x,1)}),
                         y.end          = tapply(df$y,          cv,  function(x){tail(x,1)}),
                         amplitude      = tapply(df$amplitude,  cv,  as.list))
  
  df.track <- FNTD.filter.fitted(fits, df.track)
  return(merge(df.track, fits, by = c("track.no","frame.no")))
} 