FNTD.get.trackspot.properties <- function(jimg, radius, critical.distance = NULL, no.background.Gauss = TRUE){
  
  if(class(jimg) != "FNTDjImage")
    stop("Object jimg has to be of class \"FNTDjImage\".")
  
  if(missing(critical.distance)) {
    critical.distance <- 2*radius + 1
    cat("Auto-setting criticial distance to", critical.distance, "\n")
  }
  
  results <- J("fntd/tracking/FNTDtracks",
               "getProperties",
               jimg@imagePlus, 
               jimg@tracks, 
               as.numeric(radius), 
               as.numeric(critical.distance),
               as.logical(no.background.Gauss))
  
  # Convert java-Object into a matrix, add to image and return
  resmatrix=sapply(results, .jevalArray)
  
  df <- data.frame(channel.no 	 = resmatrix[,1],
                   frame.no 	   = resmatrix[,2],
                   slice.no 	   = resmatrix[,3],
                   track.no 	   = resmatrix[,4], 
                   x             = resmatrix[,5],
                   y             = resmatrix[,6],
                   mean 		     = resmatrix[,7],
                   max 		       = resmatrix[,8],
                   center.adj    = resmatrix[,9])
  
  # set measurements below critical distance (flagged "0" by Java) to NA
  ii <- df$max == 0
  df$mean[ii]       <- NA
  df$amplitude[ii]  <- NA
  df$center.adj[ii] <- NA
  
  return(df)
} 