FNTD.plot.image <- function(jimg, scale = 1.0, showTracks = FALSE, show = TRUE, layout = NULL, title = NULL){
     
     if(class(jimg) != "FNTDjImage")
       stop("Object jimg has to be of class \"FNTDjImage\".")
     
     if((showTracks)&(is.jnull(jimg@tracks)))
       stop("Track overlay requested but no track information available.")
     
     if(showTracks){
       jimg <- new("FNTDjImage",
         imagePlus = J("fntd/access/DisplayImage", 
           "flattenImage", 
           jimg@imagePlus,
           jimg@tracks),
         tracks = .jnull())
     }
  
     if(scale != 1.0){ 
       jimg <- new("FNTDjImage",
                   imagePlus = J("fntd/access/AccessImage", 
                                 "scale", 
                                 jimg@imagePlus,
                                 as.double(scale)),
                   tracks = .jnull())
     }
    
     m  <- jimg[,,,1,1]
     nX <- dim(m)[1]
     nY <- dim(m)[2]
     nZ <- dim(m)[3]

     getColValues <- function(x){
         rgb( red   = bitwAnd(x,0x00FF0000)/(256*256), 
              green = bitwAnd(x,0x0000FF00)/256, 
              blue  = bitwAnd(x,0x000000FF),
              maxColorValue = 255)}
 
     m <- aperm(array(getColValues(as.vector(m)), dim = c(nX,nY,nZ)), perm = c(2,1,3))
     df.plot <- data.frame(x = rep(nX/2, nZ),
                           y = rep(nY/2, nZ),
                           v = rep(1, nZ),
                           z = 1:nZ)

    x.range <- range(1,nX/scale)
    y.range <- range(1,nY/scale)
    x.ticks <- pretty(x.range)
#    x.ticks <- c(head(x.range,1), x.ticks[c(-1, -length(x.ticks))], tail(x.range, 1))
    
    y.ticks <- pretty(y.range)
#    y.ticks <- c(head(y.range,1), y.ticks[c(-1, -length(y.ticks))], tail(y.range, 1))
    rev.y.ticks <- nY/scale - y.ticks + 1
    
    if(is.null(title))
      title <- ""
    
    if(is.null(layout))
      layout <- c(0,nZ)

    plotto <- xyplot(y ~ x|sprintf("slice %02d", z),
           df.plot,
           main     = title,
           as.table = TRUE,
           xlim     = x.range,
           ylim     = y.range,
           xlab     = "x / px",
           ylab     = "y / px",
           scales   = list(x = list( at = x.ticks,
                                     labels = as.character(x.ticks)),
                         y = list( at = rev.y.ticks,
                                   labels = as.character(y.ticks))),
           aspect   = nY/nX,
           layout   = layout, 
           colorkey = FALSE,
           between  = list(x = 1, y = 1),
           panel    = function(x=x,y=y,subscripts=subscripts,...){
             panel.xyplot(x=x,y=y,...)
             grid::grid.raster(m[,,subscripts], 
                         default.units = "native",
                         hjust = 0,
                         vjust = 0,
                         x = 1,
                         y = 1,
                         width = nX/scale,
                         height = nY/scale,
                         interpolate = FALSE)
           })
    if(show)
      plot(plotto)
    else
      return(plotto)
}