FNTD.real.cell.hits <- function(jimg, cimg, tracks,
                                fntd.voxel.size     = list(x.um = .1,
                                                           y.um = .1,
                                                           z.um = .1),
                                cell.voxel.size     = list(x.um = .1,
                                                           y.um = .1,
                                                           z.um = .1),
                                cell.surface.param = list(z.offset = 0,
                                                          x.slope = 0,
                                                          y.slope = 0,
                                                          image.offset = 0),
                                FNTD.surface.param = list(z.offset = 0,
                                                          x.slope = 0,
                                                          y.slope = 0,
                                                          image.offset = 0),
                                border.distance = 15,
                                LET.file = '',
                                no.pos = 1,
                                cell.dose = 1,
                                fntd.dose = 1,
                                dose.cycles = 2,
                                draw = FALSE) {
  
  
  if(class(jimg) != "FNTDjImage")
    stop("Object jimg has to be of class \"FNTDjImage\".")
  
  if(class(cimg) != "FNTDjImage")
    stop("Object cimg has to be of class \"FNTDjImage\".")
  
  tracks.list   <- list(tracks$xz.slope,
                        tracks$yz.slope,
                        tracks$xz.int.um,
                        tracks$yz.int.um,
                        tracks$range.FNTD)
  
  #Select number of tracks required to calculate requested cell dose
  dose.num.cycles <- ceiling(dose.cycles)
  dose.per.cycle  <- cell.dose/dose.num.cycles
  
  if (dose.per.cycle > fntd.dose) {
    dose.per.cycle  <- dose.per.cycle/ceiling(dose.per.cycle/fntd.dose)
    dose.num.cycles <- ceiling(cell.dose/dose.per.cycle)
  }
  
  tracks.list   <- matrix(unlist(tracks.list), ncol = 5, byrow = FALSE)
  
  cell.voxel.size.j   <- matrix(unlist(cell.voxel.size), ncol = 3, byrow = TRUE)
  cell.surface.param  <- matrix(unlist(cell.surface.param), ncol = 4, byrow = TRUE)
  FNTD.surface.param  <- matrix(unlist(FNTD.surface.param), ncol = 4, byrow = TRUE)
  
  LET <- read.table(LET.file, sep=',')
  LET <- t(matrix(unlist(LET), ncol = 3, byrow = FALSE))
  
  no.of.tracks <- dose.per.cycle/fntd.dose * nrow(tracks)
  
  CHS <- .jnew("fntd/tracking/CellHitStatistics",
               cimg@imagePlus,
               .jarray(tracks.list,"[[D", dispatch=TRUE),
               .jarray(cell.voxel.size.j,"[D"),
               .jarray(dim(jimg),"[I"),
               .jarray(FNTD.surface.param,"[D"),
               .jarray(cell.surface.param,"[D"),
               .jarray(LET,"[[D", dispatch=TRUE),
               as.double(no.of.tracks),
               as.logical(draw))
  
  nuc <- J("fntd/tracking/CellHitStatistics",
           "Nucleus_volume",
           CHS)
  
  cyt <- J("fntd/tracking/CellHitStatistics",
           "Cytoplasm_volume",
           CHS)

  res <- J("fntd/tracking/CellHitStatistics",
           "RealCellHit",
           CHS,
           as.integer(dose.num.cycles),
           as.integer(border.distance),
           as.integer(no.pos))
  
  output           = list()
  output$energy    = .jevalArray(res[[1]])
  output$length    = .jevalArray(res[[2]])
  output$no.tracks = .jevalArray(res[[3]])
  output$nuc       = nuc
  output$cyt       = cyt
  
  return(output)
}