FNTD.simulate.cell.hits <- function(jimg, tracks,
                                    fntd.voxel.size     = list(x.um = .1,
                                                               y.um = .1,
                                                               z.um = .1),
                                    FNTD.surface.param = list(z.offset = 0,
                                                              x.slope = 0,
                                                              y.slope = 0,
                                                              image.offset = 0),
                                    LET.file = '',
                                    bounding.box = 100,
                                    Rnuc = 0,
                                    Rcyt_min = 0,
                                    Rcyt_max = 0,
                                    no.of.particles = 1) {
  
  
  if(class(jimg) != "FNTDjImage")
    stop("Object jimg has to be of class \"FNTDjImage\".")
  
  tracks.list   <- list(tracks$xz.slope,
                        tracks$yz.slope,
                        tracks$xz.int.um,
                        tracks$yz.int.um,
                        tracks$range.FNTD)
  
  tracks.list   <- matrix(unlist(tracks.list), ncol = 5, byrow = FALSE)
  
  FNTD.surface.param  <- matrix(unlist(FNTD.surface.param), ncol = 4, byrow = TRUE)
  FNTD.voxel.size.j   <- matrix(unlist(fntd.voxel.size), ncol = 3, byrow = TRUE)
  
  LET <- read.table(LET.file, sep=',')
  LET <- t(matrix(unlist(LET), ncol = 3, byrow = FALSE))
  
  CHS <- .jnew("fntd/tracking/CellHitStatistics",
               .jarray(tracks.list,"[[D", dispatch=TRUE),
               .jarray(FNTD.voxel.size.j,"[D"),
               .jarray(dim(jimg),"[I"),
               .jarray(FNTD.surface.param,"[D"),
               .jarray(LET,"[[D", dispatch=TRUE))
  
  res <- J("fntd/tracking/CellHitStatistics",
           "SimulateCellHit",
           CHS,
           as.double(bounding.box),
           as.double(Rnuc),
           as.double(Rcyt_min),
           as.double(Rcyt_max),
           as.integer(no.of.particles))
  
  output           = list()
  output$cell      = rep("Art.", N)
  
  output$energy    = .jevalArray(res[[3]])
  output$dose      = .jevalArray(res[[3]]) * 1.60218E-16/(4/3*pi*Rnuc^3 * 1E-18 * 1000)
  
  output$length    = .jevalArray(res[[2]])
  output$l.cyt     = .jevalArray(res[[1]])
  
  output$no.tracks = rep(1, N)
  output$nuc       = rep(4/3*pi*Rnuc^3, N)
  output$cyt       = rep(0, N)
  
  return(output)
}