FNTD.spheroid.simulation <- function(bounding.box = 100,
                                     Imode = 1,
                                     iso_range = c(0),
                                     iso_prob  = c(0),
                                     iso_name  = c(0),
                                     Rnuc = 0,
                                     Reff = 0,
                                     LET.file = '',
                                     no.of.particles = 10000) {
  
  LET <- read.table(LET.file, sep=',')
  LET <- t(matrix(unlist(LET), ncol = 3, byrow = FALSE))
  
  CHS <- .jnew("fntd/tracking/CellHitStatistics",
               .jarray(LET,"[[D", dispatch=TRUE))

  res <- J("fntd/tracking/CellHitStatistics",
           "SimulateSpheroid",
           CHS,
           as.double(bounding.box),
           as.integer(Imode),
           .jarray(iso_range,"[D"),
           .jarray(iso_prob,"[D"),
           as.double(Rnuc),
           as.double(Reff),
           as.integer(no.of.particles))
  
  output            = list()
  output$cell       = rep("Art.", N)
  output$iso        = iso_name[.jevalArray(res[[1]])]
  
  output$v.nuc      = rep(4/3*pi*Rnuc^3, N)
  output$v.cyt      = rep(4/3*pi*(Reff^3 - Rnuc^3), N)
  
  output$energy.nuc = .jevalArray(res[[4]])
  output$dose.nuc   = output$energy.nuc * 1.60218E-16/(output$v.nuc[1] * 1E-18 * 1000)
  
  output$energy.cyt = sum(.jevalArray(res[[5]]) - .jevalArray(res[[4]]))/N
  output$dose.cyt   = output$energy.cyt * 1.60218E-16/(output$v.cyt[1] * 1E-18 * 1000)
  
  output$l.nuc      = .jevalArray(res[[3]])
  output$l.cyt      = .jevalArray(res[[2]])
  
  output$no.nuc     = rep(1, N)
  output$no.cyt     = .jevalArray(res[[9]])
  
  output$x          = .jevalArray(res[[6]])
  output$y          = .jevalArray(res[[7]])
  output$z          = .jevalArray(res[[8]])
  
  return(output)
}