################################
# FNTDjImage CLASS
################################
setClass(
  Class = "FNTDjImage",
  representation   = representation(imagePlus           = "jobjRef",
                                    tracks              = "jobjRef"),
  prototype        = list(imagePlus = .jnull(),
                          tracks		= .jnull())
)

################################
# FNTDjImage AUX FUNCTIONS
################################
checkIndex <- function(idx) {
  if (missing(idx))
    idx <- .jarray(as.integer(0))
  else
    idx <- .jarray(as.integer(idx))
  
  return(idx)
}

################################
# FNTDjImage METHODS
################################
setMethod(
  f          = "dim",
  signature  = c("FNTDjImage"),
  definition = function(x) {
    sizes <-  .jcall(.jnew("fntd/access/AccessImage"),
                     "[I",
                     "getSizes",
                     x@imagePlus)
    names(sizes) <-
      c("x", "y", "z (slices)", "t (frames)", "c (channels)")
    return(sizes)
  }
)


setMethod(
  f          = "+",
  signature  = c("FNTDjImage", "FNTDjImage"),
  definition = function(e1, e2) {
    return(new(
      "FNTDjImage",
      imagePlus   	  = J("fntd/access/AccessImage")$add(e1@imagePlus, e2@imagePlus),
      tracks 					= .jnull()
    ))
  }
)

setMethod(
  f          = "+",
  signature  = c("FNTDjImage", "numeric"),
  definition = function(e1, e2) {
    if (length(e2) > 1) {
      stop("The second argument has to be a single number.")
    }
    return(new(
      "FNTDjImage",
      imagePlus   		= J("fntd/access/AccessImage")$add(e1@imagePlus, e2),
      tracks 					= e1@tracks
    ))
  }
)

setMethod(
  f          = "-",
  signature  = c("FNTDjImage", "FNTDjImage"),
  definition = function(e1, e2) {
    return(new(
      "FNTDjImage",
      imagePlus 			= J("fntd/access/AccessImage")$subtract(e1@imagePlus, e2@imagePlus),
      tracks 					= .jnull()
    ))
  }
)

setMethod(
  f          = "-",
  signature  = c("FNTDjImage", "numeric"),
  definition = function(e1, e2) {
    if (length(e2) > 1) {
      stop("The second argument has to be a single number.")
    }
    return(new(
      "FNTDjImage",
      imagePlus   		= J("fntd/access/AccessImage")$subtract(e1@imagePlus, e2),
      tracks 					= e1@tracks
    ))
  }
)

setMethod(
  f          = "*",
  signature  = c("FNTDjImage", "FNTDjImage"),
  definition = function(e1, e2) {
    return(new(
      "FNTDjImage",
      imagePlus 			= J("fntd/access/AccessImage")$multiply(e1@imagePlus, e2@imagePlus),
      tracks 					= .jnull()
    ))
  }
)

setMethod(
  f          = "*",
  signature  = c("FNTDjImage", "numeric"),
  definition = function(e1, e2) {
    if (length(e2) > 1 & length(e2) != dim(e1)[3]) {
      stop("The dimensions of the arguments (number of slices) do not match.")
    }
    return(new(
      "FNTDjImage",
      imagePlus   		= J("fntd/access/AccessImage")$multiply(e1@imagePlus, e2),
      tracks 					= e1@tracks
    ))
  }
)

setMethod(
  f          = "/",
  signature  = c("FNTDjImage", "FNTDjImage"),
  definition = function(e1, e2) {
    return(new(
      "FNTDjImage",
      imagePlus 			= J("fntd/access/AccessImage")$divide(e1@imagePlus, e2@imagePlus),
      tracks 					= .jnull()
    ))
  }
)

setMethod(
  f          = "/",
  signature  = c("FNTDjImage", "numeric"),
  definition = function(e1, e2) {
    if (length(e2) > 1) {
      stop("The second argument has to be a single number.")
    }
    return(new(
      "FNTDjImage",
      imagePlus   		= J("fntd/access/AccessImage")$divide(e1@imagePlus, e2),
      tracks 					= e1@tracks
    ))
  }
)

setMethod(
  f          = "!",
  signature  = c("FNTDjImage"),
  definition = function(x) {
    return(new(
      "FNTDjImage",
      imagePlus     	= J("fntd/access/AccessImage")$invert(x@imagePlus),
      tracks 					= x@tracks
    ))
  }
)

setMethod(
  f          = "[[",
  signature  = "FNTDjImage",
  definition = function(x, i, j, k, m, n) {
    # Check indices
    i <- checkIndex(i)
    j <- checkIndex(j)
    k <- checkIndex(k)
    
    return(new (
      "FNTDjImage",
      imagePlus = .jcall(
        .jnew("fntd/access/AccessImage"),
        "Lij/ImagePlus;",
        "getSubHyperStack",
        x@imagePlus,
        i,
        j,
        k
      ),
      tracks    = .jnull()
    ))
  }
)


setMethod(
  f          = "[",
  signature  = "FNTDjImage",
  definition = function(x, i, j, k, l, m) {
    # Check if binary image is given for masking
    if (!missing(i)) {
      if (class(i) == "FNTDjImage") {
        if (!.jcall(.jnew("fntd/access/AccessImage"),
                    "Z",
                    "isBinary",
                    i@imagePlus)) {
          stop("Object mimg has to be a binary image")
        }
        m <- x[]
        m[as.logical((!i)[])] <- NA
        return(m)
      }
    }
    
    # Check if entire col/row selected
    sizes <- dim(x)
    if (missing(i)) 
      i <- 1:sizes[1]
    if (missing(j))
      j <- 1:sizes[2]
    if (missing(k))
      k <- 1:sizes[3]
    if (missing(l))
      l <- 1:sizes[4]
    if (missing(m))
      m <- 1:sizes[5]
    
    # Check if indices are within limits
    if (min(i) < 1 | max(i) > sizes[1] |
        min(j) < 1 | max(j) > sizes[2] |
        min(k) < 1 | max(k) > sizes[3] |
        min(l) < 1 | max(l) > sizes[4] |
        min(m) < 1 | max(m) > sizes[5])
      stop("Subscripts out of bounds")
    
    # To avoid "[I/I" error for single integer arguments, do this stupid trick
    if (length(i) == 1)
      i <- rep(i, 2)
    if (length(j) == 1)
      j <- rep(j, 2)
    if (length(k) == 1)
      k <- rep(k, 2)
    if (length(l) == 1)
      l <- rep(l, 2)
    if (length(m) == 1)
      m <- rep(m, 2)
    
    m <- array(
      .jcall(
        .jnew("fntd/access/AccessImage"),
        "[D",
        "getPixels",
        x@imagePlus,
        as.integer(k),
        as.integer(l),
        as.integer(m)
      ),
      dim = c(
        sizes[1], sizes[2], length(unique(k)), length(unique(l)), length(unique(m))
      )
    )
    return(m[i,j,,,])
  }
)

setMethod(
  f          = "mean",
  signature  = c("FNTDjImage"),
  definition = function(x) {
    m <- J("fntd/stacks/StackOperations", "sliceWiseMean", x@imagePlus)
    return(.jevalArray(m, "[[[D", simplify = TRUE))
  }
)

################################
# FNTDjImage FUNCTIONS
################################

FNTDjImage <- function(data, dim, title = "New image") {
  if (length(data) < dim[1] * dim[2] * dim[3] * dim[4] * dim[5])
    stop("data is smaller than size of new image.")
  
  imp <- .jcall(
    .jnew("fntd/access/AccessImage"),
    "Lij/ImagePlus;",
    "asImagePlus",
    as.character(title),
    as.numeric(data),
    as.integer(dim[1]),
    as.integer(dim[2]),
    as.integer(dim[3]),
    as.integer(dim[4]),
    as.integer(dim[5])
  )
  
  return(new("FNTDjImage",
             imagePlus 				= imp,
             tracks 					= .jnull()))
}

as.FNTDjImage <- function(data, title = "New Image") {
  dims <- dim(data)
  n <- length(dims)
  if (n > 5)
    stop("Array \"data\" must not have more than five dimensions.")
  
  if (n < 5)
    dim(data) <- c(dims, rep(1, 5 - n))
  
  if (length(data) < dims[1] * dims[2] * dims[3] * dims[4] * dims[5])
    stop("data is smaller than size of new image.")
  
  imp <- .jcall(
    .jnew("fntd/access/AccessImage"),
    "Lij/ImagePlus;",
    "asImagePlus",
    as.character(title),
    as.numeric(as.vector(data)),
    as.integer(dims[1]),
    as.integer(dims[2]),
    as.integer(dims[3]),
    as.integer(dims[4]),
    as.integer(dims[5])
  )
  
  return(new("FNTDjImage",
             imagePlus   			= imp,
             tracks 					= .jnull()))
}
