require(minpack.lm)

message("###########################\n...compiling results...\n")

##### 1 Reference slice #####
ii                      <- FNTD.results$data$df.trackspots$slice.no  == FNTD.get.option(FNTD.results$options, "reference.slice")
df.trackspots.ref.slice <- FNTD.results$data$df.trackspots[ii,]

## Intra-image fluctuation
cv                   <- paste(df.trackspots.ref.slice$id, df.trackspots.ref.slice$frame.no)
df.intra.image       <- data.frame(id           = tapply(df.trackspots.ref.slice$id,
                                                         cv,
                                                         unique),
                                   frame.no             = tapply(df.trackspots.ref.slice$frame.no,
                                                                 cv,
                                                                 unique),
                                   I.jkl                = tapply(df.trackspots.ref.slice$max,
                                                                 cv,
                                                                 function(x){sum(!is.na(x))}),
                                   LET.water.keV.um     = tapply(df.trackspots.ref.slice$LET.water.keV.um,
                                                                 cv,
                                                                 unique),
                                   particle.name        = tapply(df.trackspots.ref.slice$particle.name,
                                                                 cv,
                                                                 unique),
                                   mu.jkl               = tapply(df.trackspots.ref.slice$max,
                                                                 cv,
                                                                 mean,
                                                                 na.rm = TRUE),
                                   se.jkl               = tapply(df.trackspots.ref.slice$max,
                                                                 cv,
                                                                 sd,
                                                                 na.rm = TRUE),
                                   stringsAsFactors = FALSE)
df.intra.image$rel.se.jkl <- df.intra.image$se.jkl / df.intra.image$mu.jkl

# Test dependencies of intensity variation in frame
#summary(lm(rel.se.jkl ~ LET.water.keV.um + id + frame.no,
#           df.intra.image))

# LET-wise
cv                   <- paste(df.intra.image$LET.water.keV.um)
df.LET               <- data.frame(LET.water.keV.um     = tapply(df.intra.image$LET.water.keV.um,
                                                                 cv,
                                                                 unique),
                                   particle.name        = tapply(df.intra.image$particle.name,
                                                                 cv,
                                                                 unique),
                                   n.frames             = tapply(df.intra.image$frame.no,
                                                                 cv,
                                                                 length),
                                   I.l                  = tapply(df.intra.image$I.jkl,
                                                                 cv,
                                                                 mean,
                                                                 na.rm = TRUE),
                                   mu.l                 = tapply(df.intra.image$mu.jkl,
                                                                 cv,
                                                                 mean,
                                                                 na.rm = TRUE),
                                   sd.l                 = tapply(df.intra.image$mu.jkl,
                                                                 cv,
                                                                 sd,
                                                                 na.rm = TRUE),
                                   rel.se.l             = tapply(df.intra.image$rel.se.jkl,
                                                                 cv,
                                                                 mean,
                                                                 na.rm = TRUE),
                                   sd.rel.se.l         = tapply(df.intra.image$rel.se.jkl,
                                                                cv,
                                                                sd,
                                                                na.rm = TRUE))

## Intra-detector fluctuation
cv             <- paste(df.intra.image$id)
df.intra.det   <- data.frame(id                   = tapply(df.intra.image$id,
                                                           cv,
                                                           unique),
                             LET.water.keV.um     = tapply(df.intra.image$LET.water.keV.um,
                                                           cv,
                                                           unique),
                             particle.name        = tapply(df.intra.image$particle.name,
                                                           cv,
                                                           unique),
                             I.kl                 = tapply(df.intra.image$I.jkl,
                                                           cv,
                                                           mean,
                                                           na.rm = TRUE),
                             mu.kl                = tapply(df.intra.image$mu.jkl,
                                                           cv,
                                                           mean,
                                                           na.rm = TRUE),
                             sd.kl                = tapply(df.intra.image$mu.jkl,
                                                           cv,
                                                           sd,
                                                           na.rm = TRUE))

df.intra.det      <- merge(df.intra.det, df.LET, by = intersect(names(df.LET), names(df.intra.det)))
df.intra.det$ue.l <- df.intra.det$rel.se.l * df.intra.det$mu.kl / sqrt(df.intra.det$I.l)
df.intra.det$se.P.kl <- sqrt(df.intra.det$sd.kl^2 + df.intra.det$ue.l^2)
df.intra.det$rel.se.P.kl <- df.intra.det$se.P.kl / df.intra.det$mu.kl

#summary(lm(rel.se.P.kl ~ LET.water.keV.um + id,
#           df.intra.det))

plot.slice.influence.intra <- xyplot(sd.kl / ue.l ~ LET.water.keV.um,
                                     df.intra.det,
                                     grid      = TRUE,
                                     groups    = particle.name,
                                     ylab      = list("sd.kl / ue.l", font = 2, cex = 1.5),
                                     xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                                     main      = list("Influence intra-image var on intra-det var (reference slice)", font = 2, cex = 1.5),
                                     scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))

plot.slice.intra.det <- xyplot(rel.se.P.kl ~ LET.water.keV.um,
                               df.intra.det,
                               grid      = TRUE,
                               groups    = particle.name,
                               ylab      = list("sd.kl / ue.l", font = 2, cex = 1.5),
                               xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                               main      = list("Intra-detector variation (reference slice)", font = 2, cex = 1.5),
                               scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))

intra.det.slice <- mean(df.intra.det$rel.se.P.kl)


## Inter-detector fluctuation
cv             <- paste(df.intra.det$LET.water.keV.um)
df.inter.det   <- data.frame(LET.water.keV.um     = tapply(df.intra.det$LET.water.keV.um,
                                                           cv,
                                                           unique),
                             particle.name        = tapply(df.intra.det$particle.name,
                                                           cv,
                                                           unique),
                             n.detectors          = tapply(df.intra.det$id,
                                                           cv,
                                                           length),
                             mu.l                 = tapply(df.intra.det$mu.kl,
                                                           cv,
                                                           mean,
                                                           na.rm = TRUE),
                             sd.l                 = tapply(df.intra.det$mu.kl,
                                                           cv,
                                                           sd,
                                                           na.rm = TRUE))

df.inter.det$rel.se.l <- df.inter.det$sd.l / df.inter.det$mu.l
df.inter.det$rel.sd.D <- df.inter.det$rel.se.l

#summary(lm(rel.sd.D ~ LET.water.keV.um,
#           df.inter.det))

plot.slice.inter.det <- xyplot(rel.sd.D ~ LET.water.keV.um,
                               df.inter.det,
                               grid      = TRUE,
                               groups    = particle.name,
                               ylab      = list("rel.sd.D", font = 2, cex = 1.5),
                               xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                               main      = list("Inter-detector variation (reference slice)", font = 2, cex = 1.5),
                               scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))

inter.det.slice <- mean(df.inter.det$rel.sd.D)


## LET-intensity relation

plot.slice.LET.I<- xYplot(Cbind(mu.l, 
                                mu.l-sd.l/sqrt(n.frames),
                                mu.l+sd.l/sqrt(n.frames)) ~ LET.water.keV.um,
                          df.LET,
                          grid      = TRUE,
                          panel     = function(...){
                            panel.grid(h = -1, v = -1)
                            panel.xYplot(...)
                          },
                          groups    = particle.name,
                          ylab      = list("LET-mean intensity", font = 2, cex = 1.5),
                          xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                          main      = list("LET-intensity relation (reference slice)", font = 2, cex = 1.5),
                          scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))



plot.slice.u.LET.I <- xyplot(sd.l/mu.l ~ LET.water.keV.um,
                             df.LET,
                             grid      = TRUE,
                             type      = c("p", "r"),
                             ylab      = list("rel. sd. of of frame-mean intensity", font = 2, cex = 1.5),
                             xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                             main      = list("line-width (mean intensity)\nreference slice", font = 2, cex = 1.5),
                             scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))


plot.slice.intra.img <- xYplot(Cbind(rel.se.l, 
                                     rel.se.l-sd.rel.se.l/sqrt(n.frames),
                                     rel.se.l+sd.rel.se.l/sqrt(n.frames)) ~ LET.water.keV.um,
                               df.LET,
                               grid      = TRUE,
                               ylab      = list("LET-mean sd of intensity", font = 2, cex = 1.5),
                               xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                               main      = list("Intra-image variation (reference slice)", font = 2, cex = 1.5),
                               scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))

plot.slice.u.intra.img <- xyplot(sd.rel.se.l/rel.se.l ~ LET.water.keV.um,
                                 df.LET,
                                 grid      = TRUE,
                                 type      = c("p", "r"),
                                 ylab      = list("rel. sd. of of frame-mean sd of intensity", font = 2, cex = 1.5),
                                 xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                                 main      = list("line-width (intra-image variation)\nreference slice", font = 2, cex = 1.5),
                                 scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))

##### 2 Track-based #####

# Intra-image
cv                   <- paste(FNTD.results$data$df.tracks$id, FNTD.results$data$df.tracks$frame.no)
df.intra.image       <- data.frame(id                   = tapply(FNTD.results$data$df.tracks$id,
                                                                 cv,
                                                                 unique),
                                   frame.no             = tapply(FNTD.results$data$df.tracks$frame.no,
                                                                 cv,
                                                                 unique),
                                   n.tracks.jkl         = tapply(FNTD.results$data$df.tracks$track.no,
                                                                 cv,
                                                                 length),
                                   LET.water.keV.um     = tapply(FNTD.results$data$df.tracks$LET.water.keV.um,
                                                                 cv,
                                                                 unique),
                                   particle.name        = tapply(as.character(FNTD.results$data$df.tracks$particle.name),
                                                                 cv,
                                                                 unique),
                                   E.MeV.u              = tapply(FNTD.results$data$df.tracks$E.MeV.u,
                                                                 cv,
                                                                 unique),
                                   mu.jkl               = tapply(FNTD.results$data$df.tracks$mean.intensity,
                                                                 cv,
                                                                 mean,
                                                                 na.rm = TRUE),
                                   se.jkl               = tapply(FNTD.results$data$df.tracks$mean.intensity,
                                                                 cv,
                                                                 sd,
                                                                 na.rm = TRUE),
                                   str.jkl              = tapply(FNTD.results$data$df.tracks$rel.sd.intensity,
                                                                 cv,
                                                                 mean,
                                                                 na.rm = TRUE),
                                   str.adj.jkl          = tapply(FNTD.results$data$df.tracks$rel.sd.intensity.adj,
                                                                 cv,
                                                                 mean,
                                                                 na.rm = TRUE),
                                   lw.jkl               = tapply(FNTD.results$data$df.tracks$rel.line.width,
                                                                 cv,
                                                                 mean,
                                                                 na.rm = TRUE),
                                   sd.lw.jkl            = tapply(FNTD.results$data$df.tracks$rel.line.width,
                                                                 cv,
                                                                 sd,
                                                                 na.rm = TRUE),
                                   mean.theta.deg.jkl   = tapply(FNTD.results$data$df.tracks$mean.theta.deg,
                                                                 cv,
                                                                 mean,
                                                                 na.rm = TRUE),
                                   theta.corr.deg.jkl   = tapply(FNTD.results$data$df.tracks$theta.corr.deg,
                                                                 cv,
                                                                 mean,
                                                                 na.rm = TRUE))


df.intra.image$rel.se.jkl <- df.intra.image$se.jkl / df.intra.image$mu.jkl

# Test dependencies of intensity variation in frame
#summary(lm(rel.se.jkl ~ LET.water.keV.um + id + frame.no,
#           df.intra.image))

# LET-wise
cv <- df.intra.image$LET.water.keV.um
df.LET              <- data.frame( LET.water.keV.um = tapply(df.intra.image$LET.water.keV.um,
                                                             cv,
                                                             unique),
                                   particle.name    = tapply(df.intra.image$particle.name,
                                                             cv,
                                                             unique),
                                   E.MeV.u          = tapply(df.intra.image$E.MeV.u,
                                                             cv,
                                                             unique),
                                   n.frames         = tapply(df.intra.image$frame.no,
                                                             cv,
                                                             length),
                                   n.tracks.l       = tapply(df.intra.image$n.tracks.jkl,
                                                             cv,
                                                             mean,
                                                             na.rm = TRUE),
                                   mu.l                 = tapply(df.intra.image$mu.jkl,
                                                                 cv,
                                                                 mean,
                                                                 na.rm = TRUE),
                                   sd.l                 = tapply(df.intra.image$mu.jkl,
                                                                 cv,
                                                                 sd,
                                                                 na.rm = TRUE),
                                   rel.se.l             = tapply(df.intra.image$rel.se.jkl,
                                                                 cv,
                                                                 mean,
                                                                 na.rm = TRUE),
                                   sd.rel.se.l         = tapply(df.intra.image$rel.se.jkl,
                                                                cv,
                                                                sd,
                                                                na.rm = TRUE),
                                   str.l               = tapply(df.intra.image$str.jkl,
                                                                cv,
                                                                mean,
                                                                na.rm = TRUE),
                                   sd.str.l            = tapply(df.intra.image$str.jkl,
                                                                cv,
                                                                sd,
                                                                na.rm = TRUE),
                                   str.adj.l           = tapply(df.intra.image$str.adj.jkl,
                                                                cv,
                                                                mean,
                                                                na.rm = TRUE),
                                   sd.str.adj.l        = tapply(df.intra.image$str.adj.jkl,
                                                                cv,
                                                                sd,
                                                                na.rm = TRUE),
                                   lw.l                = tapply(df.intra.image$lw.jkl,
                                                                cv,
                                                                mean,
                                                                na.rm = TRUE),
                                   sd.lw.l             = tapply(df.intra.image$lw.jkl,
                                                                cv,
                                                                sd,
                                                                na.rm = TRUE),
                                   mean.theta.deg.l    = tapply(df.intra.image$mean.theta.deg.jkl,
                                                                cv,
                                                                mean,
                                                                na.rm = TRUE),
                                   sd.theta.deg.l      = tapply(df.intra.image$mean.theta.deg.jkl,
                                                                cv,
                                                                sd,
                                                                na.rm = TRUE),
                                   mean.theta.corr.deg.l= tapply(df.intra.image$theta.corr.deg.jkl,
                                                                 cv,
                                                                 mean,
                                                                 na.rm = TRUE),
                                   sd.theta.corr.deg.l  = tapply(df.intra.image$theta.corr.deg.jkl,
                                                                 cv,
                                                                 sd,
                                                                 na.rm = TRUE))


## Intra-detector fluctuation
cv             <- paste(df.intra.image$id)
df.intra.det   <- data.frame(id                   = tapply(df.intra.image$id,
                                                           cv,
                                                           unique),
                             LET.water.keV.um     = tapply(df.intra.image$LET.water.keV.um,
                                                           cv,
                                                           unique),
                             particle.name        = tapply(df.intra.image$particle.name,
                                                           cv,
                                                           unique),
                             n.tracks.kl          = tapply(df.intra.image$n.tracks.jkl,
                                                           cv,
                                                           mean,
                                                           na.rm = TRUE),
                             mu.kl                = tapply(df.intra.image$mu.jkl,
                                                           cv,
                                                           mean,
                                                           na.rm = TRUE),
                             sd.kl                = tapply(df.intra.image$mu.jkl,
                                                           cv,
                                                           sd,
                                                           na.rm = TRUE))

df.intra.det      <- merge(df.intra.det, df.LET, by = intersect(names(df.LET), names(df.intra.det)))
df.intra.det$ue.l <- df.intra.det$rel.se.l * df.intra.det$mu.kl / sqrt(df.intra.det$n.tracks.kl)
df.intra.det$se.P.kl <- sqrt(df.intra.det$sd.kl^2 + df.intra.det$ue.l^2)
df.intra.det$rel.se.P.kl <- df.intra.det$se.P.kl / df.intra.det$mu.kl

#summary(lm(rel.se.P.kl ~ LET.water.keV.um + id,
#           df.intra.det))

plot.tracks.influence.intra <- xyplot(sd.kl / ue.l ~ LET.water.keV.um,
                                      df.intra.det,
                                      grid      = TRUE,
                                      groups    = particle.name,
                                      ylab      = list("sd.kl / ue.l", font = 2, cex = 1.5),
                                      xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                                      main      = list("Influence intra-image var on intra-det var (track-based)", font = 2, cex = 1.5),
                                      scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))

plot.tracks.intra.det <- xyplot(rel.se.P.kl ~ LET.water.keV.um,
                                df.intra.det,
                                grid      = TRUE,
                                groups    = particle.name,
                                ylab      = list("sd.kl / ue.l", font = 2, cex = 1.5),
                                xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                                main      = list("Intra-detector variation (track-based)", font = 2, cex = 1.5),
                                scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))

intra.det.tracks <- mean(df.intra.det$rel.se.P.kl)


## Inter-detector fluctuation
cv             <- paste(df.intra.det$LET.water.keV.um)
df.inter.det   <- data.frame(LET.water.keV.um     = tapply(df.intra.det$LET.water.keV.um,
                                                           cv,
                                                           unique),
                             particle.name        = tapply(df.intra.det$particle.name,
                                                           cv,
                                                           unique),
                             n.detectors          = tapply(df.intra.det$id,
                                                           cv,
                                                           length),
                             mu.l                 = tapply(df.intra.det$mu.kl,
                                                           cv,
                                                           mean,
                                                           na.rm = TRUE),
                             sd.l                 = tapply(df.intra.det$mu.kl,
                                                           cv,
                                                           sd,
                                                           na.rm = TRUE))

df.inter.det$rel.se.l <- df.inter.det$sd.l / df.inter.det$mu.l
df.inter.det$rel.sd.D <- df.inter.det$rel.se.l

#summary(lm(rel.sd.D ~ LET.water.keV.um,
#           df.inter.det))

plot.tracks.inter.det <- xyplot(rel.sd.D ~ LET.water.keV.um,
                                df.inter.det,
                                grid      = TRUE,
                                groups    = particle.name,
                                ylab      = list("rel.sd.D", font = 2, cex = 1.5),
                                xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                                main      = list("Inter-detector variation (track-based)", font = 2, cex = 1.5),
                                scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))

inter.det.tracks <- mean(df.inter.det$rel.sd.D)


# Plots
plot.tracks.linewidth <- xYplot(Cbind(lw.l, 
                                      lw.l-sd.lw.l/sqrt(n.frames),
                                      lw.l+sd.lw.l/sqrt(n.frames)) ~ LET.water.keV.um,
                                df.LET,
                                panel     = function(...){
                                  panel.grid(h = -1, v = -1)
                                  panel.xYplot(...)
                                },
                                ylab      = list("rel. line-width", font = 2, cex = 1.5),
                                xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                                main      = list("line-width (track-based)", font = 2, cex = 1.5),
                                scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))

plot.tracks.intra.img <- xYplot(Cbind(rel.se.l, 
                                      rel.se.l-sd.rel.se.l/sqrt(n.frames),
                                      rel.se.l+sd.rel.se.l/sqrt(n.frames)) ~ LET.water.keV.um,
                                df.LET,
                                panel     = function(...){
                                  panel.grid(h = -1, v = -1)
                                  panel.xYplot(...)
                                },
                                ylab      = list("rel.sd. of track-mean intensities", font = 2, cex = 1.5),
                                xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                                main      = list("Intra-image variability (track-based)", font = 2, cex = 1.5),
                                scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))

plot.tracks.u.intra.img <- xyplot(sd.rel.se.l/rel.se.l ~ LET.water.keV.um,
                                  df.LET,
                                  grid      = TRUE,
                                  type      = c("p", "r"),
                                  ylab      = list("rel.sd.", font = 2, cex = 1.5),
                                  xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                                  main      = list("Linewidth of intra-image variability (track-based)", font = 2, cex = 1.5),
                                  scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))

plot.tracks.straggling <- xYplot(Cbind(str.l, 
                                       str.l-sd.str.l/sqrt(n.frames),
                                       str.l+sd.str.l/sqrt(n.frames)) ~ LET.water.keV.um,
                                 df.LET,
                                 grid      = TRUE,
                                 ylab      = list("rel.sd. of trackspot intensities", font = 2, cex = 1.5),
                                 xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                                 main      = list("Intensity straggling (track-based)", font = 2, cex = 1.5),
                                 scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))

plot.tracks.straggling.adj <- xYplot(Cbind(str.adj.l, 
                                           str.adj.l-sd.str.adj.l/sqrt(n.frames),
                                           str.adj.l+sd.str.adj.l/sqrt(n.frames)) ~ LET.water.keV.um,
                                     df.LET,
                                     grid      = TRUE,
                                     ylab      = list("rel.sd. of trackspot intensities", font = 2, cex = 1.5),
                                     xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                                     main      = list("Intensity straggling adjusted (track-based)", font = 2, cex = 1.5),
                                     scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))

plot.tracks.mean.theta <- xYplot(Cbind(mean.theta.deg.l, 
                                       mean.theta.deg.l-sd.theta.deg.l/sqrt(n.frames),
                                       mean.theta.deg.l+sd.theta.deg.l/sqrt(n.frames)) ~ LET.water.keV.um,
                                 df.LET,
                                 grid      = TRUE,
                                 ylab      = list("polar angle / deg", font = 2, cex = 1.5),
                                 xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                                 main      = list("Polar angle offset (track-based)", font = 2, cex = 1.5),
                                 scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))

plot.tracks.theta <- xYplot(Cbind(mean.theta.corr.deg.l, 
                                  mean.theta.corr.deg.l-sd.theta.corr.deg.l/sqrt(n.frames),
                                  mean.theta.corr.deg.l+sd.theta.corr.deg.l/sqrt(n.frames)) ~ LET.water.keV.um,
                            df.LET,
                            grid      = TRUE,
                            groups    = particle.name,
                            ylab      = list("polar angle / deg", font = 2, cex = 1.5),
                            xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                            main      = list("Mean polar angle (track-based)", font = 2, cex = 1.5),
                            scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5),
                            ylim      = c(-0.05, 1.45))

##### 3 LET-intensity relation #####

## Fit empirical log and loglog function
df.LET$llet       <- log10(df.LET$LET.water.keV.um)
df.LET$lmu.l      <- log10(df.LET$mu.l)

FUN.log           <- mu.l ~ b1 *( log10( LET.water.keV.um + b2) ) + b3
FUN.log.inv       <- LET.water.keV.um ~ 10^( (mu.l - b3) / b1) - b2
FUN.log.inv.deriv <- deriv(FUN.log.inv, c("b1", "b2", "b3"), func = TRUE)

LET.calib.log    <- nlsLM(FUN.log,
                          df.LET,
                          start = list(b1 = 2.5, b2 = 3, b3 = -1),
                          trace = FALSE)
v.log             <- vcov(LET.calib.log)


FUN.smp           <- lmu.l ~ llet
FUN.smp.inv       <- llet ~ (lmu.l - b1) / b2
FUN.smp.inv.deriv <- deriv(FUN.smp.inv, c("b1", "b2"), func = TRUE)

LET.calib.smp     <- lm(lmu.l ~ llet,
                        df.LET)
v.smp             <- vcov(LET.calib.smp)



## Get CIs using delta methode
df.fit <- data.frame(llet = seq( min(df.LET$llet)-0.25, 
                                 max(df.LET$llet)+0.25, 
                                 length.out = 200))
df.fit$LET.water.keV.um <- 10^df.fit$llet

df.fit$mu.l.log <- predict(LET.calib.log,    newdata = df.fit)
df.fit$mu.l.smp <- 10^predict(LET.calib.smp, newdata = df.fit)

mu.l            <- df.fit$mu.l.log
grad.log        <- attr(FUN.log.inv.deriv( b1 = coef(LET.calib.log)[1], 
                                           b2 = coef(LET.calib.log)[2], 
                                           b3 = coef(LET.calib.log)[3]),
                        "gradient")
var.log         <- apply(grad.log, 1, function(x){x <- matrix(x, ncol = 3)
x %*% v.log %*% t(x)})

lmu.l           <- log10(mu.l)
grad.smp        <- attr(FUN.smp.inv.deriv( b1 = coef(LET.calib.smp)[1], 
                                           b2 = coef(LET.calib.smp)[2]),
                        "gradient")
var.smp         <- apply(grad.smp, 1, function(x){x <- matrix(x, ncol = 2)
x %*% v.smp %*% t(x)})

# 1 sigma: quantile = 0.682689492
# 2 sigma: quantile = 0.954499736
# 3 sigma: quantile = 0.997300204
quantile      <- 0.682689492
df.fit$ci.log <- qt(quantile, nrow(df.LET)-length(coef(LET.calib.log))) * sqrt(var.log)
df.fit$ci.smp <- qt(quantile, nrow(df.LET)-length(coef(LET.calib.smp))) * sqrt(var.smp)


## Plots
plot.tracks.LET.I.log <- xYplot(Cbind(mu.l, 
                                      mu.l-sd.l/sqrt(n.frames),
                                      mu.l+sd.l/sqrt(n.frames)) ~ LET.water.keV.um,
                                df.LET,
                                panel     = function(...){
                                  panel.grid(h = -1, v = -1)
                                  panel.xyplot(df.fit$llet, 
                                               df.fit$mu.l.log, 
                                               type = "l",
                                               col  = "darkgoldenrod4")
                                  panel.xyplot(df.fit$llet, 
                                               df.fit$mu.l.smp, 
                                               type = "l",
                                               col  = "black")
                                  panel.xYplot(...)
                                },
                                ylab      = list("intensity", font = 2, cex = 1.5),
                                xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                                ylim      = c(-0.2, max(df.LET$mu.l)*1.15),
                                groups    = particle.name,
                                main      = list("LET-intensity relation (track-based)", font = 2, cex = 1.5),
                                scales    = list(x = list(log = 10, equispaced.log = FALSE), 
                                                 cex = 1.5))

plot.tracks.LET.I.loglog <- xyplot(mu.l ~ LET.water.keV.um,
                                   df.LET,
                                   panel     = function(...){
                                     panel.grid(h = -1, v = -1)
                                     panel.xyplot(df.fit$llet, 
                                                  log10(df.fit$mu.l.log), 
                                                  type = "l",
                                                  col  = "darkgoldenrod4")
                                     panel.xyplot(df.fit$llet, 
                                                  log10(df.fit$mu.l.smp), 
                                                  type = "l",
                                                  col  = "black")
                                     panel.xyplot(...)
                                   },
                                   ylab      = list("intensity", font = 2, cex = 1.5),
                                   xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                                   ylim      = c(-0.2, max(df.LET$mu.l)*1.15),
                                   groups    = particle.name,
                                   main      = list("LET-intensity relation (track-based)", font = 2, cex = 1.5),
                                   scales    = list(log = 10, equispaced.log = FALSE, cex = 1.5))

plot.tracks.I.LET.log <- xyplot(LET.water.keV.um ~ mu.l,
                                df.LET,
                                panel     = function(...){
                                  panel.grid(h = -1, v = -1)
                                  panel.xyplot(df.fit$mu.l.log, 
                                               df.fit$llet, 
                                               type = "l",
                                               col  = "darkgoldenrod4")
                                  panel.xyplot(df.fit$mu.l.log, 
                                               log10(df.fit$LET.water.keV.um - df.fit$ci.log), 
                                               type = "l",
                                               col  = "darkgoldenrod4",
                                               lty  = 2)
                                  panel.xyplot(df.fit$mu.l.log, 
                                               log10(df.fit$LET.water.keV.um + df.fit$ci.log), 
                                               type = "l",
                                               col  = "darkgoldenrod4",
                                               lty  = 2)
                                  panel.xyplot(df.fit$mu.l.smp, 
                                               df.fit$llet, 
                                               type = "l",
                                               col  = "black")
                                  panel.xyplot(df.fit$mu.l.smp, 
                                               log10(df.fit$LET.water.keV.um - 10^df.fit$ci.smp), 
                                               type = "l",
                                               col  = "black",
                                               lty  = 2)
                                  panel.xyplot(df.fit$mu.l.smp, 
                                               log10(df.fit$LET.water.keV.um + 10^df.fit$ci.smp), 
                                               type = "l",
                                               col  = "black",
                                               lty  = 2)
                                  panel.xyplot(...)
                                },
                                xlab      = list("intensity", font = 2, cex = 1.5),
                                ylab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                                xlim      = c(-0.2, max(df.LET$mu.l)*1.15),
                                groups    = particle.name,
                                main      = list("Intensity-LET relation (track-based)", font = 2, cex = 1.5),
                                scales    = list(y = list(log = 10, equispaced.log = FALSE), 
                                                 cex = 1.5))

plot.tracks.I.LET.loglog <- xyplot(LET.water.keV.um ~ mu.l,
                                   df.LET,
                                   panel     = function(...){
                                     panel.grid(h = -1, v = -1)
                                     panel.xyplot(log10(df.fit$mu.l.log), 
                                                  df.fit$llet, 
                                                  type = "l",
                                                  col  = "darkgoldenrod4")
                                     panel.xyplot(log10(df.fit$mu.l.log), 
                                                  log10(df.fit$LET.water.keV.um - df.fit$ci.log), 
                                                  type = "l",
                                                  col  = "darkgoldenrod4",
                                                  lty  = 2)
                                     panel.xyplot(log10(df.fit$mu.l.log), 
                                                  log10(df.fit$LET.water.keV.um + df.fit$ci.log), 
                                                  type = "l",
                                                  col  = "darkgoldenrod4",
                                                  lty  = 2)
                                     panel.xyplot(log10(df.fit$mu.l.smp), 
                                                  df.fit$llet, 
                                                  type = "l",
                                                  col  = "black")
                                     panel.xyplot(log10(df.fit$mu.l.smp), 
                                                  log10(df.fit$LET.water.keV.um - 10^df.fit$ci.smp), 
                                                  type = "l",
                                                  col  = "black",
                                                  lty  = 2)
                                     panel.xyplot(log10(df.fit$mu.l.smp), 
                                                  log10(df.fit$LET.water.keV.um + 10^df.fit$ci.smp), 
                                                  type = "l",
                                                  col  = "black",
                                                  lty  = 2)
                                     panel.xyplot(...)
                                   },
                                   xlab      = list("intensity", font = 2, cex = 1.5),
                                   ylab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                                   xlim      = c(-0.2, max(df.LET$mu.l)*1.15),
                                   groups    = particle.name,
                                   main      = list("Intensity-LET relation (track-based)", font = 2, cex = 1.5),
                                   scales    = list(log = 10, equispaced.log = FALSE, cex = 1.5))


plot.tracks.CI.LET <- xyplot( I(ci.log/LET.water.keV.um*100) +
                                I(10^ci.smp/LET.water.keV.um*100)~ LET.water.keV.um,
                              df.fit,
                              grid      = TRUE,
                              type      = "l",
                              col       = c("darkgoldenrod4", "black"),
                              xlab      = list("LET / (keV/um)", font = 2, cex = 1.5),
                              ylab      = list("rel. uncertainty / %", font = 2, cex = 1.5),
                              main      = list("Intensity-LET relation (track-based)", font = 2, cex = 1.5),
                              scales    = list(x = list(log = 10, equispaced.log = FALSE), cex = 1.5))

# Compile and return results
FNTD.results <- c(FNTD.results,
                  list(calibration       = list(log.log         = LET.calib.smp,
                                                empirical.log   = LET.calib.log),
                       calibration.plots = list(plot.slice.influence.intra = plot.slice.influence.intra,
                                                plot.slice.intra.det       = plot.slice.intra.det,
                                                plot.slice.inter.det       = plot.slice.inter.det,
                                                plot.slice.LET.I           = plot.slice.LET.I,
                                                plot.slice.u.LET.I         = plot.slice.u.LET.I,
                                                plot.slice.intra.img       = plot.slice.intra.img,
                                                plot.slice.u.intra.img     = plot.slice.u.intra.img,
                                                plot.tracks.influence.intra = plot.tracks.influence.intra,
                                                plot.tracks.intra.det       = plot.tracks.intra.det,
                                                plot.tracks.inter.det       = plot.tracks.inter.det,
                                                plot.tracks.linewidth       = plot.tracks.linewidth,
                                                plot.tracks.intra.img       = plot.tracks.intra.img,
                                                plot.tracks.u.intra.img     = plot.tracks.u.intra.img,
                                                plot.tracks.straggling      = plot.tracks.straggling,
                                                plot.tracks.straggling.adj  = plot.tracks.straggling.adj,
                                                plot.tracks.mean.theta      = plot.tracks.mean.theta,
                                                plot.tracks.theta           = plot.tracks.theta,
                                                plot.tracks.LET.I.log       = plot.tracks.LET.I.log,
                                                plot.tracks.LET.I.loglog    = plot.tracks.LET.I.loglog,
                                                plot.tracks.I.LET.log       = plot.tracks.I.LET.log,
                                                plot.tracks.I.LET.loglog    = plot.tracks.I.LET.loglog,
                                                plot.tracks.CI.LET          = plot.tracks.CI.LET)))
