require(Hmisc)

message("###########################\n...analysing sample results...\n")

# Make temporary copy of track data
df.tracks <- FNTD.results$data$df.tracks


## Apply calibration
if(FNTD.get.option(my.options, "calibration.function") == "log.log"){
  calcoefs <- FNTD.get.option(my.options, "calibration.coefs")$log.log
  df.tracks$LET.keV.um <- 10^((log10(df.tracks$mean.intensity)-calcoefs[1])/calcoefs[2])
}

## Compute fluence and dose
df.tracks$fluence.cm2 <- df.tracks$l.um / df.tracks$v.um3 * 1e8
df.tracks$dose.Gy     <- df.tracks$LET.keV.um * 10 * df.tracks$fluence.cm2 * 1.6e-10


## Summaries results for each frame in each sample
cv  <- paste(df.tracks$id, df.tracks$frame.no)
tmp <- data.frame(id = tapply(df.tracks$id,
                              cv,
                              unique),
                  particle.name = tapply(df.tracks$particle.name,
                                         cv,
                                         unique),
                  frame.no = tapply(df.tracks$frame.no,
                                    cv,
                                    unique),
                  n.tracks = tapply(df.tracks$track.no,
                                    cv,
                                    length),
                  fluence.cm2 = tapply(df.tracks$fluence.cm2,
                                       cv,
                                       sum,
                                       na.rm = TRUE),
                  mean.intensity = tapply(df.tracks$mean.intensity,
                                          cv,
                                          mean,
                                          na.rm = TRUE),
                  sd.intensity = tapply(df.tracks$mean.intensity,
                                          cv,
                                          sd,
                                          na.rm = TRUE),
                  LET.keV.um = tapply(df.tracks$LET.keV.um,
                                      cv,
                                      mean,
                                      na.rm = TRUE),
                  dose.Gy     = tapply(df.tracks$dose.Gy,
                                       cv,
                                       sum,
                                       na.rm = TRUE))

tt   <- FNTD.get.option(FNTD.results$options, "image.size")
tt2  <- FNTD.get.option(FNTD.results$options, "voxel.size")


tmp$p.fluence.cm2 <- tmp$n.tracks / (tt$n.x*tt2$x.um * tt$n.y*tt2$y.um) * 1e8

## Summaries results for each sample (id)
cv   <- tmp$id
tmp2 <- data.frame(id = tapply(tmp$id,
                               cv,
                               unique),
                   particle.name = tapply(tmp$particle.name,
                                          cv,
                                          unique),
                   n.frames = tapply(tmp$frame.no,
                                     cv,
                                     length),
                   n.tracks = tapply(tmp$n.tracks,
                                     cv,
                                     mean,
                                     na.rm = TRUE),
                   fluence.cm2 = tapply(tmp$fluence.cm2,
                                        cv,
                                        mean,
                                        na.rm = TRUE),
                   p.fluence.cm2 = tapply(tmp$p.fluence.cm2,
                                        cv,
                                        mean,
                                        na.rm = TRUE),
                   mean.intensity  = tapply(tmp$mean.intensity,
                                        cv,
                                        mean,
                                        na.rm = TRUE),
                   sd.intensity  = tapply(tmp$sd.intensity,
                                            cv,
                                            mean,
                                            na.rm = TRUE),
                   LET.keV.um  = tapply(tmp$LET.keV.um,
                                        cv,
                                        mean,
                                        na.rm = TRUE),
                   sd.LET.keV.um  = tapply(tmp$LET.keV.um,
                                           cv,
                                           sd,
                                           na.rm = TRUE),
                   dose.Gy     = tapply(tmp$dose.Gy,
                                        cv,
                                        mean,
                                        na.rm = TRUE),
                   sd.dose.Gy     = tapply(tmp$dose.Gy,
                                           cv,
                                           sd,
                                           na.rm = TRUE))

## Plots of distributions
plot.intensity.histo <- histogram(~mean.intensity|sprintf("id: %s", id),
                                  df.tracks,
                                  breaks   = 50,
                                  as.table = TRUE,
                                  layout   = c(5,3),
                                  type     = "count",
                                  main     = paste0("Fluorescence intensity"))

plot.theta.histo <- histogram(~theta.corr.deg|sprintf("id: %s", id),
                              df.tracks,breaks   = 50,
                              as.table = TRUE,
                              layout   = c(5,3),
                              scales = "free",
                              main     = paste0("Polar angles"))

max.L       <- max(df.tracks$LET.keV.um, na.rm = TRUE)
min.L       <- min(df.tracks$LET.keV.um, na.rm = TRUE)
tmp3        <- split(df.tracks, df.tracks$id)
df.hist     <- lapply(tmp3,
                      function(x){
                      tmp.h <- hist(log10(x$LET.keV.um),
                                    breaks = seq(log10(min.L*0.98), log10(max.L*1.02), length.out = 100),
                                    plot   = FALSE)
                      return(data.frame(id = unique(x$id),
                                        mids = 10^tmp.h$mids,
                                        counts = tmp.h$counts,
                                        density = tmp.h$density))})
df.hist     <- do.call("rbind", df.hist)
df.hist     <- df.hist[order(df.hist$id),]
plot.LET.histo <- xyplot(counts~mids|sprintf("id: %s", id),
                            df.hist,
                            type     = "h",
                            main     = paste0("LET in water"),
                            grid     = TRUE,
                            xlim     = c(0.3, 500),
                            scales   = list(log = 10, equispaced.log = FALSE))

plot.LET.histo <- histogram(~LET.keV.um|sprintf("id: %s", id),
                            df.tracks,
                            breaks   = 50,
                            as.table = TRUE,
                            layout   = c(5,3),
                            type     = "count",
                            main     = paste0("LET in water"),
                            panel    = function(...){
                              panel.grid(h = -1, v = -1)
                              panel.histogram(...)
                            },
                            scales   = list( x = list(log = 10, equispaced.log = FALSE)))


## Plots of single quantities
plot.tracks <- xyplot( n.tracks ~ factor(id),
                       tmp,
                       grid   = TRUE,
                       pch    = 3,
                       groups    = particle.name,
                       ylab      = list("number of tracks in image", font = 2, cex = 1.5),
                       xlab      = list("sample id", font = 2, cex = 1.5),
                       main      = list("Tracks", font = 2, cex = 1.5),
                       scales    = list(cex = 1.5, x = list(rot = 90)))

plot.mean.intensity <- xyplot( mean.intensity ~ factor(id),
                       tmp,
                       grid   = TRUE,
                       pch    = 3,
                       groups    = particle.name,
                       ylab      = list("mean intensity", font = 2, cex = 1.5),
                       xlab      = list("sample id", font = 2, cex = 1.5),
                       main      = list("Mean intensity", font = 2, cex = 1.5),
                       scales    = list(cex = 1.5, x = list(rot = 90)))

plot.sd.intensity <- xyplot( sd.intensity ~ factor(id),
                          tmp,
                          grid   = TRUE,
                          pch    = 3,
                          groups    = particle.name,
                          ylab      = list("intensity sd", font = 2, cex = 1.5),
                          xlab      = list("sample id", font = 2, cex = 1.5),
                          main      = list("Mean intensity", font = 2, cex = 1.5),
                          scales    = list(cex = 1.5, x = list(rot = 90)))

plot.fluence <- xyplot( fluence.cm2 ~ factor(id),
                        tmp,
                        grid   = TRUE,
                        pch    = 3,
                        groups    = particle.name,
                        ylab      = list("particle fluence / (1/cm2)", font = 2, cex = 1.5),
                        xlab      = list("sample id", font = 2, cex = 1.5),
                        main      = list("Fluence", font = 2, cex = 1.5),
                        scales    = list(cex = 1.5, x = list(rot = 90)))

plot.p.fluence <- xyplot( p.fluence.cm2 ~ factor(id),
                        tmp,
                        grid   = TRUE,
                        pch    = 3,
                        groups    = particle.name,
                        ylab      = list("planar particle fluence / (1/cm2)", font = 2, cex = 1.5),
                        xlab      = list("sample id", font = 2, cex = 1.5),
                        main      = list("Planar fluence", font = 2, cex = 1.5),
                        scales    = list(cex = 1.5, x = list(rot = 90)))

tmp2$id.no <- 1:length(tmp2$id)
plot.LET <- xYplot( Cbind(LET.keV.um, 
                          LET.keV.um-sd.LET.keV.um/sqrt(n.frames),
                          LET.keV.um+sd.LET.keV.um/sqrt(n.frames)) ~ id.no,
                    tmp2,
                    panel = function(...){
                      panel.grid(h = -1, v = -1)
                      panel.abline(h = 0, lty = 2)
                      panel.xYplot(...)
                    },  
                    pch    = 3,
                    groups    = particle.name,
                    ylab      = list("fluence-weighted LET in water / (keV/um)", font = 2, cex = 1.5),
                    xlab      = list("sample id", font = 2, cex = 1.5),
                    main      = list("LET", font = 2, cex = 1.5),
                    scales    = list(cex = 1.5, x = list(rot = 90)),
                    ylim      = c(-2.5, 52.5))

plot.dose <- xYplot( Cbind(dose.Gy, 
                           dose.Gy-sd.dose.Gy/sqrt(n.frames),
                           dose.Gy+sd.dose.Gy/sqrt(n.frames)) ~ id.no,
                     tmp2,
                     panel = function(...){
                       panel.grid(h = -1, v = -1)
                       panel.xYplot(...)
                     },  
                     pch    = 3,
                     groups    = particle.name,
                     ylab      = list("dose / Gy", font = 2, cex = 1.5),
                     xlab      = list("sample id", font = 2, cex = 1.5),
                     main      = list("Dose", font = 2, cex = 1.5),
                     scales    = list(cex = 1.5, x = list(rot = 90)),
                     ylim      = c(-0.05, 1.05))


# Compile and return results
df.meta <- merge(FNTD.results$data$df.meta, 
                 tmp2, 
                 by = intersect(names(FNTD.results$data$df.meta), 
                                names(tmp2)))

FNTD.results$data$df.meta   <- df.meta
FNTD.results$data$df.tracks <- df.tracks
FNTD.results <- c(FNTD.results,
                  list(analysis.plots = list(plot.intensity.histo = plot.intensity.histo,
                                             plot.theta.histo     = plot.theta.histo,
                                             plot.LET.histo       = plot.LET.histo,
                                             plot.tracks          = plot.tracks,
                                             plot.fluence         = plot.fluence,
                                             plot.p.fluence       = plot.p.fluence,
                                             plot.mean.intensity  = plot.mean.intensity,
                                             plot.sd.intensity    = plot.sd.intensity,
                                             plot.LET             = plot.LET,
                                             plot.dose            = plot.dose)))