rm(list = ls())
cat("\014")
options(java.parameters = "-Xmx4g")

require(FNTD)

# Read image
jimg <- FNTD.read.image(system.file("extdata", "ivana_0.5Gy_FXR_modified.tif", package = "FNTD"))
#jimg <- FNTD.read.image("D:/workspaces/R/Flora/ID_surface_flat_background/1117_modified.tif")
# jimg <- FNTD.read.image("C:/Users/kouwenbe/Downloads/329subset.tif")

# Flatfield, axial and background correction
jimg <- FNTD.apply.flatfield(jimg, method="function", parameters = list(x0  = 238.9,
                                                                        y0  = 352.4,
                                                                        a.c = -1.158e-6,
                                                                        b.c = 5.02e-5,
                                                                        mu  = 0.950))
# jimg <- FNTD.apply.axial.correction(jimg)
jimg <- FNTD.normalize.stack(jimg, 7)
jimg <- jimg - FNTD.get.background(jimg, sliding.window.length = 30)

# Find track spots ("tracking") using the point feature extraction
jimg <- FNTD.find.trackspots(jimg, 
                             radius     = 3, 
                             cutoff     = 0,
                             percentile = 5)

# Find ("link") tracks = trajectories
jimg <- FNTD.find.tracks(jimg, 
                         dynamics           = "Constant velocity", 
                         radius             = 3, 
                         link.range         = 2, 
                         displacement       = 10,
                         weight.distance    = 1, 
                         weight.features    = 1, 
                         weight.dynamics    = 1, 
                         max_angle          = 40, 
                         angle_history      = 3,
                         relink             = TRUE, 
                         relink.cost_factor = .2, 
                         relink.distance    = 50)

# Tilt tracks and filter them
jimg <- FNTD.get.tracks.of.length(jimg, min.length = 3)

# Check results visually
FNTD.show.image(jimg, showTracks = TRUE)

# Get information on trajectory intensities and orientations
tracks      <- rbind(FNTD.get.track.properties(jimg, 3, method="Gauss",
                                         voxel.size.x.um = 1, 
                                         voxel.ratio.zx = 500/131.79, 
                                         ASF = 1.2),
               FNTD.get.track.properties(jimg, 3, method="maximum",
                                         voxel.size.x.um = 1, 
                                         voxel.ratio.zx = 500/131.79, 
                                         ASF = 1.2),
               FNTD.get.track.properties(jimg, 3, method="adjacent",
                                         voxel.size.x.um = 1, 
                                         voxel.ratio.zx = 500/131.79, 
                                         ASF = 1.2))
histogram(~mean.intensity|amplitude,
          tracks,
          breaks = 50,
          main   = "Track mean intensity",
          xlab   = "intensity / a.u.",
          layout = c(1,3))

tracks.cor  <- FNTD.apply.polar.correction(tracks, 370, 100)


histogram(~mean.intensity|amplitude,
          tracks.cor,
          breaks = 50,
          main   = "Track mean intensity",
          xlab   = "intensity / a.u.",
          layout = c(1,3))
# Convert intensity to LET
LET <- function(I, a, b, c){
  10^((I-c)/a)-b
}

a <- 3.43
b <- 9.3
c <- -3.2

df <- data.frame(I = 0:5,
                 L = LET(0:5,a,b,c))
xyplot(I~log10(L),df)

# Plot angular distribution
histogram(~phi.deg,
          tracks,
          breaks = 50,
          main   = "azimuthal angle as-is",
          xlab   = "phi / deg")

histogram(~phi.corr.deg,
          tracks,
          breaks = 50,
          main   = "azimuthal angle corrected",
          xlab   = "phi / deg")

histogram(~theta.deg,
          tracks,
          breaks = 50,
          main   = "polar angle as-is",
          xlab   = "theta / deg")

histogram(~theta.corr.deg,
          tracks,
          breaks = 50,
          main   = "polar angle  corrected",
          xlab   = "theta / deg")


# Plot intensities
histogram(~mean.intensity,
          tracks,
          breaks = 50,
          main   = "Track mean intensity",
          xlab   = "intensity / a.u.")
      
histogram(~rel.sd.intensity,
          tracks,
          breaks = 50,
          main   = "Relative intensity straggling",
          xlab   = "rel. straggling")

histogram(~rel.sd.intensity.adj,
          tracks,
          breaks = 50,
          main   = "Relative intensity straggling (adjusted)",
          xlab   = "rel. straggling")

# Straggling vs. intensity
xyplot(rel.sd.intensity.adj ~ mean.intensity,
          tracks,
          main   = "Straggling vs. amplitude",
          xlab   = "mean track intensity / a.u.",
          ylab   = "relative intensity straggling",
          ylim   = c(-0.025, 0.425))

# Polar angle vs. intensity
xyplot(theta.corr.deg ~ mean.intensity,
       tracks,
       main   = "Scattering vs. amplitude",
       xlab   = "mean track intensity / a.u.",
       ylab   = "corrected polar angle / deg")