rm(list = ls())

library(libamtrack)
library(lattice)

df  <-  expand.grid( E.MeV.u  = 10^seq(log10(1), log10(1000), length.out = 30),
                     particle.no = c(1001, 2004, 4008, 6012, 8016))

res <- lapply(1:nrow(df), function(i){
           res <- AT.run.CPPSC.method(E.MeV.u          = df$E.MeV.u[i], 
                              particle.no              = df$particle.no[i],
                              fluence.cm2.or.dose.Gy   = .001,
                              material.no              = 1,
                              stopping.power.source.no = 3,
                              rdd.model                = 3,
                              rdd.parameters           = c(5e-8, 0, 0, 0),
                              er.model                 = 5,
                              gamma.model              = 4,
                              gamma.parameters         = c(1, 80, rep(0.0, 7)),
                              N2                       = 10,
                              fluence.factor           = 1,
                              write.output             = TRUE,
                              shrink.tails             = TRUE,
                              shrink.tails.under       = 1e-50,
                              adjust.N2                = FALSE,
                              lethal.events.mode       = FALSE)
           df.f <- read.table("CPPSC_detailed.log",
                            header = TRUE,
                            sep    = ";")
           df.f$E.MeV.u <- df$E.MeV.u[i]
           df.f$particle.no <- df$particle.no[i]
           return(list(res = res, df.f = df.f))})

df.res   <- do.call("rbind", lapply(1:nrow(df),
                                    function(i){
                                      data.frame(E.MeV.u = df$E.MeV.u[i],
                                                 particle.no = df$particle.no[i],
                                                 n.convolutions = res[[i]]$res$n.convolutions,
                                                 efficiency     = res[[i]]$res$relative.efficiency,
                                                 d.check.Gy     = res[[i]]$res$d.check,
                                                 S.gamma        = res[[i]]$res$S.gamma,
                                                 S.HCP          = res[[i]]$res$S.HCP)
                                    }))

df.f     <- do.call("rbind", lapply(1:nrow(df),
                                    function(i){
                                      res[[i]]$df.f
                                    }))
  
df.res$LET.water.keV.um <- AT.Stopping.Power.with.no(stopping.power.source.no = 3,
                                                  E.MeV.u = df.res$E.MeV.u,
                                                  particle.no = df.res$particle.no,
                                                  material.no = 1)$stopping.power.keV.um
df.f$LET.water.keV.um   <- AT.Stopping.Power.with.no(stopping.power.source.no = 3,
                                                     E.MeV.u = df.f$E.MeV.u,
                                                     particle.no = df.f$particle.no,
                                                     material.no = 1)$stopping.power.keV.um

plot(xyplot( efficiency ~ LET.water.keV.um,
        df.res,
        grid   = TRUE,
        type   = "p",
        groups = particle.no,
        scales = list( log = 10, equispaced.log = FALSE)))

plot(xyplot( efficiency ~ LET.water.keV.um,
        df.res,
        grid   = TRUE,
        type   = "p",
        groups = particle.no,
        scales = list( x = list(log = 10, equispaced.log = FALSE))))

plot(xyplot( d.check.Gy ~ LET.water.keV.um,
        df.res,
        grid   = TRUE,
        type   = "p",
        groups = particle.no,
        scales = list( log = 10, equispaced.log = FALSE)))

if(F){
  xyplot( S.gamma ~ LET.water.keV.um,
        df.res,
        grid   = TRUE,
        type   = "p",
        groups = particle.no,
        scales = list( log = 10, equispaced.log = FALSE))

xyplot( S.HCP ~ LET.water.keV.um,
        df.res,
        grid   = TRUE,
        type   = "p",
        groups = particle.no,
        scales = list( log = 10, equispaced.log = FALSE))
}

plot(xyplot( efficiency*LET.water.keV.um ~ LET.water.keV.um,
        df.res,
        grid   = TRUE,
        type   = "p",
        groups = particle.no,
        scales = list( x = list(log = 10, equispaced.log = FALSE))))

plot(xyplot( efficiency*LET.water.keV.um ~ LET.water.keV.um,
        df.res,
        grid   = TRUE,
        type   = "p",
        groups = particle.no,
        scales = list( log = 10, equispaced.log = FALSE)))

if(F){
  xyplot( f ~ d.Gy| sprintf("E = %03d", as.integer(E.MeV.u)) * 
          sprintf("%s",AT.particle.name.from.particle.no(particle.no)),
        df.f,
        grid    = TRUE,
        type    = "S",
        as.table = TRUE,
        layout  = c(5,3),
        scales  = list( log = 10, equispace.log = FALSE))}

lm(I(log10(efficiency*LET.water.keV.um)) ~ I(log10(LET.water.keV.um)),
   df.res)