message("###########################\n Start, compiling meta data...\n")

# Collect arguments and get paths
args                   <- commandArgs(trailingOnly = TRUE)

if(length(args) >= 1){
  my.options <- FNTD.set.option(my.options, "use.tracking.results", as.logical(args[1]))
}

files        <- list.files( path       = FNTD.get.option(my.options, "data.path"),
                            recursive  = TRUE,
                            pattern    = "*.tif|*.zip",
                            full.names = TRUE)
df.files     <- data.frame(id   = substr(gsub(".tif", "", basename(files)), 
                                         1, 
                                         FNTD.get.option(my.options, "id.length")),
                           file = files,
                           stringsAsFactors = FALSE)

df.meta         <- read.xlsx(file.path(FNTD.get.option(my.options, "data.path"), 
                                       FNTD.get.option(my.options, "meta.data.file")), 
                             1,
                             stringsAsFactors = FALSE)
df.meta         <- merge(df.meta, df.files, by = "id")

df.tracking.par <- read.xlsx(file.path(FNTD.get.option(my.options, "data.path"), 
                                       FNTD.get.option(my.options, "tracking.par.file")), 
                             1,
                             stringsAsFactors = FALSE)

# Check if tracking data fit data set
if(!all(df.meta$id%in%df.tracking.par$id)){
  stop("At least one sample id not found in the tracking parameters.")
}

# Merge meta and tracking data
df.meta   <- merge(df.meta, df.tracking.par, by = "id")

# Add LET
df.meta$LET.water.keV.um <- NA
ii                       <- is.na(df.meta$E.MeV.u) | df.meta$E.MeV.u == "NA"
if(sum(!ii)>=1){
  df.meta$LET.water.keV.um[!ii] <- AT.Stopping.Power(stopping.power.source = "ICRU", 
                                                  E.MeV.u = df.meta$E.MeV.u[!ii], 
                                                  particle.no = AT.particle.no.from.particle.name(df.meta$particle.name[!ii]), 
                                                  material.no = AT.material.no.from.material.name("Water, Liquid"))$stopping.power.keV.um
}

if(FNTD.get.option(my.options, "use.tracking.results")){
  load(file.path(FNTD.get.option(my.options, "data.path"), 
                 "df.meta.Rda"))
}else{
  save(df.meta, file = file.path(FNTD.get.option(my.options, "data.path"), 
                                 "df.meta.Rda"))
}
