message("###########################\n...correcting for FOV non-uniformity...\n")

df.trackspots.n3 <- df.trackspots.n2

# Slice and Frame-wise tracks
if(FNTD.get.option(my.options, "correction.FOV.uniform") == "Slice and Frame-wise tracks"){
  
  cv <- paste(df.trackspots.n2$id, df.trackspots.n2$frame.no, df.trackspots.n2$slice.no)
  df.norm.FOV <- list()
  i  <- 1
  for (cur.cv in unique(cv)){
    # cur.cv <- unique(cv)[1]
    ii <- cur.cv == cv
    mod <- lm(amplitude ~ poly(x, 2) + poly(y, 2),
              df.trackspots.n2[ii,])
    jj <- is.na(df.trackspots.n2$amplitude[ii])
    df.trackspots.n3$amplitude[ii]      <- NA
    df.trackspots.n3$amplitude[ii][!jj] <- df.trackspots.n2$amplitude[ii][!jj] / predict(mod) * mean(predict(mod))
    
    df.norm.FOV[[i]]  <- list(id       = unique(df.trackspots.n3$id[ii]),
                              frame.no = unique(df.trackspots.n3$frame.no[ii]),
                              slice.no = unique(df.trackspots.n3$slice.no[ii]),
                              model    = mod,
                              stdev    = sd(predict(mod)) / mean(predict(mod)))
    i <- i+1
  }
}

# Frame-wise tracks
if(FNTD.get.option(my.options, "correction.FOV.uniform") == "Frame-wise tracks"){
  
  cv <- paste(df.trackspots.n2$id, df.trackspots.n2$frame.no)
  df.norm.FOV <- list()
  i  <- 1
  for (cur.cv in unique(cv)){
    # cur.cv <- unique(cv)[1]
    ii <- cur.cv == cv
    mod <- lm(amplitude ~ poly(x, 2) + poly(y, 2),
              df.trackspots.n2[ii,])
    jj <- is.na(df.trackspots.n2$amplitude[ii])
    df.trackspots.n3$amplitude[ii]      <- NA
    df.trackspots.n3$amplitude[ii][!jj] <- df.trackspots.n2$amplitude[ii][!jj] / predict(mod) * mean(predict(mod))
    
    df.norm.FOV[[i]]  <- list(id       = unique(df.trackspots.n3$id[ii]),
                              frame.no = unique(df.trackspots.n3$frame.no[ii]),
                              slice.no = rep("all", length(unique(df.trackspots.n3$id[ii]))),
                              model    = mod,
                              stdev    = sd(predict(mod)) / mean(predict(mod)))
    i <- i+1
  }
}


# Sample-wise tracks
if(FNTD.get.option(my.options, "correction.FOV.uniform") == "Sample-wise tracks"){
  
  cv <- paste(df.trackspots.n2$id)
  df.norm.FOV <- list()
  i <- 1
  for (cur.cv in unique(cv)){
    # cur.cv <- unique(cv)[1]
    ii <- cur.cv == cv
    mod <- lm(amplitude ~ poly(x, 2) + poly(y, 2),
              df.trackspots.n2[ii,])
    jj <- is.na(df.trackspots.n2$amplitude[ii])
    df.trackspots.n3$amplitude[ii]      <- NA
    df.trackspots.n3$amplitude[ii][!jj] <- df.trackspots.n2$amplitude[ii][!jj] / predict(mod) * mean(predict(mod))
    
    df.norm.FOV[[i]] <- list(id       = unique(df.trackspots.n3$id[ii]),
                             model    = mod,
                             frame.no = rep("all", length(unique(df.trackspots.n3$id[ii]))),
                             slice.no = rep("all", length(unique(df.trackspots.n3$id[ii]))),
                             stdev    = sd(predict(mod)) / mean(predict(mod)))
    i <- i + 1
  }
}

df.norm.factor.FOV <- do.call("rbind",
                              lapply(df.norm.FOV,
                                     function(x){ data.frame(id = x$id, 
                                                             frame.no = x$frame.no,
                                                             slice.no = x$slice.no,
                                                             stdev = x$stdev)}))
df.fun.FOV         <- do.call("rbind",
                              lapply(df.norm.FOV,
                                     function(x){ df <- expand.grid(id = x$id, 
                                                              frame.no = x$frame.no,
                                                              slice.no = x$slice.no,
                                                              x        = seq(1,504, by = 20),
                                                              y        = seq(1,504, by = 20))
                                                  df$factor <- predict(x$model, 
                                                                       newdata = df)
                                                  df}))

if(F){
  levelplot(factor ~ x*y|sprintf("s: %s", slice.no)*sprintf("f: %s", frame.no)*id,
          df.fun.FOV,
          aspect = 1,
          cut    = 15,
          layout = c(10,3),
          col.regions = rainbow(n = 16))
}

plot.FOV.corr         <- xyplot( stdev ~ slice.no|sprintf("id: %s",id),
                                 df.norm.factor.FOV,
                                 grid   = TRUE,
                                 pch    = 3,
                                 groups = frame.no,
                                 main   = paste0("FOV-nonuniformity width (1 s.d., ",
                                                 FNTD.get.option(my.options, "correction.FOV.uniform"),
                                                 ")"))
