message("###########################\n...process tracking results...\n")

df.images             <- do.call("rbind", lapply(tracks, function(x){return(x$df.images)}))
df.images             <- merge(df.meta, df.images, by = "id")

df.trackspots         <- do.call("rbind",lapply(tracks, function(x){return(x$df.trackspots)}))
df.trackspots         <- merge(df.meta, df.trackspots, by = "id")

df.trackspots$amp     <- df.trackspots$A / (2*pi*df.trackspots$s^2)
df.trackspots$amp.raw <- df.trackspots$A.raw / (2*pi*df.trackspots$s^2) - df.trackspots$B.raw

df.trackspots$particle.name <- factor(df.trackspots$particle.name,
                                      c("1H", "4He", "12C", "16O", "FNTDino"))
df.images$particle.name     <- factor(df.images$particle.name,
                                      c("1H", "4He", "12C", "16O", "FNTDino"))

df.trackspots$amplitude <- switch(FNTD.get.option(my.options, "amplitude.method"),
                                  "maximum"         = df.trackspots$max, 
                                  "center"          = df.trackspots$center, 
                                  "center-adjacent" = df.trackspots$center.adj,
                                  "Gauss"           = df.trackspots$amp,
                                  "mean"            = df.trackspots$mean)


# For LSM, get adjusted countrate with saturation correction
if(FNTD.get.option(my.options, "device") == "LSM"){
  
  dwell.time.us   <- df.tracking.par$t.us[match(df.trackspots$id, df.tracking.par$id)]
  laser.power     <- df.tracking.par$lp[match(df.trackspots$id, df.tracking.par$id)]/100
  
  count.rate.raw  <- (df.trackspots$amplitude + df.trackspots$mean.supp) / dwell.time.us
  saturation.count.rate.MHz <- FNTD.get.option(my.options, "saturation.count.rate.MHz")
  count.rate.corr <- -saturation.count.rate.MHz*log(1-count.rate.raw/saturation.count.rate.MHz)
  
  counts          <- count.rate.corr * dwell.time.us
  laser.factor    <- (laser.power/.1)^0.165
  if(FNTD.get.option(my.options, "correction.background") == "After tracking"){
    B               <- df.trackspots$mean.supp
  }else{
    B               <- 0
  }
  count.rate.adj  <- (counts*laser.factor - B) / (laser.power*dwell.time.us)
  
  df.trackspots$amplitude <- count.rate.adj
}


# Filter tracks wtr length
cv <- paste(df.trackspots$id, df.trackspots$frame.no, df.trackspots$track.no)
df.track.length <- data.frame(id = tapply(as.character(df.trackspots$id), cv, unique),
                              frame.no = tapply(df.trackspots$frame.no, cv, unique),
                              track.no = tapply(df.trackspots$track.no, cv, unique),
                              n.trackspots = tapply(df.trackspots$amplitude, cv, length))

ii <- df.track.length$n.trackspots >= FNTD.get.option(my.options, "min.track.length") & 
  df.track.length$n.trackspots <= FNTD.get.option(my.options, "max.track.length")
df.track.length <- df.track.length[ii,]

cv1 <- unique(paste(df.track.length$id, df.track.length$frame.no, df.track.length$track.no))
cv2 <- paste(df.trackspots$id, df.trackspots$frame.no, df.trackspots$track.no)
ii        <- cv2%in%cv1
df.trackspots <- df.trackspots[ii,]

# If requested print out tracking details
if(FNTD.get.option(my.options, "document.tracking")){
  message("###########################\n...plotting tracking results...\n")
  plots                 <- lapply(tracks, function(x){return(x$plots)})
  pdf("TrackingResults.pdf")
  lapply(plots, function(x){lapply(x, plot)})
  dev.off()
}


