track.and.link <- function(file.idx){
  # DEBUG: file.idx <- 1
  
  # Read file
  jimg          <- FNTD.read.image(df.meta$file[file.idx])
  
  # Keep selected slices only
  jimg          <- jimg[[FNTD.get.option(my.options, "select.slices"),,]]
  dims          <- dim(jimg)
  n.frames      <- dims[4]
  
  # Prepare output data structures
  plots         <- list(n.frames)
  df.trackspots <- data.frame()
  df.images     <- expand.grid(id              = df.meta$id[file.idx],
                               slice.no        = 1:dims[3],
                               frame.no        = 1:n.frames,
                               n.x             = dims[1],
                               n.y             = dims[2],
                               total.intensity = NA,
                               net.intensity   = NA,
                               bkg.intensity   = NA)
  
  # Frame loop
  for(frame.no in 1:n.frames){
    # DEBUG: frame.no <- 1
    cat("Processing frame", 
        frame.no, 
        "of", 
        n.frames, 
        "in file", 
        basename(df.meta$file[file.idx]), 
        "\n")
    
    # Select frame
    kimg  <- jimg[[,frame.no,]]
    
    # Select corresponding rows in result structure
    ii    <- df.images$frame.no == frame.no
    
    # Get total intensity
    df.images$total.intensity[ii] <- mean(kimg)
    
    # Subtract background
    bimg  <- FNTD.get.background(kimg, 
                                 sliding.window.length = FNTD.get.option(my.options, 
                                                                         "background.window.size"))
    if(FNTD.get.option(my.options, "correction.background") == "During tracking"){
      kimg  <- kimg - bimg
    }
    
    # Get net and background intensity
    df.images$net.intensity[ii] <- mean(kimg)
    df.images$bkg.intensity[ii] <- mean(bimg)
    
    # Normalize intensity (at this point for tracking only)
    timg  <- FNTD.normalize.stack(kimg, FNTD.get.option(my.options, 
                                                        "reference.slice"))
    
    # Find track spots ("tracking") using the point feature extraction
    timg <- FNTD.find.trackspots(timg, 
                                 radius     = as.integer(df.meta$radius[file.idx]), 
                                 cutoff     = as.numeric(df.meta$cutoff[file.idx]),
                                 percentile = as.numeric(df.meta$percentile[file.idx])) 
    
    # Find ("link") tracks = trajectories
    timg <- FNTD.find.tracks(timg, 
                             radius             = 3, 
                             link.range         = 5, 
                             displacement       = 20,
                             weight.distance    = 1, 
                             weight.intensity   = 1, 
                             weight.velocity    = 1, 
                             weight.angle       = 1, 
                             max_angle          = 40, 
                             angle_history      = 3,
                             relink             = TRUE, 
                             relink.cost_factor = .2, 
                             relink.distance    = 50)  
   
    # Remove short tracks
    timg      <- FNTD.get.tracks.of.length(timg, 
                                           min.length = FNTD.get.option(my.options, 
                                                                        "min.track.length"))
    
    # Copy trackspot information back to non-normalized image
    kimg@tracks   <- timg@tracks
    
    
    # Get trackspot information from net image and background image
    tmp           <- FNTD.get.trackspot.properties(kimg, 
                                                   bimg, 
                                                   radius = FNTD.get.option(my.options, 
                                                                            "evaluation.radius"))
    
    # If requested, save tracking plots
    if(FNTD.get.option(my.options, "document.tracking")){
      plots[[frame.no]] <- FNTD.plot.image(timg, 
                                           showTracks = TRUE, 
                                           title = paste(df.meta$id[file.idx], 
                                                         frame.no),
                                           show  = FALSE)
    }else{
      plots[[frame.no]] <- NULL
    }
    
    # Get trackspot information from raw image (fit larger radius and get background from that)
    rimg          <- jimg[[,frame.no,]]
    rimg@tracks   <- timg@tracks
    tmp2          <- FNTD.get.trackspot.properties(rimg, radius = 6, no.background.Gauss = FALSE)
    
    # Put things together
    tmp$max.raw        <- tmp2$max
    tmp$center.raw     <- tmp2$center
    tmp$center.adj.raw <- tmp2$center.adj
    tmp$B.raw          <- tmp2$B
    tmp$A.raw          <- tmp2$A
    tmp$s.raw          <- tmp2$s
    
    tmp$id             <- df.meta$id[file.idx]
    tmp$frame.no       <- frame.no
    
    df.trackspots      <- rbind(df.trackspots, tmp)
    
    # Remove java objects
    FNTD.remove(kimg, bimg, rimg, timg)
  }
  
  # Remove java objects
  FNTD.remove(jimg)
  
  return(list(plots         = plots, 
              df.trackspots = df.trackspots,
              df.images     = df.images))
}
