\name{FNTD.apply}
\alias{FNTD.apply}

\title{
R's apply function tailored for use with FNTDjImage data.}
\description{
Applies a function repeatedly to a subset of an FNTDjImage, e.g. all slices or frames.}
\usage{
FNTD.apply(jimg, cycle = c("slices", "frames", "channels"), fun, ...)
}

\arguments{
  \item{jimg}{
The FNTDjImage object the function will be applied on.
}
  \item{cycle}{
Defines the subsets \code{fun} is applied to. Any combination (character vector) of "\code{x}", "\code{y}", "\code{slices}", "\code{frames}", and "\code{channels}".}
  \item{fun}{
The function to apply.}
  \item{\dots}{
Additional arguments for \code{fun}.
}
}

\seealso{
\code{\link{apply}}
}

\author{
Steffen Greilich
}

\examples{
# Get mean pixel value and standard deviation for image stack
jimg <- FNTD.read.image(system.file("extdata", "mh2003.tif.zip", package = "FNTD"))
FNTD.apply(jimg, fun = mean)
FNTD.apply(jimg, fun = sd)

bimg <- FNTD.get.background(jimg, 50)
FNTD.apply(bimg, fun = mean)
FNTD.apply(bimg, fun = sd)
}
