\name{FNTD.apply.flatfield}
\alias{FNTD.apply.flatfield}

\title{
Correct FNTD images for influence of microscope flatfield
}
\description{
The lateral illumination pattern of a confocal microscope is usually not uniform across acquired images. Detected intensities at the edges are smaller than those in the center and can with this function corrected using a number of different methods.
}

\usage{
FNTD.apply.flatfield(jimg, method = "file",
                           file = system.file("extdata", "flatfields", "FXR700RG_DKFZ_150611.tif.zip", package = "FNTD"),
                           parameters = list(x0 = 238.9,
                                             y0 = 352.4,
                                             a.c = -1.158e-06,
                                             b.c = 5.02e-05,
                                             mu = 0.95),
                           n.tiles = 8)
}

\arguments{
  \item{jimg}{
FNTDjimage object
}
  \item{method}{
Can be \code{file}, \code{function}, or \code{tiles}
}
  \item{file}{
For \code{method = 'file'} this is an image file, that contains the flatfield image. Data for the FXR700RG data (DKFZ) are shipped with the package.
}
  \item{parameters}{
For \code{method = 'function'} parameters for a parabolic fit: f(r) = mu / (a.c*r^2 + b.c*r + 1) and r^2 = (x-x0)^2 + (y-y0)^2 (\code{x0,y0} center position in pixel coordinates, \code{mu} mean intensity of normalized image).
}
  \item{n.tiles}{
For \code{method = 'tiles'} (experimental) the number of tiles to subdivide the image into. It is assumed that each tile should have the same intensity for a homogenous field of particles.
}
}

\value{
Corrected FNTDjImage object.
}

\author{
Steffen Greilich
}

\references{
Stadler (2015)\cr
Schudell (2013)
}

\seealso{
\code{evalute.tracks} demo
}