\name{FNTD.find.tracks}
\alias{FNTD.find.tracks}

\title{Find trajectories}

\description{Uses a heavily modifies version of the particle linker function from the MOSAIC tool suite for ImageJ to connect trackspots to trajectories ('tracks').}

\usage{(jimg, link.range = 2,
              displacement = 10,
              dynamics = "Constant velocity",
              radius = 3,
              weight.distance = 1,
              weight.features = 1,
              weight.dynamics = 1,
              max_angle = 20,
              angle_history = 3,
              relink = FALSE,
              relink.cost_factor = 0.1,
              relink.distance = 40)
}

\arguments{
  \item{jimg}{FNTDjImage including track spot (from \code{\link{FNTD.find.trackspots}} or \code{\link{FNTD.import.tracks}})}
  \item{link.range}{Number of slices the routine searches for related track spots, "2" means that a gap of one slice may occur.}
  \item{displacement}{Maximum distance (in pixels) from one trackspot to another one in the next slice to be considered on the same trajectory.}
  \item{dynamics}{Defines the nature of the track. Can be \code{Brownian motion}, \code{Straigh line} or \code{Constant velocity}. The latter means that the particle both moves on a straight line AND over a constant distance between slices. Only the first letter has to be given (case ignored).}
  \item{radius}{Radius of a particle. Free movement within a track is allowed when the displacement is less than radius/2.}
  \item{weight.distance}{Weighting factor for the distance.}
  \item{weight.dynamics}{Weighting factor for the dynamics.}
  \item{weight.features}{Weighting factor for the shape features .}
  \item{max_angle}{Maximum allowed angle between the displacement vector of an existing trajectory and a candidate linking particle.}
  \item{angle_history}{The amount of particles in each trajectory that is used to calculate the displacement vector. Decreasing this number allows for larger 'bents' in trajectories.}
  \item{relink}{Whether to employ the trajectory relinker routine. Abrupt ending of trajectories due to trajectory crossing can, partly, be solved by this routine.}
  \item{relink.cost_factor}{The maximum allowed cost, based on angle differences and distance between two trajectories, that is allowed for relinking. The total allowed cost is the relink.distance squared times this parameter.}
  \item{relink.distance}{The maximum allowed distance between the end- or startpoints of two trajectories.}
}

\details{
Note that track spots that cannot be assigned to a track will keep \code{track.no == 0} and are excluded from further analysis such as \code{\link{FNTD.get.track.properties}}, \code{\link{FNTD.get.tracks.of.length}} or \code{\link{FNTD.filter.tracks}} but are still accessible via \code{\link{FNTD.get.tracks}}.
}

\value{
\code{FNTDjImage} with linked trackspots in the \code{@tracks} slot.
}

\references{
Sbalzarini, I. F. & Koumoutsakos, P. Feature point tracking and trajectory analysis for video imaging in cell biology Journal of Structural Biology, Elsevier, 2005, 151, 182-195.
}

\author{Jasper J. M. Kouwenberg, Steffen Greilich}

\seealso{\code{evaluate.tracks} demo.}
