\name{FNTD.find.trackspots}
\alias{FNTD.find.trackspots}

\title{
Find particle track positions ("trackspots") in an image using a point feature detection.}
\description{
Finds particle track positions ("trackspots") in a single or in multiple images in an FNTDjImage object and stores them in this object as a Java pointer.
}
\usage{
FNTD.find.trackspots(jimg, radius, cutoff, percentile = 0, threshold = 0)
}

\arguments{
  \item{jimg}{FNTDjImage object}
  \item{radius}{Approximate radius of the tracks to be found. Will also be used to include those pixels employed to obtain the weighted center position.}
  \item{cutoff}{Threshold for non-particle discrimination: discard false positives.}
  \item{percentile}{The percentage of pixels, starting from maximum pixel value, which are considered for particle selection.}
  \item{threshold}{The minimum value of a pixel for it be considered for particle selection. When a threshold value is given, the percentile parameter is discarded.}
}

\details{
Uses the spot detection plugin for ImageJ by the MOSAIC group.

Trackspot detection is highly dependent on the parameters set. It is therefore adviced to find the best settings interactively. This can be done by using the corresponding ImageJ Mosaic plugin, storing the best settings found in a table (multi-line for multiple images, if applicable) and using these settings in an R script.\cr

Repeated use of the function will result in overwriting the previous particle track positions stored in the FNTDjImage object.}

\value{
The FNTDjImage object used as input with the track positions added in the @tracks slot.}

\references{
Klimpki, G. (2014),	Fluence-based dosimetry using fluorescence nuclear track detectors, M.Sc. thesis, Department of Physics and Astronomy, Heidelberg University. URL: http://www.ub.uni-heidelberg.de/archiv/17356.

MOSAIC Group (MPI-CBG). MosaicSuite for ImageJ and Fiji - Models, simulation and algorithms for interdisciplinary computing, 2013. URL: http://mosaic.mpi-cbg.de.}

\author{
Steffen Greilich, Jasper J. M. Kouwenberg
}

\examples{
# Use given settings
filename <- system.file("extdata", "sg35041_s03.lsm", package = "FNTD")
jimg     <- FNTD.read.image(filename)
FNTD.show.image(jimg)
jimg    <- FNTD.find.trackspots(jimg, radius = 3, cutoff = .0, percentile = 5)
#df     <- FNTD.get.tracks(jimg)
}
