\name{FNTD.get.background}
\alias{FNTD.get.background}

\title{
Fluorescence background from a proton/ion (i.e. track) irradiated FNTD.
}
\description{
Return image of the background.
}
\usage{
FNTD.get.background(jimg, sliding.window.length = 50)
}

\arguments{
  \item{jimg}{
Object of class FNTDjImage
}
  \item{sliding.window.length}{
Side length of the sliding window (in pixels).
}
}
\details{
Uses the "Subtract Background" ImageJ plugin by the MOSAIC group, i.e. a sliding window, histogram-based estimation of variable background. If sliding window is chosen to small it will include remainers of track signal, if chose too large the background neglects small scale variation.

To obtain the net signal, subtract the background image from input image.
}
\value{
Object of class FNTDjImage, of same properties as the input image.}
\references{
Cardinale, J., 2010. Histogram-based Background Subtractor for ImageJ. ETH Zurich. Technical Report.\cr
Hoof, S., 2013.	Correlation of Particle and Background Signal in Al2O3:C,Mg Fluorescent Nuclear Track Detectors. B.Sc. thesis,Department of Physics and Astronomy, Heidelberg University.
}
\author{
Steffen Greilich
}

\seealso{
\url{http://mosaic.mpi-cbg.de/}
}
\examples{
\dontrun{
# Get background for different sliding window settings
filename    <- system.file("extdata", "sg35041_s03.tif", package = "FNTD")
jimg        <- FNTD.read.image(filename)
FNTD.show.image(jimg)
FNTD.show.image(FNTD.get.background(jimg, 10))
FNTD.show.image(FNTD.get.background(jimg, 30))
FNTD.show.image(FNTD.get.background(jimg, 100))

# Get net signal image by subtraction
bimg        <- FNTD.get.background(jimg, 50)
nimg        <- jimg
}
}
