\name{FNTD.get.mask}
\alias{FNTD.get.mask}

\title{
Masking images
}
\description{
Creates a mask (i.e. a binary image) from given trackspot positions. Around each trackspot a circle with given radius is drawn. 
}
\usage{
FNTD.get.mask(jimg, radius, critical.distance = NULL)
}

\arguments{
  \item{jimg}{
FNTDjImage on which trackspots have been detected and positions stored using \code{\link{FNTD.find.trackspots}}.
}
  \item{radius}{
Radius of circles around trackspot positions (in pixels).
}
  \item{critical.distance}{
Minimum distance between two trackspots (in pixels). If they are closer, drawing the cricles is skipped (same as for \code{\link{FNTD.get.track.properties}}. If now value is given, 2*radius+1 is use.
}
}
\details{
Use ! operator to invert the mask. 
}
\value{
An binary FNTDjImage mimg of same size than \code{jimg}. Binary means 8-bit integer with two value (0,255) only. The image can be used to index jimg, i.e. jimg[mimm] will set all pixel to NaN that are masked.}

\author{
Steffen Greilich}

\examples{
filename <- system.file("extdata", "sg35041_s03.lsm", package = "FNTD")
jimg     <- FNTD.read.image(filename)
FNTD.show.image(jimg)
jimg     <- FNTD.find.trackspots(jimg, radius = 3, cutoff = 3.0, percentile = 0.5)

# Get mask with 5 px radius around all tracks
mimg     <- FNTD.get.mask(jimg, radius = 5, critical.distance = 0)

# Show mask and inverse mask
FNTD.show.image(mimg)
FNTD.show.image(!mimg)

# Apply mask by using it as index on original image
# 1. Mean value of all pixels in jimg
mean(jimg[])
# 2. Mean value for track areas defined as mask
mean(jimg[mimg], na.rm = TRUE)
# 3. Mean value of image pixel NOT around tracks
mean(jimg[!mimg], na.rm = TRUE)
}

