\name{FNTD.get.track.properties}
\alias{FNTD.get.track.properties}

\title{
Get information about particle tracks
}

\description{
Will return properties of particle tracks (trajectories) assinged to an FNTDjImage or from a dataframe containing trackspot information (as returned by \code{\link{FNTD.get.trackspot.properties}}).)
}

\usage{
FNTD.get.track.properties(x, radius,
                             critical.distance,
                             method = "maximum", 
                             voxel.size    = list(x.um = .1,
                                                  y.um = .1,
                                                  z.um = .1),
                             surface.param = list(z.offset = 0,
                                                  x.slope = 0,
                                                  y.slope = 0,
                                                  image.offset = 0,
														                      theta.cor = FALSE),
                             optical.param = list(n1 = 1.517,
                                                  n2 = 1.7659,
                                                  NumAp = 1.40),
                             image.size    = list(n.x = 1024,
                                                  n.y = 1024,
                                                  n.slices = 30),
									           depth.refinement = FALSE)
}

\arguments{
  \item{x}{Either an FNTDjImage object or a dataframe containing trackspot properties}
  \item{radius}{Radius (in pixels) around a track spot position that is considered to belong to this track. Not to be confused with radius parameter in FNTD.find.trackspots!. This parameters has not to be provided if \code{x} is a dataframe.}
  \item{critical.distance}{Two tracks may not be closer as this distance (in pixels) to be evaluated. If no value is given, it is set to 2*radius+1.}
  \item{method}{Method to evaluate fluorescence amplitude of the trackspots. Can be \code{"maximum"}, \code{"center"}, \code{"center-adjacent"}, or \code{"Gauss"}. For the latter, a two-dimensional (symmetric) Gaussian bell-shape curve will be fitted to the trackspots (including all pixels in the given radius) and the area under the curve will be used as measure of track intensity (mean, sd). For \code{"maximum"}, the maximum intensity of the trackspots will be used instead, for \code{"center"} the center pixel, for \code{"center-adjacent"} the center plus the four adjacent pixels (at sampling densities equal or higher than the Nyquist frequency, the methods are \"more or less\" equivalent, with "center-adjacent" showing reduced (shot) noise.}
  \item{voxel.size}{The voxel dimensions without ASF corrections given in um.}
  \item{surface.param}{Fit parameters obtained from a linear fit of the surface reflection. Used to correct tilting during FNTD scanning. Also corrects the theta angles of the tracks when \code{theta.cor} is TRUE.}
  \item{optical.param}{Refractive indices and numerical aperture for the scanning setup.}
  \item{image.size}{Image dimensions in pixels. Only required when x is a dataframe.}
  \item{depth.refinement}{Deploy a routine that aims to refine the endpoint of each trajectory by weighing the intensities of the last trackpoints.}
}

\details{
The routine will run \code{\link{FNTD.get.trackspot.properties}} first - the arguments will be passed on accordingly. If no track (trajectory) information is available (i.e. track.no == 0) the routine will exit.
}

\value{
A data frame containing track index, length (number of slices), first and last slice, the mean intensity, and the variation (1 s.d.), absolute and relative, of the intensity along a track. Two versions of the variation are reported: as-is (i.e. the standard deviation along a track) and "adjusted" (i.e. the standard deviation of the residuals of a linear fit of the intensities along a track. This can compensate remaining intensity trends with depths that could for example not be corrected by the \code{\link{FNTD.normalize.stack}} function.)}

\author{
Steffen Greilich, Jasper J. M. Kouwenberg
}

