\name{FNTD.get.tracks.of.length}
\alias{FNTD.get.tracks.of.length}

\title{Retain tracks with a specified length}

\description{
 All tracks in the data set, related to \code{jimg}, having a length at least of \code{min.length} are retained while all others are rejected.}

\usage{FNTD.get.tracks.of.length(jimg, min.length = 3)}

\arguments{
  \item{jimg}{FNTDjImage}
  \item{min.length}{Minimum number of slices, a track has to cover, to be retained.}
}

\details{
  The track information are obtained using \code{FNTD.get.track(jimg)}. The track length is determined for each track by subtracting the smallest slice number from the largest. If the length is greater or equal \code{min.length} it is copied to a result data frame.
  
The default minimum of 3 allows for reasonable linear regression.   
}
                     
\value{A \code{FNTDjImage} containing only the tracks of desired length.}
                     
\seealso{\code{\link{FNTD.get.track.fits}}}
                     
\author{Steffen Greilich}
                     
\examples{
\dontrun{
  jimg <- FNTD.read.image(system.file("extdata", "mh2003.tif.zip", package = "FNTD"))
  jimg <- FNTD.find.trackspots( jimg, radius = 3, cutoff = 3.0, percentile = 0.5)
  jimg <- FNTD.find.tracks(jimg, dynamics = "Constant velocity")
  jimg <- FNTD.get.tracks.of.length(jimg)
  # accessing the data frame with tracks of desired length
  df   <- FNTD.get.tracks(jimg)
}
}                     