\name{FNTD.get.trackspot.properties}
\alias{FNTD.get.trackspot.properties}

\title{
Get numeric information about particle track spots
}

\description{
Will return the properties of all particle track spots assinged to an FNTDjImage (using \code{\link{FNTD.find.trackspots}} before.)
}

\usage{
FNTD.get.trackspot.properties(jimg, simg, radius, critical.distance = NULL, method = "adjacent")
}

\arguments{
  \item{jimg}{FNTDjImage object}
  \item{simg}{FNTDjImage object, supplementary image on which the trackspot position of \code{jimg} will be applied to retrieve max, mean etc., most frequent application would be a background image of jimg.}
  \item{radius}{Radius (in pixels) around a track position that is considered to belong to this track. Not to be confused with radius parameter in FNTD.find.trackspots!}
  \item{critical.distance}{Two tracks may not be closer as this distance (in pixels) to be evaluated. If no value is given, it is set to 2*radius+1.}
  \item{method}{Method to evaluate fluorescence amplitude of the trackspots. Can be \code{"maximum"}, \code{"adjacent"}, or \code{"Gauss"}. For the latter, a two-dimensional (symmetric) Gaussian bell-shape curve will be fitted to the trackspots (including all pixels in the given radius) and the area under the curve will be used as measure of track intensity (mean, sd). For \code{"maximum"}, the maximum intensity of the trackspots will be used instead, for \code{"adjacent"} the maximum plus the four adjacent pixels (at sampling densities equal or higher than the Nyquist frequency, the methods are \"more or less\" equivalent, with "adjacent" showing reduced (shot) noise.}
}
\details{
The critical distance is necessary as some (but not all!) properties cannot be determined correctly when tracks are overlapping. Those tracks will be skipped in the analysis (and the properties set to NA not be included into further analyses) and can be identified by their track index which will be preserved. If for example the maximum intensity is the only important parameter the critical distance can be set lower than 2*radius+1, on the other hand the radius should not be too small, as the maximum intensity might not coincide with the weighted track center.

Prelimiary studies revealed that at optimal sampling density for the microscope read-out, properties directly computed (max, width, ...) is equal or superior to the Gaussian fit.

Generally, this function should be applied on background subtracted images.
}

\value{
A data frame containing track spot index, slice, frame and channel index, x and y track spot position. Reported properties are area (might be smaller as pi*radius**2 if track spot is at the border of an image), max, min, mean (depends on radius), as well as the three parameters (area A, standard deviation s, background B) of the Gaussian fits, if chosen.

For track spots that are not considered as they are below the critical distance, zero values will be reported. Obviously, those tracks have to be excluded for some further anaylses such as averaging.}

\author{
Steffen Greilich
}

\examples{
\dontrun{
filename <- system.file("extdata", "sg35041_s03.lsm", package = "FNTD")
jimg     <- FNTD.read.image(filename)
FNTD.show.image(jimg)
jimg   <- FNTD.find.trackspots(jimg, radius = 3, cutoff = 3.0, percentile = 0.5)
df     <- FNTD.get.trackspot.properties(jimg, radius = 5, do.Gaussian = TRUE)

# Get mean track intensity
max(df$max[df$max != 0])
}
}
