\name{FNTD.show.image}
\alias{FNTD.show.image}

\title{
Display an FNTD image
}
\description{
Function displays the image in a separate window. If image contains trackspot and/or track information, they can be overlayed.
}

\usage{
FNTD.show.image(jimg, showTracks = FALSE)
}

\arguments{
  \item{jimg}{FNTDjImage object}
  \item{showTracks}{If TRUE, track spots and tracks will be displayed as an overlay.}
}

\details{
The image can be zoomed in by left-clicking, and zoomed out by a right mouse click. It can also be dragged (if image is larger than window) by holding down the left mouse button and moving into the desired direction. Pressing "c starts the "Adjust contrast and brightness" dialog from ImageJ.\cr

Trajectories are displayed in different (random) colors to distinguish them. Track spots that are assigned to a trajectory are shown as pink squares, all other as red circles.

The routine uses showImage method from ImagePlus class and overlays in ImageJ plus customized image- and mouse-listeners.\cr

}

\author{
Steffen Greilich
}

\examples{
\dontrun{
# Read and show example image (without tracks)
filename <- system.file("extdata", "sg35041_s03.lsm", package = "FNTD")
jimg     <- FNTD.read.image(filename)
FNTD.show.image(jimg)

# Show example image (with track spots)
jimg   <- FNTD.find.trackspots(jimg, radius = 3, cutoff = 3.0, percentile = 0.5)
FNTD.show.image(jimg, showTracks = TRUE)

# Show image with multiple slices and tracks
filename <- system.file("extdata", "mh2003.tif.zip", package = "FNTD")
jimg     <- FNTD.read.image(filename)
jimg     <- FNTD.find.trackspots(jimg, radius = 3, cutoff = 3.0, percentile = 0.5)
jimg     <- FNTD.find.tracks(jimg, dynamics = "Constant velocity")
FNTD.show.image(jimg, showTracks = TRUE)
}
}
