\name{FNTDjImage-class}
\alias{FNTD main image class}
\alias{FNTD image access and methods}
\alias{FNTDjImage-class}
\alias{FNTDjImage}
\alias{as.FNTDjImage}
\alias{+,FNTDjImage,FNTDjImage-method}
\alias{+,FNTDjImage,numeric-method}
\alias{-,FNTDjImage,FNTDjImage-method}
\alias{-,FNTDjImage,numeric-method}
\alias{*,FNTDjImage,FNTDjImage-method}
\alias{*,FNTDjImage,numeric-method}
\alias{/,FNTDjImage,FNTDjImage-method}
\alias{/,FNTDjImage,numeric-method}
\alias{!,FNTDjImage-method}
\alias{[,FNTDjImage-method}
\alias{[[,FNTDjImage-method}
\alias{dim,FNTDjImage-method}
\alias{length,FNTDjImage-method}
\alias{mean,FNTDjImage-method}
\docType{class}


\title{Class \code{"FNTDjImage"}}

\description{This class stores image objects for use with the FNTD package. It uses of the ImageJ ImagePlus class with benefits for performance, memory use and the application of standard ImageJ routines. 

The ImagePlus class stores images in five-dimensional "hyperstacks", with lateral dimensions x and y, slices (stack) z, frames t, and channels c.}

\section{Structure of the class}{
FNTDjImages contain:
\tabular{ll}{
imagePlus \tab Java pointer to the ImagePlus object \cr
tracks \tab Java pointer to particle track positions (NULL, if \code{\link{FNTD.find.trackspots}} has not been run yet.)}}

\section{Methods}{
  \tabular{ll}{
    \code{jimg[[k,l,m]]}\tab Returns an FNTDjImage object which is a substack of jimg, where k the slice indices, and l and m the frame and channel indices.\cr
    \code{jimg[i,j,k,l,m]}\tab Returns the pixel values as an array, where i,j are the lateral indices (can be vectors as for any R matrix or array), k the slice indices, and l and m the frame and channel indices.\cr
    \code{dim(jimg)}\tab Returns the dimensions of the image \code{jimg} as a five-element integer vector.\cr
    \code{length(jimg)}\tab Returns the total number of pixels in the image, i.e. x*y*z*t*c.\cr
    \code{mean(jimg)}\tab Returns the mean pixel value per slice.\cr
    \code{!jimg}\tab Returns the inverted image - useful for images that serve as masks.\cr
    \code{jimg1 +,-,*,/ jimg2} \tab Returns an image with pixel-wise addition, subtraction, mulitplication or division between \code{jimg1} and \code{jimg2}. The result is always converted to 32 bit float (no matter the original format) to prevent clipping or round-off problems. Any particle track information associated with the images will be lost.\cr
    \code{jimg1 +,-,*,/ x} \tab Returns an image with pixel-wise addition, subtraction, multiplication or division bewteen \code{jimg1} and a single number \code{x} (for multiplication: also an array matching the number of slices in length. Then, slice-wise multiplication is done). Again, the result will be 32 bit float. Particle track information is retained.\cr
    \code{FNTDjImage(data, dim, title = "New Image")}\tab Creates an FNTDjImage object with dimensions dim (five-dimensional vector: x, y, z, t, c) from data (vector).\cr
    \code{as.FNTDjImage(array)}\tab Transforms array (up to five dimensions) to an FNTDjImage object.\cr
	}
}

\author{
Steffen Greilich
}

\examples{
# Read example image
filename <- system.file("extdata", "sg35041_s03.lsm", package = "FNTD")
jimg     <- FNTD.read.image(filename)

# Show java pointer
jimg@imagePlus

# Dimensions
dim(jimg)

# Compute mean and sd for original image and 2*image
FNTD.apply(jimg, fun = mean)
FNTD.apply(jimg, fun = sd)

jimg.new <- jimg + jimg
FNTD.apply(jimg.new, fun = mean)
FNTD.apply(jimg.new, fun = sd)

jimg.new <- jimg + 100
FNTD.apply(jimg.new, fun = mean)
FNTD.apply(jimg.new, fun = sd)

# Get pixels for sub-image
require(lattice)
values <- jimg[1:100, 1:100, 1, 1, 1]
levelplot(values,
          col.regions = grey((0:100)/101),
          ylim = c(100:1))

# Creates FNTD image from vector or array
dim      <- c(100,100,5,2,1)
pixel    <- rnorm(dim[1]*dim[2]*dim[3]*dim[4]*dim[5])
nimg     <- FNTDjImage(pixel,
                       dim,
                       "From simple vector")
FNTD.show.image(nimg)

m        <- array(pixel,
                  dim)
mimg     <- as.FNTDjImage(m,
                          "From array")
FNTD.show.image(mimg)

}

\keyword{classes}
