\name{FNTDoptions}
\alias{FNTDoptions}
\alias{FNTD.options}
\alias{FNTD.get.option}
\alias{FNTD.set.option}

\title{Options for sample processing and analysis}

\description{Class and functions to control the entire process of sample analysis.}

\usage{
FNTD.get.option(options, name)
FNTD.set.option(options, name, value)
FNTD.options(...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{options}{An object of class \code{FNTDoptions}.}
  \item{name}{Option name, see details}
  \item{value}{Option value to set}
  \item{...}{Any number of options to override defaults, see details.}
}

\details{
\describe{
Options are:
\item{\code{data.path}}{Full path to data location. Images will be sought recursively in all subfolders at the given location. The path is also used for for the tracking and meta data files. \bold{Default: \code{"."}}.}

\item{\code{tracking.par.file}}{Name and path relative to \code{data.path} of the file (Excel format) containing parameters for tracking, i.e. \code{radius}, \code{cutoff}, and \code{percentile}, together with sample \code{id}. See also \code{\link{FNTD.find.trackspots}}. \bold{Default: \code{"TrackingParameters.xlsx"}}}

\item{\code{meta.data.file}}{Name and path relative to \code{data.path} of the file (Excel format) containing meta data. Mandatory are sample \code{id}, \code{particle.name}, \code{E.MeV.u} (beam energy), but the latter two can be set to NA if not known. Additional variables can be given. \bold{Default: \code{"MetaData.xlsx"}}}

\item{\code{id.length}}{The script assumes that the sample id is reflected in the first \code{n} characters of the filenames, so for example sg1000_C12_200MeV.tif yields 'sg1000' with \code{id.length = 6}. \bold{Default: \code{6}}}

\item{\code{device}}{\code{'FXR'} or \code{'LSM'}. \bold{Default: \code{"FXR"}}}

\item{\code{data.type}}{\code{'slices'} or \code{'stacks'}. Some operations will be skipped for the first choice (angles etc.). \bold{Default: \code{"Stacks"}}}

\item{\code{use.tracking.results}}{If true, result for tracking and linking automatically saved during a previous run will be used. Can save a lot of time but to be used with caution! \bold{Default: \code{FALSE}}}

\item{\code{select.slices}}{Which slices (one-based index) should be used for analysis? For FXR, often the first slice (surface) is removed. Use R index syntax. \bold{Default: \code{2:21}}}

\item{\code{reference.slice}}{Reference slice for sperical abberation correction and slice on which slice specific analyses will be run. \bold{Default: \code{1}}}

\item{\code{min.track.length}}{Minimum number of slices (not trackspots, i.e. gaps are allowed) to be covered by trajectories to be accepted. \bold{Default: \code{0}}}

\item{\code{max.track.length}}{Maximum number of slices (not trackspots, i.e. gaps are allowed) to be covered by trajectories to be accepted. Equals usually the number of slices in the stack. \bold{Default: \code{20}}}

\item{\code{amplitude.methode}}{Method of assessing fluorescence amplitude of trackspots. \bold{Default: \code{"maximum"}}.
  \describe{
  \item{\code{"maximum"}}{Value of hottest pixel within disc of radius \code{evaluation.radius} around trackspot center.}
  \item{\code{"mean"}}{Mean fluorescence signal within disc of radius \code{evaluation.radius} around trackspot center. Corresponds to sum, except for a constant factor.}
    \item{\code{"center"}}{Value of pixel at trackspot center.}
    \item{\code{"center.adj"}}{Value of pixels at trackspot center and four adjecent pixels (top, right, bottom, left).}
    \item{\code{"Gauss"}}{Amplitude of a 2D Gaussian fit (same width in x and y, no rotation), i.e. volume under curve / width^2.}
  }
}

\item{\code{evaluation.radius}}{Radius around trackspot center used in amplitude assessment.  \bold{Default: \code{3}}}

\item{\code{background.correction}}{Method of assessing and subtracting fluorescence background.  \bold{Default: \code{"During tracking"}}
  \describe{
  \item{\code{"none"}}{No background will be assessed and subtracted.}
  \item{\code{"During tracking"}}{Histogram-based background assessment using the 'Background Subtractor' plugin of MOSAIC ToolSuite, but allowing for negative pixel values. A sliding window size has to be given. The background is subtracted pixel-wise from the entire original image before amplitude assessment.}
  \item{\code{"After tracking"}}{Same as "During tracking", but the mean background intensity within a disc of \code{evaluation.radius} around the trackspot center is subtracted from the amplitude that is evaluated on the original, non-background subtracted image. Should yield almost same results as "During tracking", but useful for debugging as fluctuations / bias in the background can be studied.}
  }
}

\item{\code{correction.fluctuation}}{Method of correction for (intra-detector) sensitivity fluctuation between frames (= stacks). The correction factors are reported in a plot. \bold{Default: \code{"Slice-wise tracks"}}
  \describe{
  \item{\code{"none"}}{No correction will be applied.}
  \item{\code{"Slice-wise tracks"}}{Each slice in a specific depth will be normalized to the mean sum of amplitude from slices in this depths over all frames/stacks in a sample.}
  \item{\code{"Sample-wise tracks"}}{All slices in a specific depth will be normalized to the mean sum of amplitude from all slices over all frames/stacks in a sample.}
  }
}

\item{\code{correction.spher.abb}}{Method of correction for spherical abberation. \bold{Default: \code{"Frame-wise tracks"}}
  \describe{
  \item{\code{"none"}}{No correction will be applied.}
  \item{\code{"Frame-wise total"}}{}
  \item{\code{"Frame-wise tracks"}}{}
  \item{\code{"Sample-wise total"}}{}
  \item{\code{"Sample-wise tracks"}}{}
  }
}

\item{\code{correction.FOV.uniform}}{Method of correction for field-of-view non-uniformity ("flatfield"). \bold{Default: \code{"Frame-wise tracks"}}
  \describe{
  \item{\code{"none"}}{No correction will be applied.}
  \item{\code{"Measurement"}}{}
  \item{\code{"Parameter"}}{}
  \item{\code{"Slice and Frame-wise tracks"}}{}
  \item{\code{"Frame-wise tracks"}}{}
  \item{\code{"Sample-wise tracks"}}{}
  }
}

\item{\code{correction.angular}}{Method of correction for intensity decrease due to tilting of tracks. \bold{Default: \code{"Hyperbola"}}
  \describe{
  \item{\code{"none"}}{No correction will be applied.}
  \item{\code{"Hyperbola"}}{Formula from Bartz et al., 2014, with parameter \code{angular.corr.factor}}
  }
}

\item{\code{angular.corr.factor}}{Weighting factor in Bartz et al., 2014, angular correction. \bold{Default: \code{3.5}}
}

\item{\code{correct.saturation}}{Dead-time correction for saturation effects in the photon-counting APD (Zeiss LSM710 only). \bold{Default: \code{TRUE}}
}

\item{\code{saturation.count.rate.MHz}}{Characteristic count-rate in MHz for saturation correction, cf. Greilich et al., 2013. \bold{Default: \code{16.2}}
}

\item{\code{voxel.size}}{List with voxel size in um, three entries for x (\code{x.um}), y (\code{y.um}), and z (\code{z.um}). \bold{Default: \code{x.um = 100/504, y.um = 100/504, z.um = 5}}
}

\item{\code{image.size}}{List with image dimensions in pixel units, three entries for x (\code{n.x}), y (\code{n.y}), and z (\code{n.slices}). \bold{Default: \code{n.x = 100, n.y = 100, n.slices = 20}}
}

\item{\code{optical.param}}{List with parameters for correction of refraction effects. \bold{Default: \code{n1 = 1.0, n2 = 1.77, NumAp = 0.95}}
  \describe{
  \item{\code{n1}}{Refractive index of medium between lens and FNTD, e.g. air or oil.}
  \item{\code{n2}}{Refractive index of FNTD, i.e. 1.77.}
  \item{\code{NumAp}}{Numerical Aperture of the lens used.}
  }
}

\item{\code{calibration.function}}{Function used for relation of fluorescence amplitude and intensity. \bold{Default: \code{"log.log"}}
  \describe{
  \item{\code{"log.log"}}{Linear model in double log scale.}
  \item{\code{"empiral.log"}}{Formula from Klimpki et al., 2015}
  }
}

\item{\code{calibration.coefs}}{Parameters used with relation of fluorescence amplitude and intensity. \bold{Default: \code{-0.13832, 0.54482}}
  \describe{
  \item{\code{"log.log"}}{Intercept and slope.}
  \item{\code{"empiral.log"}}{a, b, c}
  }
}
}
}

\value{
\code{FNTD.options()} returns an \code{FNTDoptions} object with defaults, except for those values that were overridden by the user. \code{FNTD.get.option} returns the value of the option requested. \code{FNTD.set.option} returns a message if the setting was successful.
}

\author{
S. Greilich}

\examples{
my.options <- FNTD.options(data.path            = "/data/home/greilich/workspace/mySamples",
                           meta.data.file       = "meta.data.xlsx",
                           tracking.par.file    = "tracking.parameters.xlsx",
                           id.length            = 3,
                           use.tracking.results = FALSE,
                           select.slices        = 2:21,
                           image.size           = list(n.x = 504,
                                                       n.y = 504,
                                                       n.slices = 20))}
