# Possible further arguments
# Regularization Channel 1
# gd.addNumericField("Reg_ch1",0.05,3);
# Add Min int Channel 1
# gd.addNumericField("Min_int_ch1",0.15,3);
# PSF xy
# gd.addNumericField("PSF_xy", 1.0, 3);
# PSF z
# gd.addNumericField("PSF_z", 1.0, 3);

if(F){

rm(list = ls()) 
require(FNTD)


fiji.path  <- path.expand("~/opt/Fiji/ImageJ-linux64")
image.name <- "mh2003_crop_frames.tif"

##########################################
# Separate (hyper)stack into single images
cur.path   <- getwd()
jimg       <- FNTD.read.image(file.path(cur.path, "tests", image.name))

file.names <- .jcall(.jnew("fntd/stacks/StackOperations"), 
       "[S",
       "stackToImageFiles", 
       jimg@imagePlus, 
       as.character(cur.path), 
       as.character("tmp"))

###########################################
# Create macros
lapply(file.names, 
       function(x, p){ arg  <- paste0( "run(\"Squassh\", \"filepath=", 
                                       file.path(p, x), "\" );")
                       write(arg, paste0(x, ".ijm"))
                       paste0("Macro written for ", x)},
       p = cur.path)

###########################################
# Start Fiji instances
lapply(file.names,
       function(x, f){ system(command = paste0( f,
                                                " -batch ",
                                                paste0(x, ".ijm")),
                              intern = FALSE, ignore.stdout = FALSE, ignore.stderr = FALSE,
                              wait = FALSE, input = NULL)
                       paste0("Fiji started for ", x)},
       f = fiji.path)

###########################################
# Gather results and cleanup
ll <- lapply(file.names,
       function(x){  n <- sub("\\.[[:alnum:]]+$", "", basename(as.character(x)))
                     df <- read.table(paste0(n, "_ObjectsData_c1.csv"),
                                      header = TRUE,
                                      comment.char = "%",
                                      sep    = ";")
                     df$file <- n
                     file.remove(paste0(n, "_ObjectsData_c1.csv"))
                     file.remove(paste0(n, "ImagesData.csv"))
                     file.remove(paste0(n, "_outline_overlay_c1.zip"))
                     file.remove(paste0(n, ".ijm"))
                     file.remove(paste0(n, ".tif"))
                     return(df)})
df <- do.call("rbind", ll)

ll <- lapply( strsplit(df$file, split = "_"),
              function(x){ c( as.numeric(gsub("c", "", x[[2]])),
                              as.numeric(gsub("f", "", x[[3]])),
                              as.numeric(gsub("s", "", x[[4]])))})
df2 <- as.data.frame(do.call("rbind", ll))
names(df2) <- c("channel.no", "frame.no", "slice.no")
df <- cbind(df2, df)
}