source(file="include/default.R")

require(ggplot2)
require(Cairo)

files.no      <- c()
type.fit      <- "gauss"
step          <- 0.01
cell          <- c()
D             <- cell.dose.ext[1:10]
a_d           <- 1.66

Sz    <- matrix(rep(0, length(cell.dose.ext)*length(cell)), ncol=length(cell.dose.ext))
Sd    <- exp(-a_d * D)
q_fit <- rep(0, length(cell))

tr.data <- list()
for (i in 1:length(D)) {
  load(paste('Data/Spheroids/bottom0file', min(files.no), '-', max(files.no), '_', D[i], 'Gy.rda', sep=''))
  tr.data[[i]] <- res$dose[res$cell %in% cell]
}

fit <- optim(.5*a_d, az.fit, Sd=Sd, zdata=tr.data, method="BFGS", control=list(reltol=1e-9))
cat("a_z:", fit$par)

Cairo(width = 900, height = 600, dpi = 200, file=paste("Data/Plots/Cell/Monolayer_survival.png", sep=""), type="png")
par(mar = c(5,4,1,1), family="Times New Roman", cex=.75)

plot(D, 100*Sd, lty = 1, log="y", xlab="Absorbed dose (Gy)", ylab="Expected survival (%)")
grid(col = "black", lty = "dotted", equilogs = F, lwd=.3)
par(new=T)
plot(D, 100*Sd, lty = 1, log="y", xlab="Absorbed dose (Gy)", ylab="Expected survival (%)")

lines(D, 100*az.fit(a_d, zdata=tr.data, error=F), lty = 2)
lines(D, 100*az.fit(fit$par, zdata=tr.data, error=F), lty = 3)
legend('bottomleft', c("Experimental", "Uncorrected microdosimetry", "Corrected microdosimetry"), lty=c(NA,2,3), pch=c(1,NA,NA), bg="white")
dev.off()

D_1per_ext = approx(az.fit(a_d, zdata=tr.data, error=F), D, 0.25)
D_1per = approx(az.fit(fit$par, zdata=tr.data, error=F), D, 0.25)
cat("RBE:", D_1per_ext$y/D_1per$y)
