source(file="include/default.R")
# source(file="include/parallel.R")

files.no      <- c()
cells.no      <- c()

no.pos        <- 1
angles        <- 0
angle.cut     <- "bottom"

cell.dose     <- 0

do.save       <- F
do.draw       <- F
show.time     <- F
res <- list()

if (do.draw) no.pos  <- 1

tot.tracks <- data.frame()

for (fileid in files.no)
{
  load(file=paste('Data/Tracks/tracks_', as.character(settings$filename[fileid]), '.rda', sep=""))
  
  tot.tracks <- rbind(tot.tracks, tracks)
}

jimg <- FNTD.read.image(paste(Image.Dir, as.character(settings$filename[files.no[1]]),"_fluorescence.tif",sep=""))
jimg <- FNTD.create(jimg)
n    <- dim(jimg)[1:3]

for (cut.angle in angles) {
  if (angle.cut == "bottom")
    adj.tracks <- tot.tracks[tracks$theta.deg > cut.angle,]
  else
    adj.tracks <- tot.tracks[tracks$theta.deg < cut.angle,]
  
  fntd.dose <- FNTD.get.macro.dose(adj.tracks, LET.file = c(LET.files$water, LET.files$water),
                                     voxel.size = voxel.size,
                                     image.size = list(n.x = n[1], 
                                                       n.y = n[2],
                                                       n.slices = n[3]),
                                     border.distance      = border.dist,
                                     pre.target.heights   = c(0),
                                     target.layer.height  = 8)

  cat(" -- Angle:", cut.angle, "deg. FNTD dose:", round(fntd.dose,3), " Gy\n")
  
  for (dose in cell.dose)
  {
    res <- list()
    
    cat("Dose:", dose, "Gy\n")
    cat("Cell: ")
    for (i in cells.no) {
      if (i != cells.no[1])
        cat(paste(rep("\b", nchar(toString(i-1))), collapse=""))
      
      cat(i)
      
      cimg <- FNTD.read.image(paste(Cell.Image.Dir, "cell_", i, "_crop.tif",sep=""))
      
      if (show.time) start.time <- proc.time()[3]
      
      data <- FNTD.real.cell.hits(jimg, cimg, adj.tracks,
                                  fntd.voxel.size  = voxel.size,
                                  cell.voxel.size  = cell.voxel.size,
                                  cell.surface.param = list(z.offset = 0,
                                                            x.slope = 0,
                                                            y.slope = 0,
                                                            image.offset = 0),
                                  LET.file = LET.files$water,
                                  fntd.dose = fntd.dose, cell.dose = dose,
                                  dose.cycles = dose.cycles,
                                  draw = do.draw,
                                  border.distance = border.dist,
                                  no.pos = no.pos)
      
      if (show.time) cat("\n - Calculation time per cell:", (proc.time()[3] - start.time)/no.pos*1000, "ms\n")
      
      res$cell      = c(res$cell,      rep(i,length(data$energy)))
      res$energy    = c(res$energy,    data$energy)
      res$length    = c(res$length,    data$length)
      res$no.tracks = c(res$no.tracks, data$no.tracks)
      res$nuc       = c(res$nuc,       rep(data$nuc,length(data$energy)))
      res$cyt       = c(res$cyt,       rep(data$cyt,length(data$energy)))
    }
    cat('\n\n')
    
    res$dose = res$energy * 1.60218E-16/(res$nuc * 1E-18 * 1000) #keV * (keV to J) / (m^3 * kg/m^3)
    
    if (do.save) { save(res, file=paste('Data/Spheroids/', angle.cut, cut.angle, 'file', min(files.no), '-', max(files.no), '_', dose, 'Gy.rda', sep='')) }
    # cat('\nDose Ratio\t:', round(mean(res$dose)/dose, 5), '\n')
    
    # res <- res[res$length>0]
    # res <- res[!is.nan(res$energy) & !is.nan(res$length)]
    # cat('av. LET\t\t:', round(mean(res$energy/res$length, na.rm=T), 1), '±', round(sd(res$energy/res$length, na.rm=T), 1), ' keV/um\n\n')
  }
}