source(file="Include/default.R")
require(Cairo)
require(RColorBrewer)

p   <- function(i, mean) {
  return(mean^i*exp(-mean)/factorial(i))
}

energies = c(4000, 5470, 7000, 10000, 15000, 20000)
ranges   = rep(0, length(energies))
modes    = 1:4

ratio = matrix(rep(0, length(modes)*length(energies)), ncol=length(energies))

for (Imode in modes) {
  for (engy in 1:length(energies)) {
    load(file=paste('Data/Spheroids/Spheroid_am241_', energies[engy], 'keV_', Smode[Imode],'.rda', sep=''))
    
    ratio[Imode, engy] = mean(res$dose.nuc)/((res$v.nuc[1] * mean(res$dose.nuc) + res$v.cyt[1] * mean(res$dose.cyt)) / (res$v.nuc[1] + res$v.cyt[1]))
  }
}

LET <- read.table(LET.files$water, sep=',')

for (j in 1:length(ranges))
  ranges[j] = approx(x=LET$V1, y=LET$V3, energies[j], method = "linear")$y

colorpal = brewer.pal(length(modes)+1, "Set1")
# CairoWin()
Cairo(width = 900, height = 600, dpi = 200, file=paste('Data/Plots/Cell/Spheroid_ratios.png', sep=''), type="png")
xlas = c(100, 1, 0.01, 0.0001)
par(mar = c(5,4,5,1), family="Times New Roman", cex=.75)

plot(energies,  100*ratio[1,], type="l", main="", ylim=c(0, 200), xlim=c(0, max(energies)),
     ylab="Nucleus-Cytoplasm dose ratio (%)", xlab="Particle energy (keV)", lwd=0)
grid(col = "black", lty = "dotted", equilogs = TRUE, lwd=.3)
par(new=T)
plot(energies,  100*ratio[1,], type="o", main="", ylim=c(0, 200), xlim=c(0, max(energies)),
     ylab="Nucleus-Cytoplasm dose ratio (%)", xlab="Particle energy (keV)")

axis(1, at=c(0, energies), labels=c(0, round(ranges / ((6785/(4/3*pi))^(1/3)), 1)), side = 3, col="black")
mtext("Particle range (cell radii)", side = 3, line = 3, cex=.75)

for (Imode in 2:4)
  lines(energies, 100*ratio[Imode,], type="o", lty = Imode, pch = Imode)

legend('bottomleft', c(Smode), lty=modes, pch=modes, ncol = 2, bg="white")

dev.off()