source(file="include/default.R")

LET <- read.table(LET.files$water, sep=',')

Rnuc <- (742/(4/3*pi))^(1/3)
Rcyt <- (6785/(4/3*pi))^(1/3)
# Rnuc = 5
# Rcyt = 7
Reff <- .905*Rcyt

N    <- 20000
it   <- 1

for (Imode in 1:4) {
  for (engy in c(4000, 7000, 10000, 15000, 20000)) {
    iso_engy <- c(engy)
    iso_rang <- approx(x=LET$V1, y=LET$V3, iso_engy, method = "linear")$y
    iso_prob <- c(1)
    iso_prob <- cumsum(iso_prob)/sum(iso_prob)
    
    ratio_mean <- rep(0,it)
    dhit_mean  <- rep(0,it)
    for (i in 1:it) {
      res <- FNTD.spheroid.simulation(bounding.box    = max(iso_rang)+Rcyt,
                                      Imode           = Imode,
                                      iso_range       = iso_rang,
                                      iso_prob        = iso_prob,
                                      iso_name        = iso_name,
                                      Rnuc            = Rnuc,
                                      Reff            = Reff,
                                      LET.file        = LET.files$water,
                                      no.of.particles = N)
      
      ratio_mean[i] = mean(res$dose.nuc)/res$dose.cyt
      dhit_mean[i]  = mean(res$dose.nuc)
    }
    
    cat("\nNucleus-Cytoplasm Dose ratio:", mean(ratio_mean), "±", sd(ratio_mean)/mean(ratio_mean)*100, "%\n")
    cat("Mean dose/hit:", mean(dhit_mean), "±", sd(dhit_mean)/mean(dhit_mean)*100, "%\n\n")
    
    save(res, file=paste('Data/Spheroids/Spheroid_am241_', iso_engy[1], 'keV_', Smode[Imode],'.rda', sep=''))
  }
}

# cat("\nError: ", length(res$x[((res$x + Reff*sqrt(2)*1)^2 + (res$y - Reff*sqrt(2)*0)^2 + (res$z - Reff*sqrt(2)*0)^2) < Rnuc^2]))
