source(file="Include/default.R")
require(Cairo)
require(RColorBrewer)

p   <- function(i, mean) {
  return(mean^i*exp(-mean)/factorial(i))
}

a_z     = 1.922568
a_d     = 1.66
zn      = seq(0.1, 3, 0.1)
k       = 20
energy  = 5470
modes   = 1:4
do.plot = T


Sz = matrix(rep(0, length(modes)*k), ncol=k)
Sd = matrix(rep(0, length(modes)*k), ncol=k)
D  = matrix(rep(0, length(modes)*k), ncol=k)

for (Imode in modes) {
  cat(Smode[Imode], "\n")
  load(file=paste('Data/Spheroids/Spheroid_am241_', energy, 'keV_', Smode[Imode],'.rda', sep=''))
  
  dens <- density(res$dose.nuc, na.rm = T, from = 0, to = max(res$dose.nuc), n = 256)
  zny  <- multihit.distribution(k, dens)
  x    <- (0:(length(zny[[length(1)]]) - 1)*(dens$x[2] - dens$x[1]))
  
  calfac = ((res$v.nuc[1] * mean(res$dose.nuc) + res$v.cyt[1] * mean(res$dose.cyt)) / (res$v.nuc[1] + res$v.cyt[1]))/mean(res$dose.nuc)
  cat("1/Calfac:", 1/calfac, "\n\n")
  # calfac = 1
  for (q in 1:k) {
    Sz[Imode,q] = sum(zny[[q]]*exp(-a_z*x))
    D[Imode,q]  = sum(zny[[q]]*x) * calfac
    Sd[Imode,q] = exp(-a_d*D[Imode,q])
  }
  
  # fit <- lm(log(Sd - Sz) ~ D)
}

cat("RBE vs external")
D_1per_ext = approx(Sd[1,], D[1,], 0.25)
for (Imode in modes) {
  D_1per = approx(Sz[Imode,], D[Imode,], 0.25)
  cat("\n  RBE", Smode[Imode], ":\t", D_1per_ext$y/D_1per$y)
}

if (do.plot) {
  colorpal = brewer.pal(length(modes)+1, "Set1")
  # CairoWin()
  Cairo(width = 900, height = 600, dpi = 200, file=paste('Data/Plots/Cell/Survival_carriers.png', sep=''), type="png")
  xlas = c(100, 1, 0.01, 0.0001)
  par(mar = c(5,4,1,1), family="Times New Roman", cex=.75)
  
  plot(D[1,],  100*Sd[1,], log = "y", type="o", main="", yaxt='n', ylim=c(.5, 100), xlim=c(0, min(D[,k], na.rm = T)),
       ylab="Expected Survival (%)", xlab="Absorbed dose (Gy)")
  grid(col = "black", lty = "dotted", equilogs = T, lwd=.3)
  par(new=T)
  plot(D[1,],  100*Sd[1,], log = "y", type="o", main="", yaxt='n', ylim=c(.5, 100), xlim=c(0, min(D[,k], na.rm = T)),
       ylab="Expected Survival (%)", xlab="Absorbed dose (Gy)")
  
  axis(2,at=xlas,labels=xlas)
  
  for (Imode in modes)
    lines(D[Imode,], 100*Sz[Imode,], type="o", lty = Imode+1, pch = Imode+1)
  
  legend('bottomleft', c("External", Smode), lty=c(1,modes+1), pch=c(1,modes+1), bg="white")
  dev.off()
}