source(file="Include/default.R")

files.no      <- c()

detour.range <- 5

HR                <- T
proc.track.length <- T
filter.border     <- T
show              <- T

if (HR) {
  voxel.size               <- voxel.size.HR
  optical.param$depth.corr <- "Partial"
  depth.refinement         <- "Full"
} else {
  optical.param$depth.corr <- "Full"
  depth.refinement         <- "Full"
}

i <- 0
for (fileid in files.no)
{
  i <- i+1
  
  filename            <- as.character(settings$filename[fileid])
  cat(filename, "\n")
  FNTD_refl_data.file <- FNTD_refl_data[FNTD_refl_data$filename==filename,]
  settings.file       <- settings[settings$filename==filename,]
  Layers[2]           <- settings.file$air.length
  
  jimg <- FNTD.read.image(paste(Image.Dir, filename, "_fluorescence.tif",sep=""))
  n    <- dim(jimg)[1:3]
  
  jimg <- FNTD.find.trackspots(jimg, radius = settings.file$radius-2, cutoff = 0, threshold = settings.file$threshold)
  if (HR) 
    jimg <- FNTD.find.tracks(jimg,
                             drift              = 6,
                             link.range         = 3,
                             displacement       = 20,
                             weight.distance    = .5,
                             weight.intensity   = .2,
                             weight.velocity    = 1,
                             weight.angle       = 1,
                             max_angle          = 30,
                             angle_history      = 4,
                             relink             = T,
                             relink.cost_factor = 10,
                             relink.distance    = 40)
  else 
    jimg <- FNTD.find.tracks(jimg,
                             drift              = 6,
                             link.range         = 3,
                             displacement       = 10,
                             weight.distance    = .5,
                             weight.intensity   = 1,
                             weight.velocity    = .5,
                             weight.angle       = 1,
                             max_angle          = 30,
                             angle_history      = 4,
                             relink             = T,
                             relink.cost_factor = 10,
                             relink.distance    = 30)
  
  jimg <- FNTD.get.tracks.of.length(jimg, min.length = settings.file$min.length)
  
  tracks <- FNTD.get.track.properties(jimg, radius     = 5, critical.distance = 1,
                                      voxel.size       = voxel.size,
                                      surface.param    = list(z.offset     = FNTD_refl_data.file$intercept,
                                                              x.slope      = FNTD_refl_data.file$c_x,
                                                              y.slope      = FNTD_refl_data.file$c_y,
                                                              image.offset = FNTD_refl_data.file$img_offset,
                                                              theta.cor    = T),
                                      optical.param    = optical.param,
                                      depth.refinement = depth.refinement,
                                      amplitude.endpoint.mod = settings.file$end.mod,
                                      method           = amplitude.method,
                                      detour.range     = detour.range)
  
  if (filter.border)
    tracks <- FNTD.filter.borders(tracks, border.distance = border.dist.tracks,
                                  image.size = list(n.x      = n[1], 
                                                    n.y      = n[2],
                                                    n.slices = n[3]))
  
  # Track analysis
  tracks <- FNTD.get.energy(tracks, LET.files = c(LET.files$al2o3, LET.files$air, LET.files$gold), layer.thickness = Layers,
                            procedural.track.length = proc.track.length)
  tracks <- FNTD.get.range (tracks, LET.file = LET.files$water)
  
  save(tracks, file=paste('Data/Tracks/tracks_', filename, '.rda', sep=""))
  
  
  if (show) FNTD.show.image(jimg, TRUE)
  if (show) FNTD.show.image(jimg, TRUE, tracks[!is.na(tracks$detour.o) & tracks$detour.o >= .6*max(tracks$detour.o, na.rm = T),]$track.no)
  if (show) FNTD.show.image(jimg, TRUE, tracks[!is.na(tracks$detour.o) & tracks$detour.o < .004,]$track.no)
}