if (!exists("cl")) {
  cat("\014")
  rm(list = ls())
}
options(java.parameters = "-Xms6000m")

## Layers: FNTD, Air, Gold
Layers        <- c(0,0,2.416)
LET.files     <- list(al2o3 = 'Data/CSV/table_al2o3.csv',
                      air   = 'Data/CSV/table_air.csv',
                      gold  = 'Data/CSV/table_gold.csv',
                      water = 'Data/CSV/table_water.csv',
                      pe    = 'Data/CSV/table_pe.csv',
                      alu   = 'Data/CSV/table_alu.csv')

load(file = 'Data/FNTD_Refl_Regression.rda')
settings <- read.table('Data/CSV/Samples.csv', sep=',', header=TRUE)
Image.Dir <- ""
Cell.Image.Dir <- ""

## SETTINGS
cell.dose          <- c(0, 30, 60, 90, 120, 180, 240)/1000 * 3.89 # Gy
cell.dose.ext      <- c(cell.dose, (1 + 0:20/5))
border.dist        <- 100
border.dist.tracks <- 10
dose.cycles        <- 1
amplitude.method   <- "center-adjacent"
sd.fit.div         <- 2
detour.range       <- 5
iso_name           <- c("Ac225", "Fr221", "At217")
iso_engy           <- c(5930,    6550,    7200)

optical.param <- list(n1    = 1.517,
                      n2    = 1.7659,
                      NumAp = 1.40,
                      depth.corr = "Full")

voxel.size      <- list(x.um  = 0.0961,
                        y.um  = 0.0961,
                        z.um  = 0.49)

cell.voxel.size <- list(x.um  = 0.0961,
                        y.um  = 0.0961,
                        z.um  = 0.49*1.330/optical.param$n1)

voxel.size.HR   <- list(x.um  = 0.03968,
                        y.um  = 0.03968,
                        z.um  = 0.1441)

LET <- c(20.85, 28.2, 51.5, 72, 87.7, 112, 131, 161, 190, 201)
RBE <- c(1.7, 2.1, 3.0, 3.6, 4.3, 4.29, 4.49, 3.97, 3.67, 2.96)

LET.fit <- lm(RBE ~ I(LET) + I(LET^2))

Smode <- c("In Nucleus", "In Cytoplasm", "On Nuclear Membrane", "On Cell Membrane")

## FUNCTIONS
if (!exists("cl"))
  require(FNTD)

require(ggplot2)
require(Cairo)
require(beepr)

gauss.fit <- function(par, d, error = T) {
  m  <- par[1]
  sd <- par[2]
  rhat <- 1/sqrt(2*pi*sd^2) * exp(-0.5 * ((d$x - m)/sd)^2)
  
  if (error)
    return(sum((d$y - rhat)^2))
  else
    return(rhat)
}

gauss.plot <- function(x, mean, sd, n, bw) {
  dnorm(x = x, mean = mean, sd = sd) * n * bw
}

multihit.distribution <- function(k, dens) {
  dens$y <- dens$y/sum(dens$y)
  
  x    <- dens$x
  y1   <- dens$y
  ypr  <- y1
  
  zny      = list()
  zny[[1]] <- ypr
  
  if (k >= 2) {
    for (q in 2:k) {
      cat(paste(q, ",", sep=""))
      x    <- (x[2] - x[1]) * (0:(2*max(x)/(x[2] - x[1])))
      
      yn   <- rep(0, length(x))
      
      yadj <- c(y1,  rep(0, length(x) - length(y1)))
      ypr  <- c(ypr, rep(0, length(x) - length(ypr)))
      
      for (j in 1:length(x)) {
        yn[j] = sum(ypr[1:j]*yadj[(j + 1) - 1:j])
      }
      
      x    <- x [yn > 0]
      ypr  <- yn[1:length(x)]
      
      zny[[q]] <- ypr
    }
    
    k_s <- 1:k
    for (q in k_s)
      zny[[q]] <- c(zny[[q]],  rep(0, length(x) - length(zny[[q]])))
    
    cat("\b\n")
  }
  
  return(zny)
}

poisson.zdist <- function(zny, q, target.dose, x) {
  temp    <- rep(0,length(zny[[27]]))
  temp[1] <- ppois(0, lambda = target.dose/q_fit[1])
  
  res <- zny[[length(zny)]] * dpois(length(zny), lambda = target.dose/q) + temp
  for (i in (length(zny)-1):1) {
    res  <- res + zny[[i]] * dpois(i, lambda = target.dose/q)
  }
  
  return(res)
}

az.fit <- function(a_z, Sd = c(0), zdata, error =T) {
  Sz <- rep(0, length(zdata))
  for (i in 1:length(zdata)) {
    Sz[i] <- sum(exp(-a_z*zdata[[i]]))/length(zdata[[i]])
  }
  
  if (error)
    return(sum(Sz - Sd)^2)
  else
    return(Sz)
}

polyapply <- function(x, coeff) {
  y <- rep(0, length(x))
  for (i in 1:length(coeff)) 
    y <- y + (coeff[i])*x^(i-1)
  
  return (y)
}

get.sign <- function(number) {
  if (number >= 0)
    return("+")
  else
    return("-")
}
