source(file="include/default.R")

require(ggplot2)
require(Cairo)

files.no      <- c(25:32) # 1800 um
# files.no      <- c(33:41) # 2600 um
# files.no      <- c(42) # HR
cell.dose <- 1
rep       <- 20

theta.deg <- data.frame()

jimg <- FNTD.read.image(paste("D:/CSLM/E2/E2P1S3a_fluo.tif",sep=""))
jimg <- FNTD.create(jimg)
n    <- dim(jimg)[1:3]

for (i in settings$filename[files.no]) {
  load(paste('Data/Tracks/tracks_', i, '.rda', sep=""))
  
  fntd.dose <- FNTD.get.macro.dose(tracks, LET.file = Water.LET,
                                   voxel.size = voxel.size,
                                   image.size = list(n.x = n[1],
                                                     n.y = n[2],
                                                     n.slices = n[3]),
                                   border.distance = border.dist,
                                   dose.depth = 0,
                                   dose.layer.thickness = 8)
  
  dose.num.cycles <- 10
  dose.per.cycle  <- cell.dose/dose.num.cycles
  no.of.tracks    <- round(dose.per.cycle/fntd.dose * nrow(tracks))
  
  for (k in 1:rep) {
    for (j in 1:dose.num.cycles) {
    
      tracks.red <- tracks[sample(nrow(tracks), no.of.tracks),]
    
      theta.deg  <- rbind(theta.deg, data.frame("data"=tracks.red$theta.deg))
    }
  }
}

Cairo(width = 600, height = 400, file=paste("Data/Plots/2600_angular.png", sep=""), type="png")
# CairoWin()
# plot(ggplot(theta.deg, aes(x=data))+ 
#        geom_density(lwd=1, alpha=0.3)+
#        xlab("Theta Angle (deg)")+
#        ylab("Probability density")+
#        xlim(0,50)+
#        theme(legend.position="bottom",legend.direction="horizontal",legend.title=element_blank())+
#        guides(fill=guide_legend(nrow=1,byrow=TRUE)))

plot(ggplot(theta.deg, aes(x=data, fill=..count..))+ 
       geom_histogram(lwd=1, alpha=1, position="dodge", binwidth=1)+
       scale_fill_gradient("Count", low = "green", high = "red")+
       xlab("Theta Angle (deg)")+
       ylab("Frequency")+
       xlim(0,60)+
       
       theme(legend.position="none", legend.direction="horizontal", 
             legend.title=element_blank(), text = element_text(size=15), 
             panel.border = element_rect(colour = "black", fill=NA, size=.5))+
       
       guides(fill=guide_legend(nrow=1,byrow=TRUE)))
dev.off()
