source(file="include/default.R")

require(ggplot2)
require(Cairo)

# files.no      <- c(29:32) # 1800 um
# files.no      <- c(33:41) # 2600 um
files.no      <- c(42:45) # HR

dat = data.frame()

for (i in settings$filename[files.no]) {
  load(paste('Data/Tracks/tracks_', i, '.rda', sep=""))
  
  for(i in 1:nrow(tracks)) {
    track <- tracks[i,]
    
    x.t <- matrix(unlist(track$length.vec), ncol = 1, byrow = TRUE)[,1]
    y.t <- matrix(unlist(track$amplitude),  ncol = 1, byrow = TRUE)[,1]
    
    dat = rbind(dat,
                data.frame(x = abs(x.t - max(x.t)),
                           y = y.t/mean(y.t)))
  }
}

bins <- 10
binw <- 8/bins
x <- 0:(bins-1) * max(dat$x)/bins
y.m <- rep(0, bins)
y.sd <- rep(0, bins)

for (i in 1:bins) {
  y.m[i]  <- mean(dat[dat$x >= x[i] & dat$x < x[i] + binw,]$y)
  y.sd[i] <-   sd(dat[dat$x >= x[i] & dat$x < x[i] + binw,]$y)
}

Cairo(width = 1600, height = 1000, dpi = 200, file=paste("Data/Plots/1800_bragg_HR.png", sep=""), type="png")
par(mar = c(5,6,1,2))
plot(dat$x, dat$y, ylim = c(0, 2), xlim = c(0, 8), pch=".", xlab = expression(paste("Distance from end of track (", mu, "m)", sep = "")), ylab = "Spots intensity (normalized)", cex.lab=1.5, cex.axis=1.5)
# arrows(x, y.m-y.sd, x, y.m+y.sd, length=0.05, angle=90, code=3)
dev.off()

LET <- read.table(LET.files$al2o3, sep=',')

Cairo(width = 1600, height = 1000, dpi = 200, file=paste("Data/Plots/al2o3_bragg.png", sep=""), type="png")
par(mar = c(5,6,1,2))
plot(LET$V3, LET$V2, ylim = c(0, 600), xlim = c(0, 8), type="l",
     xlab = expression(paste("Distance from end of track (", mu, "m)", sep = "")), ylab = expression(paste("LET (kev/", mu, "m)", sep = "")), cex.lab=1.5, cex.axis=1.5)
dev.off()