source(file="include/default.R")

require(ggplot2)
require(Cairo)

# files.no      <- c(25) # 1800 um
# files.no      <- c(33:41) # 2600 um
# files.no      <- c(45) # HR
files.no      <- c(47) # HR2

exp.fit <- function(b, a, x, y = NULL, error = TRUE) {
  rhat <- a + (1-a)*(1 - exp(-b*x))
  
  if (error)
    return(sum((y - rhat)^2))
  else
    return(rhat)
}

dat = data.frame()

for (i in settings$filename[files.no]) {
  load(paste('Data/Tracks/tracks_', i, '.rda', sep=""))
  
  for(i in 4) {
    track <- tracks[i,]
    
    x.t <- matrix(unlist(track$length.vec), ncol = 1, byrow = TRUE)[,1]
    y.t <- matrix(unlist(track$amplitude),  ncol = 1, byrow = TRUE)[,1]
    
    dat = rbind(dat,
                data.frame(x = abs(x.t - max(x.t)),
                           y = y.t/mean(y.t)))
  }
}

a   <- tail(dat$y, 1)
fit <- optim(1, exp.fit, x = dat$x, y = dat$y, a = a,
             method="BFGS", control=list(reltol=1e-3))
cat("Endpoint:", -log(1 - (.95 - a)/(1 - a))/fit$par[1])

plot(dat$x, dat$y, ylim = c(0, 2),
     xlab = "Range (um)", ylab = "Spots intensity (normalized)",
     main = "Track intensity profile")
lines(dat$x, exp.fit(fit$par[1], a, dat$x, error = F))