source(file="include/default.R")

require(ggplot2)
require(Cairo)

# files.no      <- c(25:32) # 1800 um
files.no      <- c(33:41) # 2600 um
# files.no      <- c(42:45) # HR

total.energy <- F
Sim.file     <- 'Data/CSV/Energy_spectrum_1800.csv'

plot.name    <- "1800_energy_Full"
file.plot    <- F
binwidth     <- 50
lower.r      <- 0
higher.r     <- 5500

if (max(files.no) > 41) {
  voxel.size <- voxel.size.HR
  plot.name  <- paste(plot.name, "_HR", sep = "")
}

energy.df <- data.frame()

for (i in settings$filename[files.no]) {
  load(paste('Data/Tracks/tracks_', i, '.rda', sep=""))
#   tracks <- FNTD.get.energy(tracks, LET.files = LET.files, layer.thickness = Layers,
#                             procedural.track.length = T)
  
  if (total.energy)
    energy.df <- rbind(energy.df,
                       data.frame("data"=tracks$energy.total))
  else 
    energy.df <- rbind(energy.df,
                       data.frame("data"=matrix(unlist(tracks$energy), ncol = 3, byrow = TRUE)[,1]))
}

energy.df <- data.frame("data"=energy.df$data[energy.df$data < higher.r])

## Calculate fit
dens        <- density(energy.df$data, na.rm = T)
whole.fit   <- optim(c(dens$x[dens$y==max(dens$y)]-50, sd(energy.df$data, na.rm = T)/3, max(dens$y)),
                     gauss.fit, d=dens, method="BFGS", control=list(reltol=1e-9))
whole.fit.y <- gauss.fit(whole.fit$par, dens, error = F)

## Output whole.fit
cat("X:", voxel.size$x.um, "um , Y:", voxel.size$y.um, " um, Z:", voxel.size$z.um, "um\n\n")
cat("E peak:\t\t", round(dens$x[dens$y==max(dens$y)],0), "keV\n")
cat("E sd:\t\t", round(whole.fit$par[2],0), "keV =", 2.35E-3*round(whole.fit$par[2],0), "um\n")

## Calculate bins and deviation from fit
bins <- hist(energy.df$data, breaks=seq(lower.r,higher.r,by=binwidth), plot=FALSE)
bins$counts_sub <- bins$counts - gauss.plot(x = bins$mids, mean = whole.fit$par[1], sd = whole.fit$par[2],  n = nrow(energy.df), bw = binwidth)

plot.df <- data.frame(counts     = bins$counts,
                      counts_sub = bins$counts_sub,
                      mids       = bins$mids)

# Calculate Simulation SD
Sim.energy             <- read.table(Sim.file, sep=',', header = TRUE)
Sim.energy             <- data.frame(matrix(unlist(Sim.energy), nrow=nrow(Sim.energy), byrow=F))

Sim.energy$X2          <- Sim.energy$X2/sum(Sim.energy$X2)
Sim.energy$X1          <- Sim.energy$X1*1000
Sim.mean               <- sum(Sim.energy$X1 * Sim.energy$X2)
Sim.sd                 <- sqrt(sum(Sim.energy$X1^2 * Sim.energy$X2) - Sim.mean^2)

Sim.energy$X2          <- Sim.energy$X2*max(bins$counts)/max(Sim.energy$X2)

cat("Sim peak:\t", round(Sim.energy[Sim.energy$X2==max(Sim.energy$X2),]$X1,0), "keV\n")
cat("Sim sd:\t\t", round(Sim.sd,0), " keV\n")
cat("System sd:\t", round(sqrt(whole.fit$par[2]^2 - Sim.sd^2),0), "keV\n")

if (file.plot) {
  Cairo(width = 1600, height = 1000, dpi = 200, file=paste("Data/Plots/", plot.name, ".png", sep=""), type="png")
} else
  CairoWin()
  par(mar = c(5,6,1,1))
  plot(plot.df$mids, plot.df$counts, xlim=c(0, 6000), type="h", ylab="Frequency", xlab="Energy (keV)", cex.lab=1.5, cex.axis=1.5)
  lines(Sim.energy$X1, Sim.energy$X2, lwd=2)

if(file.plot) dev.off()