source(file="include/default.R")

require(ggplot2)
require(Cairo)

files.no      <- c(29:32) # 1800 um
# files.no      <- c(33:41) # 2600 um
# files.no      <- c(42:45) # HR

file.plot <- F
maxb      <- .4

range.df <- data.frame()

for (i in settings$filename[files.no]) {
  load(paste('Data/Tracks/tracks_', i, '.rda', sep=""))
  
  range.df <- rbind(range.df,
                    data.frame("csda"=tracks$csda.length,
                               "proj"=tracks$proj.length,
                               "detour"=tracks$detour.o,
                               "energy"=matrix(unlist(tracks$energy), ncol = 3, byrow = TRUE)[,1]))
}

range.df.adj <- range.df[!is.na(range.df$detour) & range.df$detour <= maxb,]

dens        <- density(range.df.adj$csda, na.rm = T)
whole.fit   <- optim(c(dens$x[dens$y==max(dens$y)]+1, sd(range.df.adj$csda, na.rm = T)/3, max(dens$y)),
                     gauss.fit, d=dens, method="BFGS", control=list(reltol=1e-9))
whole.fit.y <- gauss.fit(whole.fit$par, dens, error = F)

## Output whole.fit
cat("CSDA peak:\t", round(dens$x[dens$y==max(dens$y)],0), "um\n")
cat("CSDA sd:\t", round(whole.fit$par[2],0), "um\n")

cat("Scattering:\t\t", round(mean(range.df.adj$detour),3), "\n")

if (file.plot) {
  Cairo(width = 1600, height = 1000, dpi = 200, file=paste("Data/Plots/1800_scatter_HR.png", sep=""), type="png")
} else
  CairoWin()
  par(mar = c(5,6,1,2))
  hist(range.df.adj$detour, breaks = seq(0, maxb, by=0.005),
     xlab = "Relative Scattering", ylab = "Frequency", main="", cex.lab=1.5, cex.axis=1.5)
  box()
  
if(file.plot) dev.off()