################################
# FNTDimage CLASS
################################
setClass( Class            = "FNTDimage",
          representation   = representation( data                = "matrix",
		                                     sizeX               = "numeric",
											 sizeY               = "numeric",
											 sizeZ               = "numeric",
											 rescans             = "numeric",
											 dwell.time.us       = "numeric",
											 laser.power.percent = "numeric",
											 pinhole.diameter.um = "numeric",
											 lens.type           = "character",
											 tracks              = "FNTDtracks",
											 mask                = "FNTDmask"),
		  prototype        = list( data = matrix( data     = NA,
		                                          nrow     = 0,
												  ncol     = 4,
												  dimnames = list( NULL, 
												                   c("x", "y", "z", "value"))),
									sizeX               = 0,
									sizeY               = 0,
									sizeZ               = 0,
									rescans             = 0,
									dwell.time.us       = 0,
									laser.power.percent = 0,
									pinhole.diameter.um = 0,
									lens.type           = "N/A",
									mask                = new("FNTDmask"),
									tracks              = new("FNTDtracks")))

################################
# FNTDimage METHODS
################################
setMethod(f          = "[", 
          signature  = "FNTDimage",
		  definition = function(x, i){
		   
		   if(length(i) == 0){
		     return()
		   }
		   
		   if(!(class(i)%in%c("integer", "numeric"))){
		     stop("Non-numeric indices given.")
		   }
		   
		   if((sum(i<0))|(sum(i>(x@sizeX*x@sizeY*x@sizeZ)))){
		     stop("Indices out of limits.")
		   }
		   
		   return(x@data[i,4])
		  })

setMethod(f          = "length", 
          signature  = c("FNTDimage"),
          definition = function(x) {
                           return(length(x@data[,4]))
})

setMethod(f          = "sum", 
          signature  = c("FNTDimage"),
          definition = function(x) {
						     return(sum(x@data[x@mask@mask != 0,4]))
})
						   
setMethod(f          = "mean", 
          signature  = c("FNTDimage"),
          definition = function(x) {
						     return(mean(x@data[x@mask@mask != 0,4]))
})

setMethod(f          = "median", 
          signature  = c("FNTDimage"),
          definition = function(x) {
            return(median(x@data[x@mask@mask != 0,4]))
          })

setMethod(f          = "sd", 
          signature  = c("FNTDimage"),
          definition = function(x) {
						     return(sd(x@data[x@mask@mask != 0,4]))
})

setMethod(f          = "min", 
          signature  = c("FNTDimage"),
          definition = function(x) {
						     return(min(x@data[x@mask@mask != 0,4]))
})

setMethod(f          = "max", 
          signature  = c("FNTDimage"),
          definition = function(x) {
						     return(max(x@data[x@mask@mask != 0,4]))
})

setMethod(f          = "+", 
          signature  = c("FNTDimage", "numeric"),
          definition = function(e1, e2) {
                           if(length(e2) != 2){
						     stop("Second operand has to be vector of two numeric values (x- and y-shift)")
						   }
						   return(new("FNTDimage",
						               data     = matrix( data = c( e1@data[,1] + e2[1],
									                                e1@data[,2] + e2[2],
																	e1@data[,3],
																	e1@data[,4]),
														  ncol = 4,
														  dimnames = list( NULL, 
												                   c("x", "y", "z", "value"))),
									   sizeX    = e1@sizeX,
									   sizeY    = e1@sizeY,
									   sizeZ    = e1@sizeZ,
									   tracks   = e1@tracks,
									   mask     = e1@mask))
						   })

setMethod(f          = "*", 
          signature  = c("FNTDimage", "numeric"),
          definition = function(e1, e2) {
                           if(length(e2) != 1){
						     stop("Second operand has to be a single value (e.g. pixel to um conversion factor)")
						   }
						   return(new("FNTDimage",
						               data     = matrix( data = c( e1@data[,1] * e2,
									                                e1@data[,2] * e2,
																	e1@data[,3],
																	e1@data[,4]),
														  ncol = 4,
														  dimnames = list( NULL, 
												                   c("x", "y", "z", "value"))),
									   sizeX    = e1@sizeX,
									   sizeY    = e1@sizeY,
									   tracks   = e1@tracks,
									   mask     = e1@mask))
						   })

setMethod(f          = "/", 
          signature  = c("FNTDimage", "numeric"),
          definition = function(e1, e2) {
						   return(new("FNTDimage",
						               data     = matrix( data = c( e1@data[,1],
									                                e1@data[,2],
																	e1@data[,3],
																	e1@data[,4] / e2),
														  ncol = 4,
														  dimnames = list( NULL, 
												                   c("x", "y", "z", "value"))),
									   sizeX    = e1@sizeX,
									   sizeY    = e1@sizeY,
									   sizeZ    = e1@sizeZ,
									   tracks   = e1@tracks,
									   mask     = e1@mask))
						   })
						   
setMethod(f          = "+", 
          signature  = c("FNTDimage", "FNTDimage"),
          definition = function(e1, e2) {
						   if((e1@sizeX != e2@sizeX)|(e1@sizeY != e2@sizeY)|(e1@sizeZ != e2@sizeZ)){
						     stop("Images have to be of same size.")
						   }
						   return(new("FNTDimage",
						               data     = matrix( data = c( e1@data[,1],
									                                e1@data[,2],
																	e1@data[,3],
																	e1@data[,4] + e2@data[,4]),
														  ncol = 4,
														  dimnames = list( NULL, 
												                   c("x", "y", "z", "value"))),
									   sizeX    = e1@sizeX,
									   sizeY    = e1@sizeY,
									   sizeZ    = e1@sizeZ,
									   tracks   = e1@tracks,
									   mask     = e1@mask))
						   })

setMethod(f          = "-", 
          signature  = c("FNTDimage", "FNTDimage"),
          definition = function(e1, e2) {
						   if((e1@sizeX != e2@sizeX)|(e1@sizeY != e2@sizeY)|(e1@sizeZ != e2@sizeZ)){
						     stop("Images have to be of same size.")
						   }
						   return(new("FNTDimage",
						               data     = matrix( data = c( e1@data[,1],
									                                e1@data[,2],
																	e1@data[,3],
																	e1@data[,4] - e2@data[,4]),
														  ncol = 4,
														  dimnames = list( NULL, 
												                   c("x", "y", "z", "value"))),
									   sizeX    = e1@sizeX,
									   sizeY    = e1@sizeY,
									   sizeZ    = e1@sizeZ,
									   tracks   = e1@tracks,
									   mask     = e1@mask))
						   })

setMethod(f          = "*", 
          signature  = c("FNTDimage", "FNTDimage"),
          definition = function(e1, e2) {
						   if((e1@sizeX != e2@sizeX)|(e1@sizeY != e2@sizeY)|(e1@sizeZ != e2@sizeZ)){
						     stop("Images have to be of same size.")
						   }
						   return(new("FNTDimage",
						               data     = matrix( data = c( e1@data[,1],
									                                e1@data[,2],
																	e1@data[,3],
																	e1@data[,4] * e2@data[,4]),
														  ncol = 4,
														  dimnames = list( NULL, 
												                   c("x", "y", "z", "value"))),
									   sizeX    = e1@sizeX,
									   sizeY    = e1@sizeY,
									   sizeZ    = e1@sizeZ,
									   tracks   = e1@tracks,
									   mask     = e1@mask))
						   })
						   
setMethod(f          = "/", 
          signature  = c("FNTDimage", "FNTDimage"),
          definition = function(e1, e2) {
						   if((e1@sizeX != e2@sizeX)|(e1@sizeY != e2@sizeY)|(e1@sizeZ != e2@sizeZ)){
						     stop("Images have to be of same size.")
						   }
						   return(new("FNTDimage",
						               data     = matrix( data = c( e1@data[,1],
									                                e1@data[,2],
																	e1@data[,3],
																	e1@data[,4] / e2@data[,4]),
														  ncol = 4,
														  dimnames = list( NULL, 
												                   c("x", "y", "z", "value"))),
									   sizeX    = e1@sizeX,
									   sizeY    = e1@sizeY,
									   sizeZ    = e1@sizeZ,
									   tracks   = e1@tracks,
									   mask     = e1@mask))
						   })



setMethod(f          = "plot", 
          signature  = "FNTDimage",
          definition = function(x, y, ...) {
          
		  if(class(x) != "FNTDimage"){
		    stop("Please provide FNTDimage object")
		  }
		  
		  img        <- x
		  cur.call   <- as.list(sys.call())
		  args.given <- names(cur.call)
		  
		  if( !("tracks"%in%args.given)){
			  tracks <- FALSE
		  }else{
			  tracks <- eval(cur.call$tracks)
		  }

		  if( !("mask"%in%args.given)){
			  mask   <- FALSE
		  }else{
			  mask   <- eval(cur.call$mask)
		  }

		  if( !("frequency"%in%args.given)){
			  frequency   <- FALSE
		  }else{
			  frequency   <- eval(cur.call$frequency)
		  }
		  
		  if( !("double.labels"%in%args.given)){
			  double.labels   <- FALSE
		  }else{
			  double.labels   <- eval(cur.call$double.labels)
		  }

		  if(mask == TRUE){
			  mask <- img@mask@mask
		  }else{
			  mask <- rep(1, length(img))
		  }
			
		  
		  if( !("col.regions"%in%args.given)){
			  col.regions <- grey((0:100)/100)
		  }else{
			  col.regions <- eval(cur.call$col.regions)
		  }

		  if( "at"%in%args.given){
			  at          <- eval(cur.call$at)
		  }else{
			  at          <- seq(from = min(img@data[,4]), to  = max(img@data[,4]), length.out = 101)
		  }

          xlab        <- "x"
		  ylab        <- "y"
		  aspect      <- (max(img@data[,2]) - min(img@data[,2]))/(max(img@data[,1]) - min(img@data[,1]))
		  cuts        <- length(col.regions)-1
		  xlim        <- c(min(img@data[,1]),max(img@data[,1]))
		  ylim        <- c(max(img@data[,2]),min(img@data[,2]))
		  subset      <- mask > 0
		  
		  if(tracks == TRUE & length(img@tracks)>0){
			  panel       = function(...){ panel.levelplot(...)
										   panel.xyplot(img@tracks@x, img@tracks@y, pch = 13, cex = 2.5)
										}
		  }else{
			  panel       = function(...){ panel.levelplot(...)
										}
		  }
		  
		  if(frequency){
		    values <- get.frequency.MHz(img)
		  }else{
		    values <- img@data[,4]
		  }
		  
		  if(double.labels){
		     tickos   <- head(pretty(range(values))[-1], -1)
			 if(frequency){
			   tickos2 <- tickos * get.norm.factor(img)
			   titles  <- "frequency / MHz\n(abs. counts)"
			 }else{
			   tickos2 <- tickos / get.norm.factor(img)
			   titles  <- "abs. counts\n(frequency / MHz)"
			 }
  			 colorkey <- list(    space = 'right',
								  title = "HHHAAAAALLLLO",
								  labels = list( at    = tickos,
                                                 labels = paste(as.character(tickos), 
												                " (",
																as.character(signif(tickos2, digits = 2)),
																")",
																sep = "")))
		  }else{
			 if(frequency){
			   titles  <- "frequency / MHz"
			 }else{
			   titles  <- "abs. counts"
			 }
		     colorkey <- list( space = 'right',
			                   title = title)
		  }
		  
		  if(img@sizeZ == 1){
		    show.strip <- FALSE
		  }else{
		    show.strip <- TRUE
		  }
											  
		  levelplot( values ~ img@data[,1]*img@data[,2]|sprintf("z = %d", img@data[,3]),
										panel       = panel,
										xlab        = xlab,
										ylab        = ylab,
										strip       = show.strip,
										subset      = subset,
										aspect      = aspect,
										cuts        = cuts,
										col.regions = col.regions,
										at          = at,
										colorkey    = colorkey,
										xlim        = xlim,
										ylim        = ylim,
										useRaster   = TRUE,
										as.table    = TRUE,
										legend      = list( top = list( fun = grid::textGrob(titles, 
										                                                     just = c("right", "top"), 
																							 x = 1.05, 
																							 y = 1.0))))
		})
									

		
###############################
# FNTD image FUNCTIONS
###############################

bin.pixels <- function(img, n.pixels) {
						   if(class(img) != "FNTDimage"){
						     stop("Object img has to be of class \"FNTDimage\".")
						   }
						   if((length(n.pixels) != 1)|(class(n.pixels)!="numeric")|(n.pixels < 1)){
						     stop("Object n.pixels has to be a single numeric value.")
						   }
						   
						   if(n.pixels - floor(n.pixels) > 0){
						     n.pixels <- floor(n.pixels)
							 cat("Truncating n.pixels to", n.pixels, "\n")
						   }
						   
						   x <- floor(img@data[,1]/n.pixels)
						   new.y <- floor(img@data[,2]/n.pixels)
						   return(new("FNTDimage",
						               data     = matrix( data = c( as.vector(tapply(x, list(x, new.y), mean)),
																	as.vector(tapply(new.y, list(x, new.y), mean)),
																	as.vector(tapply(img@data[,3], list(x, new.y), mean)),
																	as.vector(tapply(img@data[,4], list(x, new.y), mean))),
														  ncol = 4,
														  dimnames = list( NULL, 
												                   c("img", "y", "z", "value"))),
									   sizeX    = length(x),
									   sizeY    = length(new.y),
									   tracks   = new("FNTDtracks"),
									   mask     = new("FNTDmask")))
						   }

generate.coordinates <- function(x.size, y.size, z.size, x.min = 0, y.min = 0, z.min = 1){
  return( list( x = rep(sort(rep((1:y.size)-1, x.size)) + x.min, z.size),
                y = rep(rep((1:x.size)-1, y.size) + y.min, z.size),
				        z = sort(rep(z.min:(z.min+z.size-1), x.size * y.size))))
}

create.image <- function(size=512,
                         bkg.mean=100, bkg.sd = 8,
                         no.tracks = 0, track.core.mean = 200, track.core.sd = 20, track.width.mean = 1.2, track.width.sd = 0,
                         track.list = NULL, shot.noise=TRUE, type=1){
  
  generate.coordinates <- function(x.size, y.size, z.size, x.min = 0, y.min = 0, z.min = 1){
    return( list( x = rep(sort(rep((1:y.size)-1, x.size)) + x.min, z.size),
                  y = rep(rep((1:x.size)-1, y.size) + y.min, z.size),
                  z = sort(rep(z.min:(z.min+z.size-1), x.size * y.size))))
  }
  
  #Creation of the background image
  coords   <- generate.coordinates(x.size = size, y.size = size, z.size = 1)
  if( bkg.sd == 0){ # No noise
    value    <- rep(bkg.mean, size * size)
  }
  if( bkg.sd < 0){  # Poisson noise
    value    <- rpois(n = size * size, lambda = bkg.mean)
  }
  if( bkg.sd > 0){  # Custom Gaussian noise
    value    <- rnorm(n = size * size, mean = bkg.mean, sd = bkg.sd)
  }
  
  background <- new( "FNTDimage",
                     data     = matrix( data = c( coords$x,
                                                  coords$y,
                                                  coords$z,
                                                  value),
                                        ncol = 4,
                                        dimnames = list( NULL, 
                                                         c("x", "y", "z", "value"))),
                     sizeX    = size,
                     sizeY    = size,
                     sizeZ    = 1,
                     tracks   = new("FNTDtracks"),
                     mask     = new("FNTDmask"))
  
  #Adding tracks (if desired)
  if((no.tracks==0)&&(is.null(track.list))){
    
    return(background)         #No track spots desired? Just return background image.
    
  }else{
    
    if(is.null(track.list)){   #No track list given? Create one from user defined values!
      
      track.list <- data.frame(x          = runif(n = no.tracks,     #Tracks within image boundaries for track.width.sd=0
                                                  min = track.width.mean,
                                                  max = size - track.width.mean),
                               y          = runif(n = no.tracks,     #Tracks within image boundaries for track.width.sd=0
                                                  min = track.width.mean,
                                                  max = size - track.width.mean))
    }
    
    no.tracks = length(track.list$x)
    
    tls <- cbind( track.list, data.frame(
      core.value = rnorm(n    = no.tracks,
                         mean = track.core.mean,
                         sd   = track.core.sd),
      width      = rnorm(n    = no.tracks,
                         mean = track.width.mean,
                         sd   = track.width.sd)))
    
    #Create track spots around the track core 
    tracks <- .C( "add_gauss",
                  n_pixel    = as.integer(length(background)),
                  x          = as.double(background@data[,1]),
                  y          = as.double(background@data[,2]),
                  value      = as.double(background@data[,4]),
                  n_tracks   = as.integer(nrow(track.list)),
                  x_pos      = as.double(tls$x),
                  y_pos      = as.double(tls$y),
                  core_value = as.double(tls$core.value),
                  width      = as.double(tls$width),
                  PACKAGE = "FNTD")
    
    #Create separate image for combined image of background and tracks
    img <- background
    if(shot.noise==TRUE){img@data[,4] <- unlist(lapply(tracks$value, rpois, n=1))}else{img@data[,4] <- tracks$value}
    img@tracks <- new("FNTDtracks", track.no = 1:length(track.list[,1]) , x = track.list[,1], y = track.list[,2])
    
    if(type==1){
      #Return image only
      return(img)}else{
        #Return a list of both images    
        return(list(img, background))}
    
  }
  
}

get.substack <- function(img, z.start, z.end){
   if(class(img) != "FNTDimage"){
	 stop("Object img has to be of class \"FNTDimage\".")
   }

   if((z.start > z.end)|(z.start <= 0)|(z.start > img@sizeZ)|(z.end <= 0)|(z.end > img@sizeZ)){
     stop("z values or range invalid.")
   }
   
   ii <- (img@data[,3] >= z.start) & (img@data[,3] <= z.start)
   
   return(new("FNTDimage", data = img@data[ii,],
							sizeX = img@sizeX,
							sizeY = img@sizeY,
							sizeZ = z.end - z.start + 1,
						    rescans             = img@rescans,
							dwell.time.us       = img@dwell.time.us,
							laser.power.percent = img@laser.power.percent,
							pinhole.diameter.um = img@pinhole.diameter.um,
							lens.type           = img@lens.type,
							tracks = new("FNTDtracks"),
							mask   = new("FNTDmask", mask = rep(1, sum(ii)))))
}
	
get.data <- function(img, use.mask = TRUE){
   if(class(img) != "FNTDimage"){
	 stop("Object img has to be of class \"FNTDimage\".")
   }
   return(img@data[img@mask@mask > 0,])
}

get.values <- function(img, use.mask = TRUE){
   if(class(img) != "FNTDimage"){
	 stop("Object img has to be of class \"FNTDimage\".")
   }
   return(img@data[img@mask@mask > 0, 4])
}
	
get.norm.factor <- function(img){
   if(class(img) != "FNTDimage"){
	 stop("Object img has to be of class \"FNTDimage\".")
   }
   return(img@rescans * img@dwell.time.us * img@laser.power.percent / 100)
}

get.frequency.MHz <- function(img, use.mask = TRUE){
   if(class(img) != "FNTDimage"){
	 stop("Object img has to be of class \"FNTDimage\".")
   }
   return(img@data[img@mask@mask > 0,4] / get.norm.factor(img))
}

### INDEXING ROUTINES ###
# ImageJ has zero-based pixel indices,
# cycling through x first
xy.to.idx <- function(x, y, x.size){
   return(x.size * y + x)
}

idx.to.xy <- function(idx, x.size){
  return(list( x = idx%%x.size,
               y = floor(idx/x.size)))
}


