################################
# FNTDbigimage CLASS
################################
setClass( Class            = "FNTDbigImage",
          representation   = representation( dataptr             = "externalptr",
		                                     sizeX               = "numeric",
											 sizeY               = "numeric"),
		  prototype        = list(  dataptr             = NULL,
									sizeX               = 0,
									sizeY               = 0))
					
################################
# FNTDbigimage METHODS
################################
setMethod(f          = "[", 
          signature  = "FNTDbigImage",
		  definition = function(x, i){
		   
		   if(length(i) == 0){
		     return()
		   }
		   
		   if(!(class(i)%in%c("integer", "numeric"))){
		     stop("Non-numeric indices given.")
		   }
		   
		   if((sum(i<0))|(sum(i>(x@sizeX*x@sizeY)))){
		     stop("Indices out of limits.")
		   }
		   
		   return(.Call("get_big_image_data",
		                x@dataptr, i,
						PACKAGE = "FNTD"))
		  })


################################
# FNTDbigimage FUNCTIONS
################################
read.raw.image <- function(filename, x.size, y.size, type = "16bit"){
 if(type != "16bit"){
   stop("Invalid type")}
   
 if((x.size < 1)|(y.size < 1)|(length(x.size)>1)|(length(y.size)>1)|(class(x.size)!="numeric")|(class(y.size)!="numeric")){
   stop("Size must be single positive integers")}
   
 if(!file.exists(filename)){
   stop("File not found")}

 if(file.info(filename)$size < x.size * y.size){
   stop("Given image size larger than file size.")}
   
 return(new("FNTDbigImage", dataptr = .Call( "read_raw_image",
                                             filename,
                                             PACKAGE = "FNTD"),
							sizeX   = x.size,
							sizeY   = y.size))
}

get.FNTD.image <- function(bimg, x.min, y.min, x.max, y.max, unprotected = FALSE){
   if(class(bimg) != "FNTDbigImage"){
	 stop("Object bimg has to be of class \"FNTDbigImage\".")
   }
   
   if((x.min < 0)|(y.min < 0)|(x.max < 0)|(y.max < 0)|
      (length(x.min)>1)|(length(y.min)>1)|(length(x.max)>1)|(length(y.max)>1)|
	  (class(x.min)!="numeric")|(class(y.min)!="numeric")|(class(x.max)!="numeric")|(class(y.max)!="numeric")){
     stop("x.min, x.max, y.min, y.max must be single positive integers")}

   if((x.max < x.min)|(y.max < y.min)){
	 stop("Invalid size given.")
   }

   x.size        <- x.max - x.min + 1
   y.size        <- y.max - y.min + 1

   if((x.size * y.size > 1e6)&(!unprotected)){
     stop("You are about to handle a very big image which is likely to stall your system. Set argument \"unprotect\" to TRUE if this operation is really what you want.")
   }
   
   coords        <- generate.coordinates(x.size = y.size, y.size = x.size, z.size = 1, x.min = x.min, y.min = y.min, z.min = 0)
   
   return(new("FNTDimage", data = matrix( data = c(coords$x,
                                                   coords$y,
												   coords$z,
												   bimg[xy.to.idx(x = coords$x, y = coords$y, x.size = bimg@sizeX)]), 
												   ncol = 4,
												   dimnames = list( NULL, 
										                            c("x", "y", "z", "value"))),
							sizeX = x.size,
							sizeY = y.size,
							sizeZ = 1,
							tracks = new("FNTDtracks"),
							mask   = new("FNTDmask")))
}


