################################
# FNTDtracks CLASS
################################
setClass( Class            = "FNTDtracks",
          representation   = representation( track.no    = "numeric",
		                                     x           = "numeric",
											 y           = "numeric"),
		  prototype        = list( track.no    = 0,
		                           x           = 0,
								   y           = 0))

################################
# FNTDtracks METHODS
################################
setMethod(f          = "length", 
          signature  = "FNTDtracks",
          definition = function(x) {
                           return(length(x@track.no))})

setMethod(f          = "c", 
          signature  = c("FNTDtracks"),
          definition = function(x, ...) {
                           if(nargs()<3){
						      cPair(x, ...)
						   }else{
						      c(x, c(...))
						   }})
	
setMethod(f          = "+", 
          signature  = c("FNTDtracks", "numeric"),
          definition = function(e1, e2) {
                           if(length(e2) != 2){
						     stop("Second operand has to be vector of two numeric values (x- and y-shift)")
						   }
						   return(new("FNTDtracks",
						               track.no = e1@track.no,
									   x        = e1@x + e2[1],
									   y        = e1@y + e2[2]))
						   })

setMethod(f          = "*", 
          signature  = c("FNTDtracks", "numeric"),
          definition = function(e1, e2) {
                           if(length(e2) != 1){
						     stop("Second operand has to be a single value (e.g. pixel to um conversion factor)")
						   }
						   return(new("FNTDtracks",
						               track.no = e1@track.no,
									   x        = e1@x * e2,
									   y        = e1@y * e2))
						   })

setMethod(f          = "&", 
          signature  = c("FNTDtracks", "FNTDtracks"),
          definition = function(e1, e2) {
                           x.range1 <- range(e1@x)
                           x.range2 <- range(e2@x)
                           y.range1 <- range(e1@y)
                           y.range2 <- range(e2@y)
						   mm       <- matrix( data = c( max(x.range1[1], x.range2[1]),
						                                 min(x.range1[2], x.range2[2]),
                                                         max(y.range1[1], y.range2[1]),
                                                         min(y.range1[2], y.range2[2])),
											   ncol = 2,
											   nrow = 2,
											   dimnames = list( c("min", "max"),
											                    c("x", "y")))														 
						   return(mm)
						   })
						   
setMethod(f          = "max", 
          signature  = c("FNTDtracks"),
          definition = function(x) {
                           return(c(max(x@x), max(x@y)))})

setMethod(f          = "plot", 
          signature  = "FNTDtracks",
          definition = function(x) {

			xyplot(x@y ~ x@x,
       			      xlab        = "x axis",
                      ylab        = "y axis")
					  })
					  
################################
# FNTDtracks FUNCTIONS
################################
cPair <- function(x, y){
   return( new( "FNTDtracks",
                track.no = 1:(length(x) + length(y)),
				x        = c(x@x, y@x),
				y        = c(x@y, y@y)))
}

read.detected.tracks <- function(filename, fileformat = "ImageJ"){
   if(!fileformat%in%c("ImageJ", "Prague")){
     stop("Unknown file format given")
   }
   if(fileformat == "ImageJ"){
     tmp <- read.table( filename,
                        header   = FALSE,
                        skip     = 1,
                        dec      = ",",
                        sep      = " ")
     # swap x and y
	 tmptmp  <- tmp[,2]
	 tmp[,2] <- tmp[,1]
	 tmp[,1] <- tmptmp
   }
   
   if(fileformat == "Prague"){
     tmp  <- read.table( filename,
                         skip = 45,
                         sep = ",",
                         dec = ".",
                         header = TRUE)[,c(2,3)]
   }
   
   return(  new( "FNTDtracks",
                  track.no   = 1:nrow(tmp),
				  x          = tmp[,1] - 0.5,   # ACHTUNG!!! Unterschiedliche Reprsentationen in ETH Mosaic und R!!!
				  y          = tmp[,2] - 0.5))  # TODO : VEREINHEITLICHEN
}


transformTrackPositions <- function( trackPositions,
                                     markers1,
					                 markers2){
	
	if(class(trackPositions) != "FNTDtracks"){
	  stop("Argument \" trackPositions \" has to be of class \"FNTDtracks\".")
	}

	# Repack data to transform into matrix
	mm  <- matrix(ncol = length(trackPositions), nrow = 2)
	mm[1,] <- trackPositions@x
	mm[2,] <- trackPositions@y
	
	# find transformation matrix
	m.trans <- findTransformation(markers1, markers2)
	
	# Transform and return as matrix
	m.res           <- t(m.trans%*%(mm - c(markers2@x[1], markers2@y[1])) + c(markers1@x[1], markers1@y[1]))
	return(new ( "FNTDtracks",
	              track.no   = 1:nrow(m.res),
				  x          = m.res[,1],
				  y          = m.res[,2]))
}

