crop.image <- function(img, x.min, y.min, x.max, y.max, invert = FALSE){
  if(class(img) != "FNTDimage"){
    stop("Argument \" img \" has to be of class \"FNTDimage\".")
  }
  
  ii <- (img@data[,1] >= x.min) &
    (img@data[,1] <=  x.max) &
    (img@data[,2] >= y.min) &
    (img@data[,2] <=  y.max)
  
  if(invert){
    ii <- !ii
  }
  
  if(sum(ii)==0){
    stop("Cropped image would contain no pixels.")
  }
  
  new.sizeX <- length(unique(img@data[ii,1]))
  new.sizeY <- length(unique(img@data[ii,2]))
  
  return( new( "FNTDimage",
               data = matrix( data = c( img@data[ii,1],
                                        img@data[ii,2],
                                        img@data[ii,3],
                                        img@data[ii,4]),
                              ncol = 4, 
                              nrow = sum(ii),
                              dimnames = list( NULL,
                                               c("x", "y", "z", "value"))),
               sizeX               = new.sizeX,
               sizeY               = new.sizeY,
               sizeZ               = img@sizeZ,
			   rescans             = img@rescans,
               dwell.time.us       = img@dwell.time.us,
               laser.power.percent = img@laser.power.percent,
               pinhole.diameter.um = img@pinhole.diameter.um,
               lens.type           = img@lens.type,
               tracks              = img@tracks,
               mask                = new("FNTDmask", mask = rep(1,new.sizeX*new.sizeY*img@sizeZ))))
}
