matchLoop <- function(set1, set2, critical.distance){
  
  max.iter <- 10
  for(i in 1:max.iter){
    # i <- 1
    if(i == 1){
      res      <- list(matched.to = new( "FNTDtracks",
                                         track.no   = rep(0, length(set1)),
                                         x          = rep(0, length(set1)),
                                         y          = rep(0, length(set1))))
    }
    
    res <- search.matches(set1              = set1,
                          set2              = set2,
                          matched.to        = res$matched.to,
                          critical.distance = critical.distance)

    cat("Iteration", i,
        "sought matches, changed:", res$n.changed,
        "\n")
    
    n.changed.match    <- res$n.changed
    res <- find.duplicates(set1           = set1,
                           matched.to     = res$matched.to) 
    
    cat("Iteration", i,
        "sought duplicated, changed:", res$n.changed,
        "\n")
    
    if(n.changed.match == 0 & res$n.changed == 0){
      break
    }
  }

  return(res$matched.to)
}


matchTrackPositions <- function(positions1, positions2, critical.distance, exclude.edge.effects = TRUE){

  #########################
  # 1. Matching-Durchlauf
  ##########################

  # Beruecksichtigung von unerwue?nschten Randeffekten. 
  # Tracks, die naeher als der kritische Radius am Rand liegen, werden 
  # in der Match-Routine nicht beruecksichtigt, da hier das Risiko, 
  # dass die Matching-Partner au?erhalb des Randes liegen relativ gro? ist. 
  # Schlie?t man diese Tracks nicht aus, bekommt man eine falsche Statistik, 
  # d.h. zu viele "unmatched" im Randgebiet.
  # Fuer manche Operationen moechte man dies nicht tun, dann flag auf FALSE
  # TODO: Durch Operatoren ersetzen
  
  if(exclude.edge.effects){
  kk <- (positions1@x >= min(positions1@x) + critical.distance) &      
        (positions1@x <= max(positions1@x) - critical.distance) &
        (positions1@y >= min(positions1@y) + critical.distance) &      
        (positions1@y <= max(positions1@y) - critical.distance)
  }else{
  kk <- rep(TRUE, length(positions1@x))
  }
  
  set1 <- new( "FNTDtracks",
               track.no = positions1@track.no[kk],
               x        = positions1@x[kk],
               y        = positions1@y[kk])
  
  matched.to   <- matchLoop( set1 = set1, 
                             set2 = positions2, 
                             critical.distance = critical.distance)
  
  df.1      <- data.frame( pos1.track.no = set1@track.no,
                           pos1.x        = set1@x,
                           pos1.y        = set1@y,
                           pos2.track.no = matched.to@track.no,
                           pos2.x        = matched.to@x,
                           pos2.y        = matched.to@y)
  
  #########################
  # 2. Matching-Durchlauf
  ##########################
  
  # Jetzt noch mal die unmatched CR39 der accepted area in die search.matches 
  # Routine stecken und gucken, ob noch Matches mit den FNTD
  # tracks des Randbereichs dabei sind
  
  # Ber?cksichtigung von unerw?nschten Randeffekten. Tracks, die n?her als der kritische Radius am Rand liegen, werden 
  # in der Match-Routine nicht ber?cksichtigt, da hier das Risiko, dass die Matching-Partner au?erhalb des Randes liegen relativ gro? ist. 
  # Schlie?t man diese Tracks nicht aus, bekommt man eine falsche Statistik, d.h. zu viele "unmatched" im Randgebiet.
  jj         <- positions2@track.no%in%matched.to@track.no   # all from positions2 that have been matched
  ll         <- (positions2@x[!jj] >= min(positions2@x) + critical.distance) &      
                (positions2@x[!jj] <= max(positions2@x) - critical.distance) &
                (positions2@y[!jj] >= min(positions2@y) + critical.distance) &      
                (positions2@y[!jj] <= max(positions2@y) - critical.distance)
  
  set1       <- new( "FNTDtracks",                             
                     track.no = positions2@track.no[!jj][ll],  
                     x        = positions2@x[!jj][ll],
                     y        = positions2@y[!jj][ll])

  set2       <- new( "FNTDtracks",                            # All die Tracks, die im Randgebiet
                     track.no = positions1@track.no[!kk],     # liegen und zun?chst nicht verwendet werden.
                     x        = positions1@x[!kk],
                     y        = positions1@y[!kk])
  
  matched.to <- matchLoop( set1 = set1, 
                           set2 = set2, 
                           critical.distance = critical.distance)
  
  df.2      <- data.frame( pos1.track.no = matched.to@track.no,
                           pos1.x        = matched.to@x,
                           pos1.y        = matched.to@y,
                           pos2.track.no = set1@track.no,
                           pos2.x        = set1@x,
                           pos2.y        = set1@y)
  
  df.match  <- rbind(df.1, df.2)
  return( list( df.match       = df.match,
                matched        = (df.match$pos1.track.no != 0) & (df.match$pos2.track.no != 0),
                unmatched.pos1 = (df.match$pos1.track.no != 0) & (df.match$pos2.track.no == 0),
                unmatched.pos2 = (df.match$pos1.track.no == 0) & (df.match$pos2.track.no != 0)))
}
