read.lsm.image <- function(filename, first.frame = NULL, last.frame = NULL){

    if(!file.exists(filename)){
	  stop("File not found.")
	}
	
	Reader        <- .jnew("fntd/lsm/FNTD_LSMFileReader")

	metadata      <- .jcall(Reader, "[S", "getMetaData", filename)
    sizes         <- .jcall(Reader, "[I", "getSize" , filename)

	if(is.null(first.frame) | is.null(last.frame)){
		data.read <- .jcall(Reader, "[T", "getFileData", filename)
	}else{
		if((first.frame <= 0)|(last.frame > sizes[3])|(first.frame > last.frame)){
			stop("Please give correct frames not exceeding stack size or in wrong order.")
		}
		
		data.read <- .jcall(Reader, "[T", "getFileData",  filename , as.integer(first.frame), as.integer(last.frame) )
		sizes[3]  <- last.frame - first.frame + 1
	}
	
    coords        <- generate.coordinates(x.size = sizes[1], y.size = sizes[2], z.size = sizes[3])
	
	# return rotated image to compensate lsm.reader error
	return( rotate.image( new( "FNTDimage",
								 data = matrix( data = c( coords$x,
														 coords$y,
														 coords$z,
														 data.read ),
												ncol = 4, 
												nrow = sizes[1] * sizes[2] * sizes[3],
												dimnames = list( NULL,
															 c("x", "y", "z", "value"))),
								 sizeX = sizes[1],
								 sizeY = sizes[2],
								 sizeZ = sizes[3],
								 rescans = as.numeric(metadata[3]),
								 dwell.time.us = as.numeric(metadata[1]),
								 laser.power.percent = as.numeric(metadata[2]),
								 pinhole.diameter.um = as.numeric(metadata[4]),
								 lens.type = metadata[5],
								 tracks   = new("FNTDtracks"),
								 mask     = new("FNTDmask", mask = rep(1, sizes[1] * sizes[2] * sizes[3]))), 
							degrees = 90) )
}


