read.text.image <-
function(filename, filename.detected.tracks = NULL, 
         x.min = NULL, x.max = NULL, y.min = NULL, y.max = NULL, 
		 rescans = 0, dwell.time.us = 0, laser.power.percent = 0){
  # read only part of the image?
  if( (!is.null(x.min)) | (!is.null(x.max)) | (!is.null(y.min)) | (!is.null(y.max))){
	  file.con <- file(filename)
	  
	  n.rows <- length(readLines( con = file.con)) 
	  n.cols <- length(gregexpr("\t", readLines( con = file.con, n = 1), fixed = TRUE)[[1]])+1
      
	  swap.min <- x.min
	  swap.max <- x.max
	  x.min    <- y.min
	  x.max    <- y.max
	  y.min    <- swap.min
	  y.max    <- swap.max
	  
	  if( (x.min <= x.max) & (y.min <= y.max) &
		  (x.min >= 0)     & (y.min >= 0)     &
		  (x.max < n.cols) & (y.max < n.rows) ){
	  
      x.size <- x.max - x.min + 1
      y.size <- y.max - y.min + 1
	  what.list <- c(rep(list(NULL), y.min), rep(list(0L), y.size), rep(list(NULL), n.rows - y.max - 1)) 
	  tmp <- unlist(scan( file   = file.con,
				          what   = what.list, 
				          sep    = "\t",
				          skip   = x.min,
				          nlines = x.size))
	  m   <- matrix(data = tmp, ncol = y.size, nrow = x.size, byrow = FALSE)
	  img           <- new( "FNTDimage",
							data = matrix( data = NA,
										   ncol = 4, 
										   nrow = x.size * y.size,
										   dimnames = list( NULL,
															c("x", "y", "z", "value"))),
							sizeX = x.size,
							sizeY = y.size,
							sizeZ = 1)
	  coords        <- generate.coordinates(x.size = x.size, y.size = y.size, z.size = 1, x.min = x.min, y.min = y.min)
	  img@data[,1]  <- coords$x
	  img@data[,2]  <- coords$y
	  img@data[,3]  <- coords$z
	  img@data[,4]  <- unlist(tmp)
 	  
	  close(file.con)
    }else{
	    stop("Invalid crop coordinates given (do not match image size).")
    }
  }else{
	   tmp           <- read.table( filename,
		    					    header    = FALSE,
		 					        dec       = ".",
							        sep       = "\t")
	   x.size        <- nrow(tmp)
	   y.size        <- ncol(tmp)
	   img           <- new( "FNTDimage",
							data = matrix( data = NA,
										   ncol = 4, 
										   nrow = x.size * y.size,
										   dimnames = list( NULL,
															c("x", "y", "z", "value"))),
							sizeX = x.size,
							sizeY = y.size,
							sizeZ = 1)
	  coords        <- generate.coordinates(x.size = x.size, y.size = y.size, z.size = 1)
	  img@data[,1]  <- coords$x
	  img@data[,2]  <- coords$y
	  img@data[,3]  <- coords$z
	  img@data[,4]  <- unlist(tmp)
 }
  
  if(!is.null(filename.detected.tracks)){
      img@tracks <- read.detected.tracks( filename = filename.detected.tracks)
  }
  
  # Add default mask 
  img@mask      <- new("FNTDmask", mask = rep(1, x.size * y.size))
  
  # Add readout parameters
  img@rescans         <- rescans
  img@dwell.time.us       <- dwell.time.us
  img@laser.power.percent <- laser.power.percent
  return (img)
}
