track.moments <- function(img, what = c("basic", "first", "second")[1]){

	if(class(img)!="FNTDimage"){
		stop("img has to be of class \"FNTDimage\".")
	}

    if((length(img@mask) == 0)|(length(img@tracks)==0)){
		stop("img has to have an appropriate mask and track information.")
    }

	df <- NULL
	if("basic"%in%what){
		track.no             <- as.vector( tapply( 	img@mask@mask, 
													img@mask@mask, 
													head, 1))
		n.pixel				 <- as.vector( tapply( img@data[,4],  
										  img@mask@mask, 
										  length))
		track.max            <- as.vector( tapply( img@data[,4],  
												   img@mask@mask, 
												   max))
		
		track.min            <- as.vector( tapply( img@data[,4],  
												   img@mask@mask, 
												   min))
		# Zero-th moment: sum / mean
		track.sum            <- as.vector( tapply( img@data[,4],  
												   img@mask@mask, 
												   sum))
		track.mean           <- as.vector( tapply( img@data[,4],  
												   img@mask@mask, 
												   mean))
		df <- data.frame(	track.no             = track.no,
							n.pixel              = n.pixel,
							track.max            = track.max,
							track.min            = track.min,
							track.sum            = track.sum,
							track.mean           = track.mean)
	}
	
	if("first"%in%what){
		# First moment: center of mass, might be different from output of tracker routine
		# as radius differs
		track.center.x        <- numeric(length(track.no))
		track.center.y        <- numeric(length(track.no))
		for(i in track.no){
		  # i <- track.no[1]
		  ii                      <- img@mask@mask == i  
		  idx                     <- which(track.no == i)
		  track.center.x[idx]     <- sum( img@data[ii, 1] * img@data[ii, 4]) / sum( img@data[ii, 4] )
		  track.center.y[idx]     <- sum( img@data[ii, 2] * img@data[ii, 4]) / sum( img@data[ii, 4] )
		}
		tmp < - data.frame(	track.center.x       = track.center.x,
							track.center.y       = track.center.y)
		if(is.null(df)){
			df <- tmp
		}else{
			df <- cbind(df, tmp)
		}
	}
	
	if("second"%in%what){
		# Second moments: with respect to center of mass comuted above (not particle tracker!)
		track.var.x           <- numeric(length(track.no))
		track.var.y           <- numeric(length(track.no))
		track.var.xy          <- numeric(length(track.no))
		for(i in track.no){
		  # i <- track.no[1]
		  ii                      <- img@mask@mask == i  
		  idx                     <- which(track.no == i)
		  track.var.x[idx]        <- sum( (img@data[ii, 1] - track.center.x[idx])^2 * img@data[ii, 4]) / sum( img@data[ii, 4] )
		  track.var.y[idx]        <- sum( (img@data[ii, 2] - track.center.y[idx])^2 * img@data[ii, 4]) / sum( img@data[ii, 4] )
		  track.var.xy[idx]       <- sum( img@data[ii, 1] * img@data[ii, 2] * img@data[ii, 4]) / sum( img@data[ii, 4] ) - track.center.x[idx]*track.center.y[idx]
	#	  track.var.x[idx]        <- sum( img@data[ii, 1]^2 * img@data[ii, 4]) / sum( img@data[ii, 4] ) - track.center.x[idx]^2
	#	  track.var.y[idx]        <- sum( img@data[ii, 2]^2 * img@data[ii, 4]) / sum( img@data[ii, 4] ) - track.center.y[idx]^2
	#	  track.var.xy[idx]       <- sum( img@data[ii, 1] * img@data[ii, 2] * img@data[ii, 4]) / sum( img@data[ii, 4] ) - track.center.x[idx]*track.center.y[idx]
		}
		tmp < - data.frame(	track.var.x          = track.var.x,
							track.var.y          = track.var.y,
							track.var.xy         = track.var.xy)
		if(is.null(df)){
			df <- tmp
		}else{
			df <- cbind(df, tmp)
		}
	}
	
	return(df)
}