\name{CoordinateTransformations}
\alias{transformImage}
\alias{findTransformation}
\alias{transformTrackPositions}

\title{Transform coordinate systems}

\description{Routines to find transformation matrix and transforms images or positions from one coordinate system to another.}

\usage{findTransformation( markers1, markers2)
transformTrackPositions( trackPositions, markers1, markers2)
transformImage(img, markers1, markers2, crop = FALSE, fast = FALSE)}


\arguments{
  \item{markers1}{ TARGET coordinate system given as FNTDmarker object (three elements representing three xy-coordinates, the first being the origin)}
  \item{markers2}{ ORIGINAL coordinate system given as FNTDmarker object (three elements representing three xy-coordinates, the first being the origin)}
  \item{trackPositions}{ track positions to rotate, object of class "FNTDtracks".}
  \item{img}{ image to rotate, object of class "FNTDimage".}
  \item{crop}{ If true, rotate image will be cropped to the largest contained square (not yet implemented).}
  \item{fast}{ If false, a simple but slow nearest neighbor algorithm will be used for image interpolation, if true, a fast, but less reliable is applied.}
}

\details{
Will create empty areas around image that have to be cropped manually by using \code{crop}}

\value{
For \code{findTransformation}, 2x2 transformation matrix. 
For \code{transformTrackPositions}, the transformed track positions as object of class "FNTDtracks". 
For \code{transformImage}, the transformed image as object of class "FNTDimage".}


\author{
Steffen Greilich
}

\examples{
# Rotate an image img according to transformation indicated by markers markers.FNTD and markers.CR39
data(img, markers.FNTD, markers.CR39)
plot(transformImage( img        = img,
                     markers1   = markers.FNTD,
                     markers2   = markers.CR39))

# Print out transformation matrix used hereby
findTransformation( markers1    = markers.FNTD, 
                    markers2    = markers.CR39)
}
