\name{FNTDimage-class}
\docType{class}
\alias{FNTDimage-class}
\alias{length,FNTDimage-method}
\alias{plot,FNTDimage-method}
\alias{[,FNTDimage-method}
\alias{*,FNTDimage,numeric-method}
\alias{+,FNTDimage,numeric-method}
\alias{/,FNTDimage,numeric-method}
\alias{+,FNTDimage,FNTDimage-method}
\alias{-,FNTDimage,FNTDimage-method}
\alias{*,FNTDimage,FNTDimage-method}
\alias{/,FNTDimage,FNTDimage-method}
\alias{length,FNTDimage-method}
\alias{mean,FNTDimage-method}
\alias{sd,FNTDimage-method}
\alias{sum,FNTDimage-method}
\alias{min,FNTDimage-method}
\alias{max,FNTDimage-method}
\alias{summary,FNTDimage-method}

\title{Class \code{"FNTDimage"}}

\description{This is the main class to hold FNTD images and related data. }

\section{Structure of the class}{
FNTD images contain:
\tabular{rcl}{
data    \tab A four column matrix with the actual image data. Columns: x, y, z, and value. Linear array with explicit coordinate values (zero based) \tab Access via \code{tracks(img)} \cr
tracks  \tab An object of FNTDtracks class (see there) \tab Access via \code{tracks(img)} \cr
mask    \tab An object of FNTDmask class (see there) \tab Access via \code{mask(img)} \cr
various \tab Various properties, such as size numeric \tab Access via \code{properties(img)}}
}

\section{Methods}{
  \itemize{
    \item{create}{\code{new("FNTDimage", ...)}: Creates a new FNTDimage object. Default value are: ..., specific own value via: ...}
    \item{[]}{\code{img[idx]}: Returns values for indices idx.}
    \item{length}{\code{length(img)}: Returns the number of data elements of the the image, i.e. pixel count}
	\item{plot}{\code{plot(img, tracks = TRUE, mask = TRUE, frequency = FALSE, double.labels = FALSE)}: Plot the image with correct orientation etc., track positions and masking if provided. If frequency is TRUE, the counting frequency will be plotted; if double.labels is TRUE, both frequency and absolute counts will be given on the colorkey.}
	\item{sum}{\code{sum(img)}: Return sum of pixel values; for pixels in mask only, if a mask is provided.}
	\item{mean}{\code{mean(img)}: Return mean of pixel values; for pixels in mask only, if a mask is provided.}
	\item{sd}{\code{sd(img)}: Return standard deviation of pixel values; for pixels in mask only, if a mask is provided.}
	\item{min}{\code{min(img)}: Return minimal pixel value; for pixels in mask only, if a mask is provided.}
	\item{max}{\code{max(img)}: Return maximal pixel value; for pixels in mask only, if a mask is provided.}
	\item{+}{\code{img + T)}: With T being a two-element numeric vector, image positions of img are translated by dx = T[1], dy = T[2].}
	\item{*}{\code{img * M)}: With M being a single numeric value, image POSITIONS of img are mulitplied by M, e.g. M = -1 for a reflection about the origin.}
	\item{/}{\code{img1 / M)}: With M being a single numeric value, image VALUES of img are divided by M, be careful not to confuse this function with \code{img * M).}}
	\item{+,-,*,/}{\code{img1 + img2}, \code{img1 - img2},\code{img1 * img2},\code{img1 / img2}: Performs arithmetric operation pixel-wise with two images, which have to be of same size.}
	}
}

\references{
NIMB paper
}

\details{
Within the plotting command, custom arguments \code{color.key} and \code{at} can be given (see \code{\link{levelplot}} for details). This allows to tweak the colors assigned to intensity values (default 100 grey values from black to white) as well as the range (default: from minimum to maximum intensity in 100 steps, i.e. vector of 101 values). Using approriate spacing in \code{at}, also logarithmic or similar display can be used.
}

\author{
Steffen Greilich
}

\examples{\dontrun{
\############################################################################################
# Read text image, add information on detected tracks (both in ImageJ format) during read
img  <- read.text.image( filename                 = system.file( 'extdata', 'sg35041_s03.txt',                    package = 'FNTD' ),
                         filename.detected.tracks = system.file( 'extdata', 'sg35041_s03_detected_particles.dat', package = 'FNTD' ))

# Plot image and track information
plot(img)
# Plot image with focus on background intensity and a coarser step size of the color key
plot(img, col.regions = heat.colors(10), at = seq(from = 100, to = 300, length.out = 11))

############################################################################################
# Apply various masks and plot result

# tracks
track.mask <- create.mask( img, type = "tracks", radius = 5, critical.distance = 11)
img        <- apply.mask( img, track.mask)
plot(img, tracks = FALSE)

# background
plot( apply.mask( img, !track.mask), tracks = FALSE)
}}

\keyword{classes}
