\name{create.image}
\alias{create.image}
\title{create.image}

\description{
Creates a custom FNTD image. Gaussian or Poissonian background noise can be added to the background signal of the image. Track spots can be added randomly or by supplying a data.frame with their positions. The particle track spots are approximated by a 3D-Gaussian. It is possible to return the background image seperately.
}

\usage{create.image(size=512,
                    bkg.mean=100, bkg.sd = 8,
                    no.tracks = 0, track.core.mean = 200, track.core.sd = 20, track.width.mean = 1.2, track.width.sd = 0,
                    track.list = NULL, shot.noise = TRUE, type = 1)}

\arguments{
  \item{size}{Size of the square image in pixel.}
  \item{bkg.mean}{Mean value of the background noise.}
  \item{bkg.sd}{Variation of the background noise. If == 0, no noise, if < 0 Poissonian noise (where the standard deviation equals the square root of the expectation value \code{bkg.mean}), if > 0 Gaussian noise with provided value will be added.}  
  \item{no.tracks}{Number of tracks to be added. Is ignored if a \code{track.list} is given by the user.}
  \item{track.core.mean}{ Mean value of the track peak.}
  \item{track.core.sd}{Standard deviation of \code{track.core.mean}.}
  \item{track.width.mean}{Radius of the particle track signal. The radius corresponds to the the sigma value of a Gaussian.}
  \item{track.width.sd}{Standard deviation of \code{track.width.mean}.}
  \item{track.list}{A data.frame with two columns: "x" and "y" indicating the track spots' positions in X- and Y-direction. If given, \code{no.tracks} is ignored.}
  \item{shot.noise}{Logical flag to indicate to indicate if overall shot noise is to be added to the image intensity values by a Poissonian error.}
  \item{type}{Setting this to "1" will return the created image only. Setting this to "2" (or any other value) will return a list containing the created image as first and the corresponding background image as second entry.}
}

\seealso{Use save.text.image to save image in ImageJ's text format for further use.}

\value{ For \code{type}=1, the result an object of class FNTDimage. Otherwise the result will be a list containing two objects of class FNTDimage.}

\author{Steffen Greilich, Sebastian Hoof}

\examples{library(FNTD)
#Creating an FNTD-image with 100 random track spots
img1 <- create.image(no.tracks=100)
#Creating an FNTD-image with tracks of 4 pre-defined positions
img2 <- create.image(track.list = data.frame( x=100*1:4, y=100*1:4 ))
plot(img1, tracks=TRUE)
plot(img2, tracks=TRUE)

#Creating an FNTD-image with 20 random tracks and no background or other noise at all. Mean background signal is 50, track signal is 100 and track radius is 2 pixel. The latter corresponds to a FWHM of about 4.7 pixel.
img3 <- create.image(bkg.mean=50, bkg.sd = 0, no.tracks = 20, track.core.mean = 100, track.core.sd = 0, track.width.mean = 2, shot.noise = FALSE)
plot(img3)
}
