\name{create.mask}
\alias{create.mask}
\title{create.mask}

\description{
Creates a mask (object FNTDmask) on an image. This object is an integer vector with same length than the image (img@data[,4]). 
}

\usage{create.mask(img, type, ...)}

\arguments{
  \item{img}{ FNTDimage object}
  \item{type}{ type of mask to create, see details}
  \item{...}{ arguments for mask creation, depending on type, see details}
}

\details{
Masks can be of several types:
\itemize{
	\item{tracks}{   A mask around all given track positions in the image (has to be provided in img@tracks). Parameters \code{radius, critical.distance}. If the latter is not given, 2*radius + 1 is used. Values of mask are then 0 (background) and >1 (track number). Use !create.mask to get the background without the tracks.}
	\item{crop}{   Cuts out a rectangular area, Parameters \code{x.min, x.max, y.min, y.max}. Values of mask are 0 or 1.}
	\item{intensity}{   Cuts out intensity range. Parameters \code{intensity.min, intensity.max} lower and upper limit for intensity, given as relative intensity (between 0 and 1, if parameter \code{relative} is TRUE (default) or absolute intensity. NB: left-handed binning, i.e. intensity >= lower value AND intensity < upper value). If \code{use.mask} is TRUE, only masked parts of the image will be used, otherwise entire image.}
	}
}

\value{An object of class FNTDmask which can be attched to an image by \code{apply.mask}}

\author{Steffen Greilich}

\examples{
\dontrun{
require(FNTD)

filename   <- system.file("extdata", "sg35041_s03.lsm", package = "FNTD")
img        <- read.lsm.image(filename)                            
img@tracks <- find.tracks(img, radius = 3, cutoff = 0.1, percentile = 0.6)

# Cut out a radius of 10 pixel around each track, NOT considering overlap
plot(apply.mask(img, create.mask(img, 
                                 type = "tracks",
                                 radius = 10,
                                 critical.distance = 0)),
     mask   = TRUE)

# Cut out a radius of 10 pixel and consider a safe margin (21 = 2*10+1 pixels)
plot(apply.mask(img, create.mask(img, 
                                 type = "tracks",
                                 radius = 10,
                                 critical.distance = 21)),
     mask   = TRUE)
}
}
