\name{track.information}
\alias{get.summary}
\alias{get.gauss.fits}
\alias{track.moments}
\title{Get information on tracks}

\description{
Functions to return quantities for any masked parts of an image - especially useful for track masks, but works in general for all masks. \code{track.moments} returns descriptive quantities (e.g. moments), while \code{get.gauss.fits} will return parameters for fitted bell-shaped curves. \code{get.summary} is an obsolete precursor of \code{track.moments} and will be depreaciated soon.
}
\usage{
track.moments(img, what)
get.summary(img)
get.gauss.fits(img)
}

\arguments{
  \item{img}{ FNTDimage, INCLUDING mask and track information}
  \item{what}{ vector any combination of the elements "basic", "first", and "second", determining which quantities to compute. basic = mean, max, min, ... (fast), first = first moments (slower), second = second moments, i.e. variance in x, y and covariance (sloooooow).
}}

\details{
Please take care of two things: \cr
a) make sure that an appropriate critical.distance parameter is applied when creating the track mask. Overlapping tracks can produce funny results here.\cr
b) the mask creation does not check if a track is partly outside the image. You can intercept those tracks by applying a condition such as \code{n.pixel >= 0.95 * pi * radius^2)}, checking if 95\% (or whatever) pixel are within the image and set your own threshold. Agains tracks that are not fully within the image may produce funny numbers.\cr\cr
Also, the center of mass (and thus the second moments) may differ from the output of the particle track routine (\code{find.tracks}), as a different radius may be applied and more/less pixels are taken into consideration. In the example below, the radius is 10 for \code{track.moments} and only 3 for \code{find.tracks}.
}

\value{
For \code{track.moments} a data frame containing\cr
\code{track.no} - number of track (viz. mask region)\cr
\code{n.pixel} - number of pixels considered for track\cr
\code{track.max} - maximum value within track\cr
\code{track.sum} - sum of values within track\cr
\code{track.mean} - average value within track\cr
\code{track.center.x, track.center.y} - center of mass for tracks (intensity weighted)\cr
\code{track.var.x, track.var.y, track.var.xy} - second intensity-weighted moments (variances - not square-rooted!, covariance) for tracks\cr
\cr
\cr
For \code{get.gauss.fits} a data frame 
TODO.
\cr
Please note: track no 0 refers to everything that is NOT masked, i.e. mask value 0, e.g. the background.
}

\author{
Steffen Greilich
}
\examples{
#filename          <- system.file("extdata", "sg35041_s03.lsm", package = "FNTD")
#img               <- read.lsm.image(filename)                            
#img@tracks        <- find.tracks( img, 
#                                  radius = 3, cutoff = 0.1, percentile = 0.6)

#radius            <- 10
#critical.distance <- 21
#img <- apply.mask(img, create.mask(img, 
#                                 type              = "tracks",
#                                 radius            = radius,
#                                 critical.distance = critical.distance))
#df <- track.moments(img)
#df

# How many tracks are not fully within image?
#sum(df$n.pixel < 0.95 * pi * radius^2)
}

